OptionOptions = class(Turbine.UI.Window)
function OptionOptions:Constructor()
    local width = 1100
    local height = 570
    local x = 500 - 125
    local y = 200
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Window.Constructor(self);
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetPosition(x,y);
    self:SetWantsKeyEvents(true)
    self:SetText("Option Options")
    self:SetVisible(true)
    self:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

    self.reloadStatus = false

    self.VisibleChanged = function()
        if self.reloadStatus and Bag then 
            Bag:BuildBag() 
        end
    end

    XIcon = "PrimePlugins/RaidTools/Images/X.tga"
    MoveIcon = "PrimePlugins/RaidTools/Images/Move.tga"

    -- Move logic
    self.Moving = false

    self.MouseDown = function(sender,args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end
  
    self.MouseUp = function()
        self.Moving = false
    end
  
    self.MouseMove = function(sender,args)
        if self.Moving then
            local newLeft = self:GetLeft() - (self.MoveX - args.X)
            local newTop = self:GetTop() - (self.MoveY - args.Y)
            self:SetPosition(newLeft,newTop)
        end
    end

    -- Options Container
    local ColorBackground = Turbine.UI.Control()
    ColorBackground:SetParent(self)
    ColorBackground:SetPosition(10, 10)
    ColorBackground:SetSize(1080, 550)
    ColorBackground:SetMouseVisible(false)
    ColorBackground:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    local EdgeColour = Turbine.UI.Control()
    EdgeColour:SetParent(self)
    EdgeColour:SetPosition(550 - 50, 15)
    EdgeColour:SetSize(100, 30)
    EdgeColour:SetMouseVisible(false)
    EdgeColour:SetBackColor(Turbine.UI.Color(0, 0, 0))

    local MoveButton = Turbine.UI.Label()
    MoveButton:SetParent(self)
    MoveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    MoveButton:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    MoveButton:SetFontStyle(Turbine.UI.FontStyle.Outline)
    MoveButton:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    MoveButton:SetForeColor(Turbine.UI.Color(1, 1, 1))
    MoveButton:SetText("Options")
    MoveButton:SetPosition(550 - 50 + 1, 15 + 1)
    MoveButton:SetSize(100 - 2, 30 - 2)
    MoveButton:SetMouseVisible(false)
    MoveButton:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

    -- Reload button
    local ReloadBackground = Turbine.UI.Control()
    ReloadBackground:SetParent(self)
    ReloadBackground:SetPosition(10, 10)
    ReloadBackground:SetSize(35, 35)
    ReloadBackground:SetMouseVisible(false)
    ReloadBackground:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

    local Image = Turbine.UI.Label()
    Image:SetParent(ReloadBackground)
    Image:SetPosition(0, 0)
    Image:SetSize(25, 25)
    Image:SetBackColor(Turbine.UI.Color(0, 102/255, 0))
    Image:SetWantsKeyEvents(false)
    Image:SetMouseVisible(false)

    local Button = Turbine.UI.Label()
    Button:SetParent(ReloadBackground)
    Button:SetPosition(0, 0)
    Button:SetSize(25, 25)
    Button:SetBackground(MoveIcon)
    Button:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    Button:SetBlendMode(Turbine.UI.BlendMode.Overlay)
    Button:SetWantsKeyEvents(false)
    Button:SetMouseVisible(true)

    Button.MouseClick = function()
        if Bag then 
            Bag:BuildBag() 
            self:SetVisible(false)
        end
    end

    -- Close Button
    local CloseBackground = Turbine.UI.Control()
    CloseBackground:SetParent(self)
    CloseBackground:SetPosition(1100 - 45, 10)
    CloseBackground:SetSize(35, 35)
    CloseBackground:SetMouseVisible(false)
    CloseBackground:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

    local Image = Turbine.UI.Label()
    Image:SetParent(CloseBackground)
    Image:SetPosition(10, 0)
    Image:SetSize(25, 25)
    Image:SetBackColor(Turbine.UI.Color(153/255, 0, 0))
    Image:SetWantsKeyEvents(false)
    Image:SetMouseVisible(false)

    local Button = Turbine.UI.Label()
    Button:SetParent(CloseBackground)
    Button:SetPosition(10, 0)
    Button:SetSize(25, 25)
    Button:SetBackground(XIcon)
    Button:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    Button:SetBlendMode(Turbine.UI.BlendMode.Overlay)
    Button:SetWantsKeyEvents(false)
    Button:SetMouseVisible(true)

    Button.MouseClick = function()
        self:SetVisible(false)
    end

    local OptionsContainer = Turbine.UI.Control()
    OptionsContainer:SetParent(self)
    OptionsContainer:SetPosition(300, 50)
    OptionsContainer:SetSize(750, 480)
    OptionsContainer:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

    OptionsDic = {} 

    local VitalOptions = VitalOptionsWindow(OptionsContainer)
    OptionsDic["Setup"] = VitalOptions

    ButtonsDic = {}

    SortedNames = {}
    for key, value in pairs(OptionsDic) do table.insert( SortedNames, key ) end
    table.sort( SortedNames )

    local index = 0
    for key, value in pairs(SortedNames) do
        local EdgeColour = Turbine.UI.Control()
        EdgeColour:SetParent(self)
        EdgeColour:SetPosition(50, 50 + index * 32)
        EdgeColour:SetSize(250, 32)
        EdgeColour:SetMouseVisible(true)
        EdgeColour:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

        local MoveButton = Turbine.UI.Label()
        MoveButton:SetParent(self)
        MoveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
        MoveButton:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
        MoveButton:SetFontStyle(Turbine.UI.FontStyle.Outline)
        MoveButton:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
        MoveButton:SetForeColor(Turbine.UI.Color(1, 1, 1))
        MoveButton:SetText(value)
        MoveButton:SetPosition(50 + 1, 50 + index * 32 + 1)
        MoveButton:SetSize(250 - 2, 32 - 2)
        MoveButton:SetMouseVisible(true)
        MoveButton:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))
        MoveButton.MouseClick = function()
            for button, _ in pairs(ButtonsDic) do
                button:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))
            end
            MoveButton:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

            for _, optionPage in pairs(OptionsDic) do
                optionPage:SetVisible(false)
            end
            OptionsDic[value]:SetVisible(true)
        end

        ButtonsDic[MoveButton] = true

        index = index + 1
    end
end

function OptionOptions:ChangesMade()
    self.reloadStatus = true
end

VitalOptionsWindow = class(Turbine.UI.Control)
function VitalOptionsWindow:Constructor(parent)
    local width = 750
    local height = 500
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Control.Constructor(self);
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetParent(parent)
    self:SetPosition(0,0);
    self:SetWantsKeyEvents(true)
    self:SetVisible(false)

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(self)
    SingleBackground:SetPosition(10 - 2, 40 - 2)
    SingleBackground:SetSize(720 + 4, 400 + 4)
    SingleBackground:SetBackColor(Turbine.UI.Color(0, 0, 0))

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(self)
    SingleBackground:SetPosition(10, 40)
    SingleBackground:SetSize(720, 400)
    SingleBackground:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    local container = Turbine.UI.ListBox()
    container:SetParent(SingleBackground)
    container:SetSize(700, 280)
    container:SetPosition(10, 30)
    --container:SetBackColor(GoldColor)
    container:SetVisible(true)
    container:SetWantsKeyEvents(true)
    container:SetMouseVisible(true)
    --container:SetBackColor(GetRandomColour())

    local containerScrollbar = Turbine.UI.Lotro.ScrollBar();
    containerScrollbar:SetOrientation( Turbine.UI.Orientation.Vertical );
    containerScrollbar:SetParent( SingleBackground );
    containerScrollbar:SetPosition( 780 - 10, 30 );
    containerScrollbar:SetSize( 10, 280 );
    container:SetVerticalScrollBar( containerScrollbar );

    for key, value in spairs(Settings.MiscAmount, function(t, a, b) return a < b end) do
        if key ~= "Miscellaneous" then
            container:AddItem(self:createCategoryBar(key))
        end
    end

    local walletContainter = Turbine.UI.ListBox()
    walletContainter:SetParent(SingleBackground)
    walletContainter:SetSize(700, 280)
    walletContainter:SetPosition(10, 30)
    --walletContainter:SetBackColor(GoldColor)
    walletContainter:SetVisible(false)
    walletContainter:SetWantsKeyEvents(true)
    walletContainter:SetMouseVisible(true)
    --walletContainter:SetBackColor(GetRandomColour())

    local walletContainterScrollbar = Turbine.UI.Lotro.ScrollBar();
    walletContainterScrollbar:SetOrientation( Turbine.UI.Orientation.Vertical );
    walletContainterScrollbar:SetParent( SingleBackground );
    walletContainterScrollbar:SetPosition( 780 - 10, 30 );
    walletContainterScrollbar:SetSize( 10, 280 );
    walletContainter:SetVerticalScrollBar( walletContainterScrollbar );

    for key, value in spairs(Settings.TrackedCurrencies, function(t, a, b) return a < b end) do
        walletContainter:AddItem(self:createWalletBar(key))
    end

    self.skillButton = Turbine.UI.Label()
    self.skillButton:SetParent(SingleBackground)
    self.skillButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.skillButton:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.skillButton:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.skillButton:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    self.skillButton:SetForeColor(Turbine.UI.Color(1, 1, 1))
    self.skillButton:SetText("Categories")
    self.skillButton:SetPosition(10, 0)
    self.skillButton:SetSize(250, 30)
    self.skillButton:SetMouseVisible(true)
    self.skillButton:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

    self.buffButton = Turbine.UI.Label()
    self.buffButton:SetParent(SingleBackground)
    self.buffButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.buffButton:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.buffButton:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.buffButton:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    self.buffButton:SetForeColor(Turbine.UI.Color(1, 1, 1))
    self.buffButton:SetText("Currencies")
    self.buffButton:SetPosition(260, 0)
    self.buffButton:SetSize(250, 30)
    self.buffButton:SetMouseVisible(true)
    self.buffButton:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))

    self.buffButton.MouseClick = function()
        self.skillButton:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))
        self.buffButton:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))
        container:SetVisible(false)
        walletContainter:SetVisible(true)
    end
    
    self.skillButton.MouseClick = function()
        self.buffButton:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))
        self.skillButton:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))
        container:SetVisible(true)
        walletContainter:SetVisible(false)
    end
end

function VitalOptionsWindow:createCategoryBar(name)
    local pluginBar = Turbine.UI.Control()
    pluginBar:SetSize(700, 40)
    pluginBar:SetBackColor(Turbine.UI.Color(0, 0, 0))

    local backgroundBar = Turbine.UI.Control()
    backgroundBar:SetSize(700 - 2, 38)
    backgroundBar:SetPosition(1, 1)
    backgroundBar:SetParent(pluginBar)
    backgroundBar:SetMouseVisible(false)
    backgroundBar:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))
    --pluginBar:SetBackColor(GetRandomColour())

    local nameLabel = Turbine.UI.Label()
    nameLabel:SetParent(pluginBar)
    nameLabel:SetSize(200, 20)
    nameLabel:SetPosition(10, 10)
    nameLabel:SetMouseVisible(false)
    nameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.LeftCenter)
    nameLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    nameLabel:SetMultiline(false)
    nameLabel:SetText(name)

    local ScaleLabel = Turbine.UI.Label()
    ScaleLabel:SetParent(pluginBar)
    ScaleLabel:SetSize(200, 30)
    ScaleLabel:SetTextAlignment(Turbine.UI.ContentAlignment.LeftCenter)
    ScaleLabel:SetPosition(230, 10)
    ScaleLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    ScaleLabel:SetText("Minimum item amount:") 

    local BarScaleBar = Turbine.UI.Lotro.TextBox()
    BarScaleBar:SetParent(pluginBar)
    BarScaleBar:SetSize(50, 30)
    BarScaleBar:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    BarScaleBar:SetPosition(440, 5)
    BarScaleBar:SetText(Settings.MiscAmount[name])

    BarScaleBar.TextChanged = function()
        if tonumber(BarScaleBar:GetText()) then
            Settings.MiscAmount[name] = tonumber(BarScaleBar:GetText())
            ChatOutput("Set Minimum item amount of category: " .. name .. " to " .. tonumber(BarScaleBar:GetText()))
        end
    end

    return pluginBar
end

function VitalOptionsWindow:createWalletBar(name)
    local pluginBar = Turbine.UI.Control()
    pluginBar:SetSize(700, 40)
    pluginBar:SetBackColor(Turbine.UI.Color(0, 0, 0))

    local backgroundBar = Turbine.UI.Control()
    backgroundBar:SetSize(700 - 2, 38)
    backgroundBar:SetPosition(1, 1)
    backgroundBar:SetParent(pluginBar)
    backgroundBar:SetMouseVisible(false)
    backgroundBar:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))
    --pluginBar:SetBackColor(GetRandomColour())

    local EffectBox = Turbine.UI.Lotro.CheckBox();
    EffectBox:SetParent(pluginBar)
    EffectBox:SetPosition(10, 10);
    EffectBox:SetSize(400,20);
    EffectBox:SetChecked(Settings.TrackedCurrencies[name])
    EffectBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    EffectBox:SetText(name);
    EffectBox:SetVisible(true);

    local ScaleLabel = Turbine.UI.Label()
    ScaleLabel:SetParent(pluginBar)
    ScaleLabel:SetSize(200, 30)
    ScaleLabel:SetTextAlignment(Turbine.UI.ContentAlignment.LeftCenter)
    ScaleLabel:SetPosition(360, 10)
    ScaleLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    ScaleLabel:SetText("Priority:") 

    local BarScaleBar = Turbine.UI.Lotro.TextBox()
    BarScaleBar:SetParent(pluginBar)
    BarScaleBar:SetSize(50, 30)
    BarScaleBar:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    BarScaleBar:SetPosition(440, 5)
    BarScaleBar:SetText(Settings.CurrencyOrder[name])

    EffectBox.CheckedChanged = function()
        Settings.TrackedCurrencies[name] = EffectBox:IsChecked()
    end

    BarScaleBar.TextChanged = function()
        if tonumber(BarScaleBar:GetText()) then
            Settings.CurrencyOrder[name] = tonumber(BarScaleBar:GetText())
            ChatOutput("Set Priority of currency: " .. name .. " to " .. tonumber(BarScaleBar:GetText()))
        end 
    end

    return pluginBar
end


plugin.GetOptionsPanel = function( self )

    options = Turbine.UI.Control()
    options:SetSize(500, 400)

    local MasterOptionsButton = Turbine.UI.Lotro.Button()
    MasterOptionsButton:SetParent(options)
    MasterOptionsButton:SetPosition(100, 100);
    MasterOptionsButton:SetSize(200,15);
    MasterOptionsButton:SetText("Open options panel");
    MasterOptionsButton:SetVisible(true);

    MasterOptionsButton.Click = function()
        if Options == nil then
            Options = OptionOptions()
        else
            Options:SetVisible(not(Options:IsVisible()))
        end
    end

    return options;
end