PrimeUI = class(Turbine.UI.Window)
function PrimeUI:Constructor(data)
    Turbine.UI.Window.Constructor(self)
    self.width = 500
    self.height = 800
    --self:SetPosition(500, 300)
    self:SetSize(self.width, self.height)
    self:SetPosition(Turbine.UI.Display.GetWidth() / 2 - 400, 20)
    self:SetBackColor(GetRandomColour())
    self:SetVisible(true)
    self:SetMouseVisible(false)

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.playerClass = self.player:GetClass()
    self.ClassAttr = self.player:GetClassAttributes()

    self.skillCallbacks = {}
    self.playerCallbacks = {}
    self.attrCallbacks = {}
    self.effectAddedCallbacks = {}
    self.effectRemovedCallbacks = {}
    self.playerChatCallback = {}
    self.enemyChatCallback = {}

    self.alignments = {
        ["BottomCenter"] = Turbine.UI.ContentAlignment.BottomCenter,
        ["BottomLeft"] = Turbine.UI.ContentAlignment.BottomLeft,
        ["BottomRight"]	= Turbine.UI.ContentAlignment.BottomRight,
        ["MiddleCenter"] = Turbine.UI.ContentAlignment.MiddleCenter,
        ["MiddleLeft"] = Turbine.UI.ContentAlignment.MiddleLeft,
        ["MiddleRight"] = Turbine.UI.ContentAlignment.MiddleRight,
        ["TopCenter"] = Turbine.UI.ContentAlignment.TopCenter,
        ["TopLeft"] = Turbine.UI.ContentAlignment.TopLeft,
        ["TopRight"] = Turbine.UI.ContentAlignment.TopRight,
    }

    self.data = data
-- [[

    --]]

    --[[
    self.data = {
        [1] = {
            ["Order"] = 1,
            ["Type"] = "Timer",
            ["Height"] = 64,
            ["Width"] = 800,
            ["Scale"] = 1,
            ["ShowText"] = true,
            ["OffsetAbove"] = 30,
            ["OffsetBelow"] = 0,
            ["BackgroundColour"] = nil,
            ["BoxOffset"] = 2,
            ["BorderSize"] = 0,
            ["Alignment"] = "Left",
            ["BorderColour"] = nil,--Turbine.UI.Color(0.08, 0.08, 0.08),
            ["BackgroundColour"] = nil,--Turbine.UI.Color(0.08, 0.58, 0.08),
            ["AlwaysVisible"] = false,
            ["Timers"] = {
                [1] = {
                    ["Icon"] = 1091838724,
                    ["Effects"] = {
                        --["Find the Path"] = 25,
                    },
                    ["PlayerHits"] = {
                        --["Trick: Enrage"] = 25,
                    },
                    ["EnemyHits"] = {
                        --["a weak melee attack"] = 15,
                        ["Erupting Earth"] = 25,
                    },
                    ["Skills"] = {

                    },
                    ["Width"] = 800,
                    ["Text"] = "IT'S GONNA BLOW",
                    ["Height"] = 64,
                    ["Offset"] = 2,
                    ["ForegroundColour"] = Settings.BossTimersFrontTexture,
                    ["BackgroundColour"] =  Settings.BossTimersBackTexture,
                    ["Time"] = 25,
                    ["Font"] = "TrajanProBold30",
                },
            },
        }
    }
    --]]
    
    

    -- All tools are contained here
    self.tools = {}
    self.UI = {}
    self.timers = {}
    self.timerCount = 0

    self.ExecuteUpdate = function(delta)    
        --We had an update
        for key,_ in pairs(self.timers) do 
            if key:UpdateText(delta) == true then 
                self.timers[key] = nil
                if tablelength(self.timers) == 0 then
                    if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
                    self.updateCallback = nil
                end
            end
        end
    end

    self:Setup()

end

function PrimeUI:Setup()
    self.currentHeight = 0
    self.currentOrder = 0
    self.sameOrderOffset = 0

    -- Setup all the tools
    for toolName, data in spairs(self.data, function(t, a, b) return t[a]["Order"] < t[b]["Order"] end) do
        -- Make sure we udpate offset before creating tool
        self.currentHeight = self.currentHeight + data["OffsetAbove"]

        -- Setup dynamic skill bar
        if data["Type"] == "Skills" then
            self:SetupSkillBar(data, self)
        end

        if data["Type"] == "MoraleBar" then
            self:SetupMoraleBar(data, self)
        end

        if data["Type"] == "PowerBar" then
            self:SetupPowerBar(data, self)
        end

        if data["Type"] == "ResourceBar" then
            self:ResourceBar(data, self)
        end

        if data["Type"] == "ValueBar" then
            self:ValueBar(data, self)
        end

        if data["Type"] == "BuffBar" then
            self:BuffBar(data, self)
        end

        if data["Type"] == "Icon" then
            self:Icon(data, self)
        end

        if data["Type"] == "Timer" then
            self:Timer(data, self)
        end

        if data["Type"] == "BarGroup" then
            self:BarGroup(data, self)
        end

        -- Make sure we udpate offset after creating tool
        self.currentHeight = self.currentHeight + data["OffsetBelow"]
    end
end

function PrimeUI:BarGroup(data)

    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(self)
    valuebar:SetPosition(self:ContentAlignement(data["Alignment"], data["Width"] + 2 * data["BorderSize"]), self.currentHeight)
    valuebar:SetSize(data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"])
    valuebar:SetMouseVisible(false)
    valuebar:SetBackColor(data["BorderColour"])

    local innerbar = Turbine.UI.Control()
    innerbar:SetParent(valuebar)
    innerbar:SetPosition(data["BorderSize"], data["BorderSize"])
    innerbar:SetSize(data["Width"], data["Height"])
    innerbar:SetMouseVisible(false)
    innerbar:SetBackColor(data["BackgroundColour"])

    local heightBefore = self.currentHeight

    for _, barData in spairs(data["Bars"], function(t, a, b) return t[a]["Order"] < t[b]["Order"] end) do

        self.currentHeight = barData["Ypos"]

        if barData["Type"] == "MoraleBar" then
            self:SetupMoraleBar(barData, innerbar)
        end

        if barData["Type"] == "PowerBar" then
            self:SetupPowerBar(barData, innerbar)
        end

        if barData["Type"] == "ResourceBar" then
            self:ResourceBar(barData, innerbar)
        end

        if barData["Type"] == "ValueBar" then
            self:ValueBar(barData, innerbar)
        end

        if barData["Type"] == "BuffBar" then
            self:BuffBar(barData, innerbar)
        end

        if data["Type"] == "Icon" then
            self:Icon(barData, innerbar)
        end

        if data["Type"] == "Timer" then
            self:Timer(barData, innerbar)
        end
        
        if barData["Type"] == "BarGroup" then
            self:BarGroup(barData, innerbar)
        end

        self.currentHeight = heightBefore
    end

    table.insert(self.UI, valuebar)

end

function PrimeUI:ValueBar(data, parent)
    -- args (parent, width, height, x, y, barAlignment, barColour, backgroundColour, maxValue, barName, vertical, afterAnimation)
    local bar = ValueBar(parent, data["Width"], data["Height"], data["Xpos"], self.currentHeight, 
    data["Alignment"], data["ForegroundColour"], data["BackgroundColour"], 20, data["Texture"])

    table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
    function(effect)
        bar:SetMaxValue(effect:GetDuration())
        bar:SetCurrentValue(effect:GetDuration())
    end))

    table.insert(self.tools, bar)
end

function PrimeUI:BuffBar(data, parent)
    local buffbar = DynamicSkillBar(parent, 0, self.currentHeight, self.width, 32 * data["Scale"], 32 * data["Scale"], self:AlignmentToNumber(data["Alignment"]), true, true)
    
    buffbar.SkillID = {}

    if tablelength(data["Effects"]) > 0 or data["AllEffects"] then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            if data["Effects"][name] or data["AllEffects"] then
                if (not(data["OnlyBuffs"]) and not(data["OnlyDebuffs"])) or (data["OnlyDebuffs"] and effect:IsDebuff()) or (data["OnlyBuffs"] and not(effect:IsDebuff())) then
                    local highlight = data["AllHighlight"]
                    if data["Effects"][name] then
                        highlight = highlight or data["Effects"][name]["Highlight"]
                    end
                    buffbar.SkillID[effect:GetID()] = true
                    buffbar:AddLabel(name, ClassTimerLabel(buffbar, 0, 0, 32, 32, effect:GetIcon(), false, false, nil, 1, 1, false, false, true, data["Scale"], name, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight), effect:GetDuration())
                    buffbar:ToggleLabel(name, true, effect:GetDuration())
                end
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local ID = effect:GetID()
            if buffbar.SkillID[ID] then
                buffbar:RemoveLabel(effect:GetName())
            end
        end))
    end

    if tablelength(data["Skills"]) > 0 then
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    local icon = value["Icon"]
                    if value["Icon"] == true then
                        icon = skill:GetSkillInfo():GetIconImageID()
                    end

                    local time = value["Time"]
                    if value["Time"] == true then
                        time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                    end

                    local highlight = data["AllHighlight"]
                    if data["Skills"][name] then
                        highlight = highlight or data["Skills"][name]["Highlight"]
                    end

                    if not(buffbar:LabelExists(key)) then
                        buffbar:AddLabel(key, ClassTimerLabel(buffbar, 0, 0, 32, 32, icon, false, false, nil, 1, 1, false, false, true, data["Scale"], key, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight), time)
                    end
                    buffbar:ToggleLabel(key, true, time)
                end))
            end
        end
    end

    table.insert(self.tools, buffbar)

end

function PrimeUI:ResourceBar(data, parent)
    local resourceBar = ResourceBar(parent, data["Width"], data["Height"], self:ContentAlignement(data["Alignment"], data["Width"] + data["BorderSize"] * 2), self.currentHeight, 9, 
                                data["ForegroundColour"], data["BackgroundColour"], data["ShowText"],
                                data["BoxOffset"], data["BorderSize"], data["BorderColour"])
    
    if self.playerClass == Turbine.Gameplay.Class.Champion then
        resourceBar:ChangeResource(self.ClassAttr:GetFervor())
        self.attrCallbacks["FervorChanged"] = AddCallback(self.ClassAttr, "FervorChanged", function(sender, args) resourceBar:ChangeResource( self.ClassAttr:GetFervor() ) end)
    elseif self.playerClass == Turbine.Gameplay.Class.Hunter then
        resourceBar:ChangeResource(self.ClassAttr:GetFocus())
        self.attrCallbacks["FocusChanged"] = AddCallback(self.ClassAttr, "FocusChanged", function(sender, args) resourceBar:ChangeResource( self.ClassAttr:GetFocus() ) end)
    elseif self.playerClass == Turbine.Gameplay.Class.RuneKeeper then
        if data["RKLayout"] then
            local Attunement = sender:GetAttunement()
            if Attunement > 10 then
                resourceBar:ChangeResource(math.abs( Attunement - 10), data["SecondForegroundColour"], data["SecondBackgroundColour"])
            else
                resourceBar:ChangeResource(math.abs( Attunement - 10), data["SecondForegroundColour"], data["SecondBackgroundColour"])
            end
            self.attrCallbacks["AttunementChanged"] = AddCallback(self.ClassAttr, "AttunementChanged", function(sender, args)
                local Attunement = self.ClassAttr:GetAttunement()
                if Attunement > 10 then
                    resourceBar:ChangeResource(math.abs( Attunement - 10), data["SecondForegroundColour"], data["SecondBackgroundColour"])
                else
                    resourceBar:ChangeResource(math.abs( Attunement - 10), data["SecondForegroundColour"], data["SecondBackgroundColour"])
                end         
            end)
            
        else
            resourceBar:ChangeResource(self.ClassAttr:GetAttunement())
            self.attrCallbacks["AttunementChanged"] = AddCallback(self.ClassAttr, "AttunementChanged", function(sender, args) resourceBar:ChangeResource( self.ClassAttr:GetAttunement() ) end)
        end
    elseif self.playerClass == Turbine.Gameplay.Class.Brawler then
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if string.gsub(effect:GetName(), "%d", "") == "Endurance " then
                resourceBar:ChangeResource( tonumber(string.gsub(effect:GetName(), "%D", "")) )
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if string.gsub(effect:GetName(), "%d", "") == "Endurance " then
                resourceBar:ChangeResource( tonumber(string.gsub(effect:GetName(), "%D", "")) )
            end
        end))
    end

    table.insert(self.tools, resourceBar)

    self.currentHeight = self.currentHeight + 20
end

function PrimeUI:SetupMoraleBar(data, parent)
    -- args (parent, x, y, width, height, textFormat, textAlignment, barAlignment, barColours, barbackground)
    local moralebar = PermanentBar(parent, 0, self.currentHeight, 500, data["Height"], data["String"], Turbine.UI.ContentAlignment.MiddleCenter, data["Alignment"], data["Colours"], "bar25")
    moralebar:SetMaxValue(self.player:GetMaxMorale())
    moralebar:SetCurrentValue(self.player:GetMorale())

    table.insert(self.playerCallbacks, {[1] = "MoraleChanged", [2] = AddCallback(self.player, "MoraleChanged", function(sender, args) moralebar:SetCurrentValueImmediate( self.player:GetMorale()) end)})
    table.insert(self.playerCallbacks, {[1] = "MaxMoraleChanged", [2] = AddCallback(self.player, "MaxMoraleChanged", function(sender, args) moralebar:SetMaxValue( self.player:GetMaxMorale() ) end)})
    table.insert(self.playerCallbacks, {[1] = "BaseMaxMoraleChanged", [2] = AddCallback(self.player, "BaseMaxMoraleChanged", function(sender, args) moralebar:SetMaxValue( self.player:GetMaxMorale() ) end)})
    table.insert(self.playerCallbacks, {[1] = "TemporaryMoraleChanged", [2] = AddCallback(self.player, "TemporaryMoraleChanged", function(sender, args) moralebar:SetTempValue( self.player:GetTemporaryMorale()) end)})

    table.insert(self.tools, moralebar)

    self.currentHeight = self.currentHeight + data["Height"]
end

function PrimeUI:SetupPowerBar(data, parent)
    -- args (parent, x, y, width, height, textFormat, textAlignment, barAlignment, barColours, barbackground)
    local powerbar = PermanentBar(parent, 0, self.currentHeight, 500, data["Height"], data["String"], Turbine.UI.ContentAlignment.MiddleCenter, data["Alignment"], data["Colours"], "bar25")
    powerbar:SetMaxValue(self.player:GetMaxPower())
    powerbar:SetCurrentValue(self.player:GetPower())

    table.insert(self.playerCallbacks, {[1] = "PowerChanged", [2] = AddCallback(self.player, "PowerChanged", function(sender, args) powerbar:SetCurrentValueImmediate( self.player:GetPower() ) end)})
    table.insert(self.playerCallbacks, {[1] = "MaxPowerChanged", [2] = AddCallback(self.player, "MaxPowerChanged", function(sender, args) powerbar:SetMaxValue( self.player:GetMaxPower() ) end)})
    table.insert(self.playerCallbacks, {[1] = "BaseMaxPowerChanged", [2] = AddCallback(self.player, "BaseMaxPowerChanged", function(sender, args) powerbar:SetMaxValue( self.player:GetMaxPower() ) end)})
    table.insert(self.playerCallbacks, {[1] = "TemporaryPowerChanged", [2] = AddCallback(self.player, "TemporaryPowerChanged", function(sender, args) powerbar:SetTempValue( self.player:GetTemporaryPower()) end)})

    
    table.insert(self.tools, Powerbar)

    self.currentHeight = self.currentHeight + data["Height"]
end

function PrimeUI:SetupWrathBar(data, parent)
    -- args (parent, x, y, width, height, textFormat, textAlignment, barAlignment, barColours, barbackground)
    local wrathBar = PermanentBar(parent, 0, self.currentHeight, 500, data["Height"], data["String"], Turbine.UI.ContentAlignment.MiddleCenter, data["Alignment"], data["Colours"], "bar25")
    wrathBar:SetMaxValue(100)
    wrathBar:SetCurrentValue(self.ClassAttr:GetWrath())

    if self.playerClass == Turbine.Gameplay.Class.Beorning then
        self.attrCallbacks["WrathChanged"] = AddCallback(self.ClassAttr, "WrathChanged", function(sender, args) wrathBar:ChangeResource( self.ClassAttr:GetWrath() ) end)
    end

    table.insert(self.tools, wrathBar)

    self.currentHeight = self.currentHeight + data["Height"]
end


function PrimeUI:Timer(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(self)
    valuebar:SetPosition(self:ContentAlignement(data["Alignment"], data["Width"] + 2 * data["BorderSize"]), self.currentHeight)
    valuebar:SetSize(data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"])
    valuebar:SetMouseVisible(false)
    valuebar:SetBackColor(data["BorderColour"])

    local innerbar = Turbine.UI.Control()
    innerbar:SetParent(valuebar)
    innerbar:SetPosition(data["BorderSize"], data["BorderSize"])
    innerbar:SetSize(data["Width"], data["Height"])
    innerbar:SetMouseVisible(false)
    innerbar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    innerbar:SetBackground(data["BackgroundColour"])

    local currentHeight = 0
    self.colourScheme = {Turbine.UI.Color(0,0,0), Turbine.UI.Color(0,0,0), Turbine.UI.Color(0.5,0.5,0.5), Turbine.UI.Color(1,1,1), Turbine.UI.Color(0,0,0), TimerColourNormal2, Settings.BossTimersFrontTexture, Settings.BossTimersBackTexture, Settings.BossTimersFrontOpacity, Settings.BossTimersBackOpacity}
    for _, timerData in pairs(data["Timers"]) do
        local timer = EffectTimer(innerbar, timerData["Width"], timerData["Height"], nil, timerData["Text"], self:FontToFont(timerData["Font"]), self.colourScheme, true, nil, 0, timerData["Icon"], nil, not(data["AlwaysVisible"]))
        timer:SetTop(currentHeight)
        if not(data["AlwaysVisible"]) then
            timer:SetVisible(false)
        end


        if tablelength(timerData["Effects"]) > 0 then
            table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
            function(effect)
                local name = effect:GetName()
                if timerData["Effects"][name] ~= nil or true then
                    local time = timerData["Time"]
                    if timerData["Time"] == true then
                        time = effect:GetDuration()
                    end

                    timer:SetTimer(time, time)
                    self.timers[timer] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end))
        end

        if tablelength(timerData["Skills"]) > 0 then
            for key, value in pairs(timerData["Skills"]) do
                if SkillEventHandler:SkillExists(key) then
                    -- Add the callback for this skill slot
                    if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                    table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill)    
                        local time = value["Time"]
                        if value["Time"] == true then
                            time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                        end

                        timer:SetTimer(time, time)
                        self.timers[timer] = true
                        if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                    end))
                end
            end
        end

        if tablelength(timerData["EnemyHits"]) > 0 then
            table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(hitName)
                if timerData["EnemyHits"][hitName] then
                    timer:SetTimer(timerData["EnemyHits"][hitName], timerData["EnemyHits"][hitName])
                    self.timers[timer] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end))

        end
    
        if tablelength(timerData["PlayerHits"]) > 0 then
            table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(hitName)
                if timerData["PlayerHits"][hitName] then
                    timer:SetTimer(timerData["PlayerHits"][hitName], timerData["PlayerHits"][hitName])
                    self.timers[timer] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end))
        end


        currentHeight = currentHeight + timerData["Height"] + timerData["Offset"]
        table.insert(self.tools, timer)
    end

    table.insert(self.UI, valuebar)
end

function PrimeUI:Icon(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(self)
    valuebar:SetPosition(self:ContentAlignement(data["Alignment"], data["Width"] + 2 * data["BorderSize"]), self.currentHeight)
    valuebar:SetSize(data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"])
    valuebar:SetMouseVisible(false)
    valuebar:SetBackColor(data["BorderColour"])

    local innerbar = Turbine.UI.Control()
    innerbar:SetParent(valuebar)
    innerbar:SetPosition(data["BorderSize"], data["BorderSize"])
    innerbar:SetSize(data["Width"], data["Height"])
    innerbar:SetMouseVisible(false)
    innerbar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    innerbar:SetBackground(data["Icon"])

    valuebar:SetStretchMode(1)
    valuebar:SetSize((data["Width"] + 2 * data["BorderSize"]) * data["Scale"], (data["Height"] + 2 * data["BorderSize"]) * data["Scale"])

    if data["Highlight"] then
        innerbar.highlight = HighlightFrame2(innerbar, data["Scale"])
        innerbar.highlight:Toggle(true) 
    end

    for i = 1, self.player:GetEffects():GetCount(), 1 do
        local effect = self.player:GetEffects():Get(i)
        if data["Effects"][effect:GetName()] ~= nil then
            local value = data["Effects"][effect:GetName()]
            valuebar:SetVisible(value)
            if innerbar.highlight then 
                innerbar.highlight:Toggle(value) 
            end
        end
    end

    if tablelength(data["Effects"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            if data["Effects"][name] ~= nil then
                local value = data["Effects"][effect:GetName()]
                valuebar:SetVisible(value)
                if innerbar.highlight then innerbar.highlight:Toggle(value) end
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            if data["Effects"][name] ~= nil then
                local value = not(data["Effects"][effect:GetName()])
                valuebar:SetVisible(value)
                if innerbar.highlight then innerbar.highlight:Toggle(value) end
            end
        end))
    end

    table.insert(self.UI, valuebar)
end

function PrimeUI:Text(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(self)
    valuebar:SetPosition(self:ContentAlignement(data["Alignment"], data["Width"] + 2 * data["BorderSize"]), self.currentHeight)
    valuebar:SetSize(data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"])
    valuebar:SetMouseVisible(false)
    valuebar:SetBackColor(data["BorderColour"])

    local innerbar = Turbine.UI.Label()
    innerbar:SetParent(valuebar)
    innerbar:SetPosition(data["BorderSize"], data["BorderSize"])
    innerbar:SetSize(data["Width"], data["Height"])
    innerbar:SetMouseVisible(false)
    innerbar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    innerbar:SetFont(self:FontToFont(data["Font"]))
    innerbar:SetText(data["Text"])

    valuebar:SetStretchMode(1)
    valuebar:SetSize((data["Width"] + 2 * data["BorderSize"]) * data["Scale"], (data["Height"] + 2 * data["BorderSize"]) * data["Scale"])


    for i = 1, self.player:GetEffects():GetCount(), 1 do
        local effect = self.player:GetEffects():Get(i)
        if data["Effects"][effect:GetName()] ~= nil then
            local value = data["Effects"][effect:GetName()]
            valuebar:SetVisible(value)
        end
    end

    if tablelength(data["Effects"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            if data["Effects"][name] ~= nil then
                local value = data["Effects"][effect:GetName()]
                valuebar:SetVisible(value)
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            if data["Effects"][name] ~= nil then
                local value = not(data["Effects"][effect:GetName()])
                valuebar:SetVisible(value)
            end
        end))
    end

    table.insert(self.UI, valuebar)
end

function PrimeUI:SetupSkillBar(data, parent)
    -- Determine bar height
    local barHeight = 0

    -- Bar -> Bar name
    -- Bar data
    -- Skills
    -- Scale

    --args (parent, x, y, width, height, labelSize, sortMethod, hide)
    local skillbar = DynamicSkillBar(parent, 0, self.currentHeight, self.width, 32 * data["Scale"], 32 * data["Scale"], 4, false)
    table.insert(self.tools, skillbar)

    -- Skill data
    -- Name
    -- ID
    -- X prio
    -- Y prio
    -- Special triggers?
    for key, value in pairs(data["Skills"]) do
        if SkillEventHandler:SkillExists(value["Name"]) then
            --args (parent, x, y, base, normal, skill, hideOnCreation, pet, highlight)
            local skillSlot = ClassSkillSlot(skillbar, 0, 0, 32, 32, value["ID"], false, false, false, data["Scale"])

            -- Add the callback for this skill slot
            if not(self.skillCallbacks[value["Name"]]) then self.skillCallbacks[value["Name"]] = {} end
            table.insert(self.skillCallbacks[value["Name"]], AddCallback(SkillEventHandler, value["Name"], function(skill) 
                skillbar:SetLabelTimer(value["Name"], math.max(0.1, skill:GetResetTime() - Turbine.Engine.GetGameTime()), skill:GetCooldown())
            end))

            -- args (skill, label, priority, skillObject, y)
            skillbar:AddLabel(value["Name"], skillSlot, value["XPrio"], nil, value["YPrio"])

            -- Update max bar height
            barHeight = math.max(barHeight, value["YPrio"])
        end
    end

    skillbar:SetStaticPositions()

    -- Update height
    self.currentHeight = self.currentHeight + barHeight * 33 * data["Scale"]
end

function PrimeUI:ContentAlignement(alignment, width)
    if alignment == "Middle" then
        return (self.width - width) / 2
    elseif alignment == "Right" then
        return self.width - width
    else
        return 0 
    end
end

function PrimeUI:AlignmentToNumber(text)
    if text == "Middle" then return 2 end
    if text == "Left" then return 1 end
    if text == "Right" then return 3 end
    return 4
end

function PrimeUI:ColourTupleToTurbineColour(tuple)
    return Turbine.UI.Color(tuple[1], tuple[2], tuple[3], tuple[4])
end

function PrimeUI:FontToFont(font)
    return Turbine.UI.Lotro.Font[font]
end

function PrimeUI:Unload()
    for _, callbackTable in pairs(self.skillCallbacks) do
        for name, callback in pairs(callbackTable) do
            RemoveCallback(SkillEventHandler, name, callback)
        end
    end

    for _, callbackPair in pairs(self.playerCallbacks) do
        RemoveCallback(self.player, callbackPair[1], callbackPair[2])
    end

    for _, callback in pairs(self.effectAddedCallbacks) do
        RemoveCallback(PlayerEffectsHandler, "EffectAdded", callback)
    end
    
    for _, callback in pairs(self.effectRemovedCallbacks) do
        RemoveCallback(PlayerEffectsHandler, "EffectRemoved", callback)
    end

    for key, value in pairs(self.enemyChatCallback) do
        RemoveCallback(ChatEventHandler, "EnemyCombat", value)
    end

    for key, value in pairs(self.playerChatCallback) do
        RemoveCallback(ChatEventHandler, "PlayerCombat", value)
    end

    for _, tool in pairs(self.tools) do
        tool:Unload()
    end

    for _, UI in pairs(self.UI) do
        UI:SetVisible(false)
        UI:SetParent(nil)
    end

end