ClassTimerLabel = class(Turbine.UI.Window)
function ClassTimerLabel:Constructor(parent, x, y, base, normal, baseIcon, visible, reverse, duration, normalOpacity, usageOpacity, hide, shade, text, scale, name, selfDelete, textAlignment, StackTextAlignment, highlight)
    Turbine.UI.Window.Constructor(self)

    self.parent = parent
    self.base = base
    self.normal = normal
    self.reverse = reverse
    self.normalOpacity = normalOpacity
    self.usageOpacity = usageOpacity
    self.hide = hide
    self.shade = shade
    self.text = text
    self.delete = selfDelete
    self.name = name
    self.textAlignment = textAlignment
    self.StackTextAlignment = StackTextAlignment

    if not(self.textAlignment) then
        self.textAlignment = Turbine.UI.ContentAlignment.MiddleCenter
    end

    if not(self.StackTextAlignment) then
        self.StackTextAlignment = Turbine.UI.ContentAlignment.TopRight
    end

    self:SetParent(parent)
    self:SetPosition(x, y)
    self:SetSize(self.normal, self.normal)
    self:SetVisible(visible)
    self:SetOpacity(normalOpacity)
    self:SetMouseVisible(false)

    self.currentNumber = -1
    self.currentPercentage = 0

    self.scale = scale
    if not(self.scale) then
        self.scale = 1
    end

    self.baseImageID = 0x41007E35;

    self.IconLabel = Turbine.UI.Label()
    self.IconLabel:SetParent(self)
    self.IconLabel:SetSize(self.base, self.base)
    self.IconLabel:SetStretchMode(1)
    self.IconLabel:SetBackground(baseIcon)
    self.IconLabel:SetSize(self.normal * self.scale, self.normal * self.scale)
    self.IconLabel:SetMouseVisible(false)

    if name then
        self.stackLabel = Turbine.UI.Label()
        self.stackLabel:SetParent(self)
        self.stackLabel:SetSize(self.base, self.base)
        self.stackLabel:SetTextAlignment(self.StackTextAlignment)
        --self.stackLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16)
        self.stackLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16)
        self.stackLabel:SetOutlineColor(Turbine.UI.Color(0,0,0))
        self.stackLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
        self.stackLabel:SetMouseVisible(false)
        self.stackLabel:SetStretchMode(1)
        self.stackLabel:SetText(string.match(name, "%d"))
    end

    if self.shade then
        self.elapsedOverlay = Turbine.UI.Control();
        self.elapsedOverlay:SetStretchMode( 1 );
        self.elapsedOverlay:SetMouseVisible( false );
        --self.elapsedOverlay:SetZOrder( 99 );
        self.elapsedOverlay:SetBackColor( Turbine.UI.Color( 1, 0, 0, 0 ) );
        self.elapsedOverlay:SetBackColorBlendMode( Turbine.UI.BlendMode.Overlay );
        self.elapsedOverlay:SetParent( self );
        self.elapsedOverlay:SetPosition(-1, -1)
        self.elapsedOverlay:SetSize(self.normal + 2 * self.scale, self.normal + 2 * self.scale)
        self.elapsedOverlay:SetMouseVisible(false)
    end
    
    if self.text then
        self.textLabel = Turbine.UI.Label()
        self.textLabel:SetParent(self)
        self.textLabel:SetSize(self.normal * self.scale, self.normal * self.scale)
        self.textLabel:SetTextAlignment(self.textAlignment)
        --self.textLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16)
        self.textLabel:SetFont(Turbine.UI.Lotro.Font.FixedSys15)
        self.textLabel:SetOutlineColor(Turbine.UI.Color(0,0,0))
        self.textLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
        self.textLabel:SetMouseVisible(false)
    end

    self.highlight = highlight
    -- Add timer overlay
    if self.highlight then
        self.highlightFrame = HighlightFrame2(self, self.scale)
        self.highlightFrame:Toggle(true)
    end

    self.startTime = Turbine.Engine.GetGameTime()
end

function ClassTimerLabel:SetTimer(timer, maxTimer)
    if timer > 99999 then self.textLabel:SetVisible(false) else self.textLabel:SetVisible(true) end
    if maxTimer then
        self.overlaydDuration = maxTimer
        self.startTime = Turbine.Engine.GetGameTime() - (maxTimer - timer)
    else
        self.overlaydDuration = timer
        self.startTime = Turbine.Engine.GetGameTime()
    end
    self.duration = timer
    self:SetOpacity(self.usageOpacity)

    --Updater:Subscribe(self)
end

function ClassTimerLabel:ExecuteUpdate(delta)

    self.duration = self.duration - delta
    if self.duration < 0 then 
        --Updater:UnSubscribe(self)
        self:SetOpacity(self.normalOpacity)
        if self.hide then self:SetVisible(false) end 

        if self.text then
            self.textLabel:SetText("")
        end

        if self.delete and self.name then self.parent:RemoveLabel(self.name) end
        return true
    end

    if self.shade then

        local percentComplete = math.floor(60 * (Turbine.Engine.GetGameTime() - self.startTime) / self.overlaydDuration)
        if self.reverse then
            percentComplete = 60 - percentComplete
        end

        if self.currentPercentage ~= percentComplete then 

            self.currentPercentage = percentComplete

            local imageId = self.baseImageID + self.currentPercentage            

            self.elapsedOverlay:SetStretchMode( 3 );
            self.elapsedOverlay:SetSize( 32, 32 );
            self.elapsedOverlay:SetBackground( imageId );
            self.elapsedOverlay:SetStretchMode( 1 );
            self.elapsedOverlay:SetSize( self.normal + 2 * self.scale, self.normal + 2 * self.scale);
        end
    end

    local newNumber = math.round(self.duration)
    if newNumber == self.currentNumber then return end

    self.currentNumber = newNumber

    if self.text then
        self.textLabel:SetStretchMode(nil)
        self.textLabel:SetStretchMode(1)
        self.textLabel:SetText(self.currentNumber)
    end
end

function ClassTimerLabel:ToggleVisible(value)
    self:SetVisible(value)
    self.IconLabel:SetVisible(value)
    if self.highlight then
        self.highlightFrame:Toggle(value)
    end
end

function ClassTimerLabel:Unload()
    self.IconLabel:SetParent(nil)
    if self.textLabel then self.textLabel:SetParent(nil) end
    if self.stackLabel then self.stackLabel:SetParent(nil) end
    if self.elapsedOverlay then self.elapsedOverlay:SetParent(nil) end
    if self.highlightFrame then self.highlightFrame:SetParent(nil) end
    self:SetParent(nil)
    self = nil
end

function ClassTimerLabel:AddBackground(icon)
    self.IconLabel:SetStretchMode(nil)
    self.IconLabel:SetSize(self.base, self.base)
    self.IconLabel:SetStretchMode(1)
    self.IconLabel:SetBackground(icon)
    self.IconLabel:SetSize(self.normal, self.normal)
end