PlayerSkillEvents = class(Turbine.UI.Control)
function PlayerSkillEvents:Constructor()
    Turbine.UI.Control.Constructor(self)

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    self.playerSkills = {}
    self.skillCallbacks = {}

    self.SkillUpdate = function(sender, event, args)
        if type(event)=="function" then
            event(args);
        else
            if type(event)=="table" then
                local size = table.getn(event);
                local i;
                for i=1,size do
                    if type(event[i])=="function" then
                        event[i](args);
                    end
                end
            end
        end
    end

    self:SetCallbacks()
end

function PlayerSkillEvents:SetCallbacks()
    self.trainedSkills = self.player:GetTrainedSkills()

    for i = 1, self.trainedSkills:GetCount(), 1 do
        local item = self.trainedSkills:GetItem(i)
        local name = item:GetSkillInfo():GetName()

        self.playerSkills[name] = item

        self.skillCallbacks[name] =  AddCallback(self.playerSkills[name], "ResetTimeChanged", function(sender, args) 
            self.SkillUpdate(self, self[name], self.playerSkills[name])
        end)
    end
end

function PlayerSkillEvents:RemoveCallbacks()
    for name, callback in pairs(self.skillCallbacks) do
        RemoveCallback(self.playerSkills[name], "ResetTimeChanged", callback)
    end

    self.skillCallbacks = nil
    self.playerSkills = nil
    self.trainedSkills = nil

    collectgarbage()

    self.playerSkills = {}
    self.skillCallbacks = {}
end

function PlayerSkillEvents:Update()
    if Turbine.Engine.GetGameTime() > self.reloadTime then
        self:Reload()
        self:SetWantsUpdates(false)
    end
end

function PlayerSkillEvents:ReloadWithDelay()
    self.reloadTime = Turbine.Engine.GetGameTime() + 5
    self:SetWantsUpdates(true)
end

function PlayerSkillEvents:Reload()
    self:RemoveCallbacks()
    self:SetCallbacks()
end

function PlayerSkillEvents:SkillExists(name)
    return self.playerSkills[name] ~= nil
end