KillCounter = class(Turbine.UI.Window)
function KillCounter:Constructor()
    Turbine.UI.Window.Constructor(self);
    local width = 32 + 40
    local height = 32
    local x = Settings.KBX 
    local y = Settings.KBY

    self.LastKillTime = 0
    self.KillAmount = 0
    self.HighestMultiKill = 0
    self.KillStreak = 0

    self:SetSize(width, height)
    self:SetPosition(x, y)
    self:SetVisible(Settings.KBVisible)
    self:SetMouseVisible(true)
    --self:SetBackColor(GetRandomColour())

    if CharStats.DailyKB == nil then CharStats.DailyKB = 0 end

    if Settings.LastDailyReset < GetLastDailyReset() then 
        SaveDailyPVMPStats()
        CharStats.DailyKB = 0
        CharStats.DailyDeaths = 0
        Settings.LastDailyReset = GetLastDailyReset()
    end

    self.SkullLabel = Turbine.UI.Label()
    self.SkullLabel:SetParent(self)
    self.SkullLabel:SetSize(32, 32)
    --self.SkullLabel:SetBackColorBlendMode(Turbine.UI.BlendMode.Normal)
    --self.SkullLabel:SetBlendMode(Turbine.UI.BlendMode.Normal)
    if CharStats.DailyKB > 100 then
        self.SkullLabel:SetBackground(SmallGoldSkullIcon)
    else
        self.SkullLabel:SetBackground(SkullIcon)
    end
    self.SkullLabel:SetMouseVisible(false)

    local RedSkullLabel = Turbine.UI.Label()
    RedSkullLabel:SetParent(self)
    RedSkullLabel:SetSize(32, 32)
    --SkullLabel:SetBackColorBlendMode(Turbine.UI.BlendMode.Normal)
    --SkullLabel:SetBlendMode(Turbine.UI.BlendMode.Normal)
    RedSkullLabel:SetBackground(RedSkullIcon)
    RedSkullLabel:SetVisible(false)
    RedSkullLabel:SetMouseVisible(false)

    self.KillCount = Turbine.UI.Label()
    self.KillCount:SetParent(self)
    self.KillCount:SetSize(40, 32)
    self.KillCount:SetPosition(32, 0)
    self.KillCount:SetFont(Turbine.UI.Lotro.Font.TrajanProBold24)
    self.KillCount:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.KillCount:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.KillCount:SetMouseVisible(false)

    -- [[
    --]]
    self.KillCount:SetText(CharStats.DailyKB)

    self.DeathCount = Turbine.UI.Label()
    self.DeathCount:SetParent(self)
    self.DeathCount:SetSize(40, 32)
    self.DeathCount:SetPosition(32, 0)
    self.DeathCount:SetFont(Turbine.UI.Lotro.Font.TrajanProBold24)
    self.DeathCount:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.DeathCount:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.DeathCount:SetVisible(false)
    self.DeathCount:SetMouseVisible(false)
    self.DeathCount:SetForeColor(Turbine.UI.Color.Red)

    self.DeathCount:SetText(CharStats.DailyDeaths)

    self.moveBox = MoveBox(self, width, height, "KB")

    self.MouseClick = function()
        self.DeathCount:SetVisible(not(self.DeathCount:IsVisible()))
        self.KillCount:SetVisible(not(self.KillCount:IsVisible()))
        self.SkullLabel:SetVisible(not(self.SkullLabel:IsVisible()))
        RedSkullLabel:SetVisible(not(RedSkullLabel:IsVisible()))
    end
end

function KillCounter:UpdateKill(kb)

    -- Update kb count
    self.KillCount:SetText(CharStats.DailyKB)

    if CharStats.DailyKB > 100 then
        self.SkullLabel:SetBackground(SmallGoldSkullIcon)
    end

    -- Kill streak
    self.KillStreak = self.KillStreak + 1
    if self.KillStreak % 5 == 0 then 
        local KillStreakMessage = KillStreaks[self.KillStreak]
        if KillStreakMessage ~= nil then
            effectsWindow:Message(7, self.KillStreak .. " Kill Streak\n".. KillStreakMessage[1], KillStreakMessage[2], false, "Killstreak Message")
        else
            effectsWindow:Message(7, self.KillStreak .. " Kill Streak", SkullStackIcon, false, "Killstreak Message")
        end
    end

    -- Multi Kills
    if Turbine.Engine.GetGameTime() - self.LastKillTime < 20 then
        self.KillAmount = self.KillAmount + 1
        if self.KillAmount > 1 and Settings.EEnabled then
            local message = nil
            local icon = nil
            if self.KillAmount > 5 then
                message = "Domination"
                icon = GoldSkullIcon
            else
                message = KillMessages[self.KillAmount]
            end            

            effectsWindow:Message(4, message, icon, false, "Kill Message")
        end
    else
        self.KillAmount = 1
    end
    if self.HighestMultiKill < self.KillAmount then self.HighestMultiKill = self.KillAmount end
    self.LastKillTime = Turbine.Engine.GetGameTime()

    -- Chat message
    local DeathCount =  CharStats["KilledBy"][kb]
    local KillCount = CharStats["KB"][kb]

    if DeathCount == nil then DeathCount = 0 end
    if KillCount == nil then KillCount = 0 end

    if Settings.EttenmoorsState and Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() < 2 then
        ChatOutput("You have vanquished " .. kb .. " <rgb=#006013>" .. KillCount .. "</rgb> times and perished <rgb=#8e0000>" .. DeathCount .. "</rgb> times!")
    elseif Settings.EttenmoorsState and Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() == 2 then
        ChatOutput("You have slaughtered " .. kb .. " <rgb=#006013>" .. KillCount .. "</rgb> times for the glory of Angmar and sacrificed yourself <rgb=#8e0000>" .. DeathCount .. "</rgb> times!")
    end
end

function KillCounter:UpdateDeath()
    if self.KillStreak > CharStats["HighestKillStreak"] then CharStats["HighestKillStreak"] = self.KillStreak end
    if self.KillStreak > AccountStats["HighestKillStreak"] then AccountStats["HighestKillStreak"] = self.KillStreak end
    self.KillStreak = 0
    self.DeathCount:SetText(CharStats.DailyDeaths)
end


function KillCounter:ToggleMove()
    self.moveBox:ToggleMove()
end

function KillCounter:GetKillStreak()
    return(self.KillStreak)
end

function KillCounter:HighestKillCombo()
    return(self.HighestMultiKill)
end

KillMessages = {
    [2] = "Double Kill",
    [3] = "Triple Kill",
    [4] = "Quadra Kill",
    [5] = "Penta Kill",
}

KillStreaks = {
    [25] = {"Nuke Ready For Launch!", NukeIcon}
}

if Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() == 2 then
    KillStreaks[10] = {"Angmar's Champion", nil}
    KillStreaks[20] = {"Sulfur Pit Survivor", nil}
    KillStreaks[30] = {"Witch King's Champion", nil}
    KillStreaks[40] = {"Mordirith's Disciple", nil}
    KillStreaks[50] = {"Sauron's Chosen Champion", SauronEyeLarge}
    KillStreaks[60] = {"Darkness Incarnate", nil}
    KillStreaks[100] = {"Morgoth's Champion", nil}
else
    KillStreaks[10] = {"Bree's Champion", nil}
    KillStreaks[20] = {"Hero of the People", nil}
    KillStreaks[30] = {"Lainedhel's Champion", nil}
    KillStreaks[40] = {"Herald of Aragorn", nil}
    KillStreaks[50] = {"Champion of the Free-People", GondorTreeIcon}
    KillStreaks[60] = {"Champion of the Light", nil}
end