local accountDamageDealt = 0
local accountDamageTaken = 0
local accountHealing = 0
local characterDamageDealt = 0
local characterDamageTaken = 0
local characterHealing = 0

local PluginStartTime = Turbine.Engine.GetGameTime()

function LoadData()
    SwapWeapons = LoadTable(Turbine.DataScope.Character, "PrimeSwapData") 
    
    CharStats = LoadTable(Turbine.DataScope.Character, "PrimeCharStats")

    local tempCharStats = CharStats
    GetBaseCharStats()
    if tempCharStats == nil then tempCharStats = {} end
    for key, value in pairs(tempCharStats) do
        if CharStats[key] ~=  nil then
            CharStats[key] = value
        end
    end

    AccountStats = LoadTable(Turbine.DataScope.Account, "PrimeAccountStats")

    local tempAccountStats = AccountStats
    GetBaseAcountStats()
    if tempAccountStats == nil then tempAccountStats = {} end
    for key, value in pairs(tempAccountStats) do
        if AccountStats[key] ~=  nil then
            AccountStats[key] = value
        end
    end

    -- Reset faulty stat if you used 6.45
    if CharStats["Version"] == 0 then
        CharStats["HighestDailyDeaths"] = 0
        CharStats["HighestDailyKBs"] = 0
        CharStats["Version"] = 6.46
    end

    if AccountStats["Version"] == 0 then
        AccountStats["HighestDailyDeaths"] = 0
        AccountStats["HighestDailyKBs"] = 0
        AccountStats["FreepKBs"] = 0
        AccountStats["Version"] = 6.46
    end

end

function GetBaseAcountStats()
    AccountStats = {}
    AccountStats["Time"] = 0
    AccountStats["KB"] = {}
    AccountStats["Reload"] = 0
    AccountStats["Death"] = 0
    AccountStats["KilledBy"] = {}
    AccountStats["DamageTaken"] = 0
    AccountStats["DamageDealt"] = 0
    AccountStats["Healing"] = 0
    AccountStats["PVPKB"] = 0
    AccountStats["PVPDeath"] = 0
    AccountStats["MisadventureDeaths"] = 0
    AccountStats["HighestDailyDeaths"] = 0
    AccountStats["HighestDailyKBs"] = 0
    AccountStats["HighestKillStreak"] = 0
    AccountStats["HighestKillCombo"] = 0
    AccountStats["CreepDeaths"] = 0
    AccountStats["CreepKBs"] = 0
    AccountStats["FreepDeaths"] = 0
    AccountStats["FreepKBs"] = 0
    AccountStats["Tracks"] = 0
    AccountStats["BossData"] = {}
    AccountStats["MythicPlus"] = {}
    AccountStats["TEQ"] = {}
    AccountStats["Version"] = 0
end

function GetBaseCharStats()
    CharStats = {}
    CharStats["Time"] = 0
    CharStats["KB"] = {}
    CharStats["Reload"] = 0
    CharStats["Death"] = 0
    CharStats["KilledBy"] = {}
    CharStats["DamageTaken"] = 0
    CharStats["DamageDealt"] = 0
    CharStats["Healing"] = 0
    CharStats["DailyKB"] = 0
    CharStats["DailyDeaths"] = 0
    CharStats["PVPKB"] = 0
    CharStats["PVPDeath"] = 0
    CharStats["MisadventureDeaths"] = 0
    CharStats["HighestDailyDeaths"] = 0
    CharStats["HighestDailyKBs"] = 0
    CharStats["HighestKillStreak"] = 0
    CharStats["HighestKillCombo"] = 0
    CharStats["Tracks"] = 0
    CharStats["Version"] = 0
end

function ReturnBossData(instanceID, bossName, instanceTier)
    if not(AccountStats["BossData"][instanceID]) then
        AccountStats["BossData"][instanceID] = {}
    end

    if not(AccountStats["BossData"][instanceID][bossName]) then
        AccountStats["BossData"][instanceID][bossName] = {}
    end

    if not(AccountStats["BossData"][instanceID][bossName][instanceTier]) then
        AccountStats["BossData"][instanceID][bossName][instanceTier] = {
            ["Wipes"] = 0,
            ["Completions"] = 0,
            ["BestTime"] = 99999999,
            ["FirstCompletion"] = -1
        }
    end

    return(AccountStats["BossData"][instanceID][bossName][instanceTier])
end

function SaveBossData(instanceID, bossName, instanceTier, newTime, completion)

    -- Update completion/wipe count
    if completion then
        AccountStats["BossData"][instanceID][bossName][instanceTier]["Completions"] = AccountStats["BossData"][instanceID][bossName][instanceTier]["Completions"] + 1
    else
        AccountStats["BossData"][instanceID][bossName][instanceTier]["Wipes"] = AccountStats["BossData"][instanceID][bossName][instanceTier]["Wipes"] + 1
    end

    -- Update best time
    if AccountStats["BossData"][instanceID][bossName][instanceTier]["BestTime"] > newTime and completion then
        AccountStats["BossData"][instanceID][bossName][instanceTier]["BestTime"] = newTime
    end

    -- Update first completion
    if completion and AccountStats["BossData"][instanceID][bossName][instanceTier]["FirstCompletion"] < 0 then
        AccountStats["BossData"][instanceID][bossName][instanceTier]["FirstCompletion"] = Turbine.Engine.GetLocalTime()
    end
    
end

function SaveMythicPlusData(instanceID, name, value)
    if not(AccountStats["MythicPlus"][name]) then AccountStats["MythicPlus"][name] = {} end
    if not(AccountStats["MythicPlus"][name][instanceID]) then AccountStats["MythicPlus"][name][instanceID] = {} end
    table.insert(AccountStats["MythicPlus"][name][instanceID], value)
end

function AddMythicPlusScore(instanceID, name, value)
    if not(AccountStats["TEQ"][name]) then AccountStats["TEQ"][name] = {} end
    if not(AccountStats["TEQ"][name][instanceID]) then 
        AccountStats["TEQ"][name][instanceID] = value 
    else
        if AccountStats["TEQ"][name][instanceID]["Score"] < value["Score"] then
            AccountStats["TEQ"][name][instanceID] = value 
        end
    end
end

function AddKB(kb, pvp)
    if kb == nil then return end
    if CharStats["KB"][kb] == nil then
        CharStats["KB"][kb] = 1
    else
        CharStats["KB"][kb] = CharStats["KB"][kb] + 1 
    end

    if AccountStats["KB"][kb] == nil then
        AccountStats["KB"][kb] = 1
    else
        AccountStats["KB"][kb] = AccountStats["KB"][kb] + 1 
    end

    if pvp and Settings.EttenmoorsState then 
        CharStats.DailyKB = CharStats.DailyKB + 1
        if KillTracker ~= nil then KillTracker:UpdateKill(kb) end
        AccountStats["PVPKB"] = AccountStats["PVPKB"] + 1
        CharStats["PVPKB"] = CharStats["PVPKB"] + 1

        if Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() == 2 then
            AccountStats["CreepKBs"] = AccountStats["CreepKBs"] + 1
        else
            AccountStats["FreepKBs"] = AccountStats["FreepKBs"] + 1
        end
     end 
end

function AddDamageTaken(dam)
    characterDamageTaken = characterDamageTaken + dam
    accountDamageTaken = accountDamageTaken + dam
end

function AddDamageDealt(dam)
    characterDamageDealt = characterDamageDealt + dam
    accountDamageDealt = accountDamageDealt + dam
end

function AddHealing(heal)
    characterHealing = characterHealing + heal
    accountHealing = accountHealing + heal
end

function AddReload()
    CharStats["Reload"] = CharStats["Reload"] + 1
    AccountStats["Reload"] = AccountStats["Reload"] + 1
end

function AddTrack()
    CharStats["Tracks"] = CharStats["Tracks"] + 1
    AccountStats["Tracks"] = AccountStats["Tracks"] + 1
end

function AddDeath(kb, misadventure)
    if CharStats["KilledBy"] == nil then  CharStats["KilledBy"] = {} end
    if AccountStats["KilledBy"] == nil then  AccountStats["KilledBy"] = {} end

    CharStats["Death"] = CharStats["Death"] + 1
    AccountStats["Death"] = AccountStats["Death"] + 1

    if kb == nil and not(misadventure) then return end

    if kb ~= nil then 
        if CharStats["KilledBy"][kb] == nil then
            CharStats["KilledBy"][kb] = 1
        else
            CharStats["KilledBy"][kb] = CharStats["KilledBy"][kb] + 1 
        end

        if AccountStats["KilledBy"][kb] == nil then
            AccountStats["KilledBy"][kb] = 1
        else
            AccountStats["KilledBy"][kb] = AccountStats["KilledBy"][kb] + 1 
        end
    elseif misadventure then
        CharStats["MisadventureDeaths"] = CharStats["MisadventureDeaths"] + 1
        AccountStats["MisadventureDeaths"] = AccountStats["MisadventureDeaths"] + 1
    end

    if KillTracker ~= nil and Settings.EttenmoorsState then 
        CharStats.DailyDeaths = CharStats.DailyDeaths + 1
        AccountStats["PVPDeath"] = AccountStats["PVPDeath"] + 1
        CharStats["PVPDeath"] = CharStats["PVPDeath"] + 1
        KillTracker:UpdateDeath() 

        if Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() == 2 then
            AccountStats["CreepDeaths"] = AccountStats["CreepDeaths"] + 1
        else
            AccountStats["FreepDeaths"] = AccountStats["FreepDeaths"] + 1
        end
    end
end

function SaveStats()
    local pluginUptime = Turbine.Engine.GetGameTime() - PluginStartTime

    -- Daily pvmp stats
    SaveDailyPVMPStats()

    if KillTracker ~= nil then
        if KillTracker:GetKillStreak() > CharStats["HighestKillStreak"] then CharStats["HighestKillStreak"] = KillTracker:GetKillStreak()  end
        if KillTracker:GetKillStreak() > AccountStats["HighestKillStreak"] then AccountStats["HighestKillStreak"] = KillTracker:GetKillStreak()  end
        if KillTracker:HighestKillCombo() > CharStats["HighestKillCombo"] then CharStats["HighestKillCombo"] = KillTracker:HighestKillCombo() end
        if KillTracker:HighestKillCombo() > AccountStats["HighestKillCombo"] then AccountStats["HighestKillCombo"] = KillTracker:HighestKillCombo() end
    end

    -- Handle char stats
    CharStats["Time"] = CharStats["Time"] + pluginUptime
    CharStats["DamageTaken"] = CharStats["DamageTaken"] + characterDamageTaken
    CharStats["DamageDealt"] = CharStats["DamageDealt"] + characterDamageDealt
    CharStats["Healing"] = CharStats["Healing"] + characterHealing
    SaveTable(Turbine.DataScope.Character, "PrimeCharStats", CharStats)

    -- Handle account stats
    AccountStats["Time"] = AccountStats["Time"] + pluginUptime
    AccountStats["DamageTaken"] = AccountStats["DamageTaken"] + accountDamageTaken
    AccountStats["DamageDealt"] = AccountStats["DamageDealt"] + accountDamageDealt
    AccountStats["Healing"] = AccountStats["Healing"] + accountHealing
    SaveTable(Turbine.DataScope.Account, "PrimeAccountStats", AccountStats)
end

function SaveDailyPVMPStats()
    if CharStats.DailyDeaths > CharStats["HighestDailyDeaths"] then CharStats["HighestDailyDeaths"] = CharStats.DailyDeaths end
    if CharStats.DailyDeaths > AccountStats["HighestDailyDeaths"] then AccountStats["HighestDailyDeaths"] = CharStats.DailyDeaths end
    if CharStats.DailyKB > CharStats["HighestDailyKBs"] then CharStats["HighestDailyKBs"] = CharStats.DailyKB end
    if CharStats.DailyKB > AccountStats["HighestDailyKBs"] then AccountStats["HighestDailyKBs"] = CharStats.DailyKB end
end

Stats = class(Turbine.UI.Lotro.Window)
function Stats:Constructor()
    local width = 1100
    local height = 750
    local x = math.floor(Turbine.UI.Display.GetWidth() / 2 - (width /2))
    local y = Settings.LOY
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Lotro.Window.Constructor(self);
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetPosition(x,y);
    self:SetWantsKeyEvents(true)
    self:SetText("Stats")
    self:SetVisible(true)

    local charKBs = 0

    for key, value in pairs(CharStats["KB"]) do charKBs = charKBs + value end

    local accKBs = 0

    for key, value in pairs(AccountStats["KB"]) do accKBs = accKBs + value end

    self.CreateCharStatLabels = function()
        local charItems = Turbine.UI.Control();
        charItems:SetParent(self)
        charItems:SetVisible(true)
        charItems:SetSize( 500, 400 );
        charItems:SetPosition(0, 0)
        charItems:SetMouseVisible(false)

        GetStatLabel(charItems, 20, 30, "Character stats:")
        GetStatLabel(charItems, 20, 50, "Uptime: " .. Round(CharStats["Time"]/3600, 1) .. "h")
        GetStatLabel(charItems, 20, 70, "Reloads: " .. CharStats["Reload"])
        GetStatLabel(charItems, 20, 90, "Deaths: " .. CharStats["Death"])
        GetStatLabel(charItems, 20, 110, "Misadventure Deaths: " .. CharStats["MisadventureDeaths"])
        GetStatLabel(charItems, 20, 130, "Killing Blows: " .. charKBs)
        GetStatLabel(charItems, 20, 170, "Character PVMP Stats")
        GetStatLabel(charItems, 20, 190, "Killing Blows: " .. CharStats["PVPKB"])
        GetStatLabel(charItems, 20, 210, "Deaths: " .. CharStats["PVPDeath"])
        GetStatLabel(charItems, 20, 230, "Highest Daily Killing Blows: " .. CharStats["HighestDailyKBs"])
        GetStatLabel(charItems, 20, 250, "Highest Daily Deaths: " .. CharStats["HighestDailyDeaths"])
        GetStatLabel(charItems, 20, 270, "Highest Killstreak: " .. CharStats["HighestKillStreak"])
        GetStatLabel(charItems, 20, 290, "Highest Multi-Kill: " .. CharStats["HighestKillCombo"])
        GetStatLabel(charItems, 20, 310, "Tracks: " .. CharStats["Tracks"])
    end

    self.CreateAccountStatLabels = function()
        local accItems = Turbine.UI.Control();
        accItems:SetParent(self)
        accItems:SetVisible(true)
        accItems:SetSize( 500, 400 );
        accItems:SetPosition(0, 310)
        accItems:SetMouseVisible(false)


        GetStatLabel(accItems, 20, 30, "Account stats:")
        GetStatLabel(accItems, 20, 50, "Uptime: " .. Round(AccountStats["Time"]/3600, 1) .. "h")
        GetStatLabel(accItems, 20, 70, "Reloads: " .. AccountStats["Reload"])
        GetStatLabel(accItems, 20, 90, "Deaths: " .. AccountStats["Death"])
        GetStatLabel(accItems, 20, 110, "Misadventure Deaths: " .. AccountStats["MisadventureDeaths"])
        GetStatLabel(accItems, 20, 130, "Killing Blows: " .. accKBs)
        GetStatLabel(accItems, 20, 170, "Account PVMP stats: ")
        GetStatLabel(accItems, 20, 190, "Killing Blows: " .. AccountStats["PVPKB"])
        GetStatLabel(accItems, 20, 210, "Deaths: " .. AccountStats["PVPDeath"])
        GetStatLabel(accItems, 20, 230, "Highest Daily Killing Blows: " .. AccountStats["HighestDailyKBs"])
        GetStatLabel(accItems, 20, 250, "Highest Daily Deaths: " .. AccountStats["HighestDailyDeaths"])
        GetStatLabel(accItems, 20, 270, "Highest Killstreak: " .. AccountStats["HighestKillStreak"])
        GetStatLabel(accItems, 20, 290, "Highest Multi-Kill: " .. AccountStats["HighestKillCombo"])
        GetStatLabel(accItems, 20, 310, "Creep Deaths: " .. AccountStats["CreepDeaths"])
        GetStatLabel(accItems, 20, 330, "Creep Killing Blows: " .. AccountStats["CreepKBs"])
        GetStatLabel(accItems, 20, 350, "Freep Deaths: " .. AccountStats["FreepDeaths"])
        GetStatLabel(accItems, 20, 370, "Freep Killing Blows: " .. AccountStats["FreepKBs"])
        GetStatLabel(accItems, 20, 390, "Tracks: " .. AccountStats["Tracks"])
    end

    self.CreateCharStatLabels()
    self.CreateAccountStatLabels()

    local DescriptionLabel = Turbine.UI.Label()
    DescriptionLabel:SetParent(self)
    DescriptionLabel:SetSize(200, 15)
    DescriptionLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    DescriptionLabel:SetMouseVisible(false)
    --NameLabel:SetMultiline(false)
    DescriptionLabel:SetPosition(250, 35)
    DescriptionLabel:SetText("Character KBs")

    local container = Turbine.UI.ListBox()
    container:SetParent(self)
    container:SetSize(200, height - 100)
    container:SetPosition(250, 50)
    --container:SetBackColor(GoldColor)
    container:SetVisible(true)
    container:SetWantsKeyEvents(true)
    container:SetMouseVisible(true)

    local containerScrollbar = Turbine.UI.Lotro.ScrollBar();
    containerScrollbar:SetOrientation( Turbine.UI.Orientation.Vertical );
    containerScrollbar:SetParent( self );
    containerScrollbar:SetPosition( 450 - 10, 50 );
    containerScrollbar:SetSize( 10, height - 100 );
    container:SetVerticalScrollBar( containerScrollbar );

    local SortedEffects = {}
    for key, value in pairs(CharStats["KB"]) do table.insert( SortedEffects, key ) end
    table.sort( SortedEffects )

    for _, value in pairs(SortedEffects) do
        local listItem = Turbine.UI.Control();
        listItem:SetVisible(true)
        listItem:SetSize( 200, 15 );
        listItem:SetMouseVisible(true)
    
        local NameLabel = Turbine.UI.Label()
        NameLabel:SetParent(listItem)
        NameLabel:SetSize(200, 15)
        NameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
        NameLabel:SetMouseVisible(false)
        NameLabel:SetMultiline(false)
        NameLabel:SetTop(2)
        NameLabel:SetText(CharStats["KB"][value] .. " - " .. value)
    
        container:AddItem( listItem );
    end

    local DescriptionLabel = Turbine.UI.Label()
    DescriptionLabel:SetParent(self)
    DescriptionLabel:SetSize(200, 15)
    DescriptionLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    DescriptionLabel:SetMouseVisible(false)
    --NameLabel:SetMultiline(false)
    DescriptionLabel:SetPosition(450, 35)
    DescriptionLabel:SetText("Account KBs")

    local container = Turbine.UI.ListBox()
    container:SetParent(self)
    container:SetSize(200, height - 100)
    container:SetPosition(450, 50)
    --container:SetBackColor(GoldColor)
    container:SetVisible(true)
    container:SetWantsKeyEvents(true)
    container:SetMouseVisible(true)

    local containerScrollbar = Turbine.UI.Lotro.ScrollBar();
    containerScrollbar:SetOrientation( Turbine.UI.Orientation.Vertical );
    containerScrollbar:SetParent( self );
    containerScrollbar:SetPosition( 650 - 10, 50 );
    containerScrollbar:SetSize( 10, height - 100 );
    container:SetVerticalScrollBar( containerScrollbar );

    local SortedEffects = {}
    for key, value in pairs(AccountStats["KB"]) do 
        if value ~= 0 then
            table.insert( SortedEffects, key ) 
        else
            AccountStats["KB"][key] = nil
        end
    end
    table.sort( SortedEffects )

    for _, value in pairs(SortedEffects) do
        local listItem = Turbine.UI.Control();
        listItem:SetVisible(true)
        listItem:SetSize( 200, 15 );
        listItem:SetMouseVisible(true)
    
        local NameLabel = Turbine.UI.Label()
        NameLabel:SetParent(listItem)
        NameLabel:SetSize(200, 15)
        NameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
        NameLabel:SetMouseVisible(false)
        NameLabel:SetMultiline(false)
        NameLabel:SetTop(2)
        NameLabel:SetText(AccountStats["KB"][value] .. " - " .. value)
    
        container:AddItem( listItem );
    end

    -- Char Deahts

    local DescriptionLabel = Turbine.UI.Label()
    DescriptionLabel:SetParent(self)
    DescriptionLabel:SetSize(200, 15)
    DescriptionLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    DescriptionLabel:SetMouseVisible(false)
    --NameLabel:SetMultiline(false)
    DescriptionLabel:SetPosition(650, 35)
    DescriptionLabel:SetText("Character Deaths")

    local container = Turbine.UI.ListBox()
    container:SetParent(self)
    container:SetSize(200, height - 100)
    container:SetPosition(650, 50)
    --container:SetBackColor(GoldColor)
    container:SetVisible(true)
    container:SetWantsKeyEvents(true)
    container:SetMouseVisible(true)

    local containerScrollbar = Turbine.UI.Lotro.ScrollBar();
    containerScrollbar:SetOrientation( Turbine.UI.Orientation.Vertical );
    containerScrollbar:SetParent( self );
    containerScrollbar:SetPosition( 850 - 10, 50 );
    containerScrollbar:SetSize( 10, height - 100 );
    container:SetVerticalScrollBar( containerScrollbar );

    local SortedEffects = {}
    for key, value in pairs(CharStats["KilledBy"]) do table.insert( SortedEffects, key ) end
    table.sort( SortedEffects )

    for _, value in pairs(SortedEffects) do
        local listItem = Turbine.UI.Control();
        listItem:SetVisible(true)
        listItem:SetSize( 200, 15 );
        listItem:SetMouseVisible(true)
    
        local NameLabel = Turbine.UI.Label()
        NameLabel:SetParent(listItem)
        NameLabel:SetSize(200, 15)
        NameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
        NameLabel:SetMouseVisible(false)
        NameLabel:SetMultiline(false)
        NameLabel:SetTop(2)
        NameLabel:SetText(CharStats["KilledBy"][value] .. " - " .. value)
    
        container:AddItem( listItem );
    end

    --Char Deaths
    local DescriptionLabel = Turbine.UI.Label()
    DescriptionLabel:SetParent(self)
    DescriptionLabel:SetSize(200, 15)
    DescriptionLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    DescriptionLabel:SetMouseVisible(false)
    --NameLabel:SetMultiline(false)
    DescriptionLabel:SetPosition(850, 35)
    DescriptionLabel:SetText("Account Deaths")

    local container = Turbine.UI.ListBox()
    container:SetParent(self)
    container:SetSize(200, height - 100)
    container:SetPosition(850, 50)
    --container:SetBackColor(GoldColor)
    container:SetVisible(true)
    container:SetWantsKeyEvents(true)
    container:SetMouseVisible(true)

    local containerScrollbar = Turbine.UI.Lotro.ScrollBar();
    containerScrollbar:SetOrientation( Turbine.UI.Orientation.Vertical );
    containerScrollbar:SetParent( self );
    containerScrollbar:SetPosition( 1050 - 10, 50 );
    containerScrollbar:SetSize( 10, height - 100 );
    container:SetVerticalScrollBar( containerScrollbar );

    local SortedEffects = {}
    for key, value in pairs(AccountStats["KilledBy"]) do table.insert( SortedEffects, key ) end
    table.sort( SortedEffects )

    for _, value in pairs(SortedEffects) do
        local listItem = Turbine.UI.Control();
        listItem:SetVisible(true)
        listItem:SetSize( 200, 15 );
        listItem:SetMouseVisible(true)
    
        local NameLabel = Turbine.UI.Label()
        NameLabel:SetParent(listItem)
        NameLabel:SetSize(200, 15)
        NameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
        NameLabel:SetMouseVisible(false)
        NameLabel:SetMultiline(false)
        NameLabel:SetTop(2)
        NameLabel:SetText(AccountStats["KilledBy"][value] .. " - " .. value)
    
        container:AddItem( listItem );
    end
end

function GetStatLabel(parent, x, y, text)
    local DescriptionLabel = Turbine.UI.Label()
    DescriptionLabel:SetParent(parent)
    DescriptionLabel:SetSize(250, 15)
    DescriptionLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    DescriptionLabel:SetMouseVisible(false)
    --NameLabel:SetMultiline(false)
    DescriptionLabel:SetPosition(x, y)
    DescriptionLabel:SetText(text) 
end
