function Round(num, numDecimalPlaces)
    local mult = 10^(numDecimalPlaces or 0)
    local number = math.floor(num * mult + 0.5) / mult
    if (number % 1) == 0 and numDecimalPlaces ~= 0 then
        return tostring(number .. ".0")
    else
        return number
    end
end

function tablelength(T)
    local count = 0
    if T ~= nil then
        for _ in pairs(T) do count = count + 1 end
    end
    return count
end


function Tablelength(T)
    return tablelength(T)
end

function PrintTable(T)
    ChatOutput("Printing list")
    for k, item in pairs(T) do
        ChatOutput(k.." "..item[6])
    end
end


function Compare(a, b)
    --ChatOutput(a[3]..b[3])
    if a ~= nil and b ~= nil then
        return a[2] < b[2]
    end
end

function Compare2(a, b)
    --ChatOutput(a[3]..b[3])
    if a ~= nil and b ~= nil then
        return a[2] > b[2]
    end
end

function ChatOutput(string) Turbine.Shell.WriteLine(pluginString .. string) end

function DirectChatOutput(string) Turbine.Shell.WriteLine(string) end

function Print(string) if true then Turbine.Shell.WriteLine(tostring(string)) end end

function GetRandomColour() return(Turbine.UI.Color(math.random(), math.random(), math.random())) end

function AddListItem(text, box, ItemSelectedLabel, extraText)
        
    local listItem = Turbine.UI.Control();
    listItem:SetVisible(true)
    listItem:SetSize( 200, 15 );
    listItem:SetMouseVisible(true)

    listItem.MouseClick = function()
        ItemSelectedLabel:SetText(text)
    end

    local ttttext = text

    if extraText ~= nil then
        ttttext = text .. " - " .. extraText
    end

    local NameLabel = Turbine.UI.Label()
    NameLabel:SetParent(listItem)
    NameLabel:SetSize(200, 15)
    NameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    NameLabel:SetMouseVisible(false)
    --NameLabel:SetMultiline(false)
    NameLabel:SetTop(2)
    NameLabel:SetText(ttttext)

    box:AddItem( listItem );
end

function FormatBigNumber(number)
    if number == nil then return 0 end
    local returnString = nil
    if number < 1000 then
        returnString = number
    elseif number >= 1000 and number < 1000000 then
        returnString = Round((number / 1000), 1) .. "k"
    elseif number >= 1000000 and number < 1000000000 then
        returnString = Round((number / 1000000),1) .. "m"
    elseif number >= 1000000000 and number < 1000000000000 then 
        returnString = Round((number / 1000000000), 1) .. "b"
    elseif number >= 1000000000000 then
        returnString = Round((number / 1000000000000), 1) .. "t"
    else
        returnString = number
    end

    return returnString
end

function FormatBigNumberVitals(number)
    if number == nil then return 0 end
    local returnString = nil
    if number < 1000 then
        returnString = Round(number, 0)
    elseif number >= 1000 and number < 1000000 then
        returnString = Round((number / 1000), 1) .. "k"
    elseif number >= 1000000 and number < 1000000000 then
        returnString = Round((number / 1000000),1) .. "m"
    elseif number >= 1000000000 and number < 1000000000000 then 
        returnString = Round((number / 1000000000), 1) .. "b"
    elseif number >= 1000000000000 then
        returnString = Round((number / 1000000000000), 1) .. "t"
    else
        returnString = number
    end

    return returnString
end

function ClearUI() 
    --if Settings.MEnabled then messageWindow:Clear() end
    
    if Settings.SEnabled then skillCounters:Clear() end
    if Settings.EEnabled then effectsWindow:Clear() end
    if Settings.SPEnabled then skillTimers:Clear() end
    if Settings.SEnabled then skillCounters:Clear() end
    if Settings.SPEnabled then skillTimers:Clear() end
    if Settings.SEnabled then skillCounters:Clear() end
    if Settings.SPEnabled then skillTimers:Clear() end
    if Settings.SEnabled then skillCounters:Clear() end
    if Settings.SPEnabled then skillTimers:Clear() end
end

function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

function GetLastReset()
    return(math.floor((Turbine.Engine.GetLocalTime() - BeginningOfPrimeTime)/SecondsInWeek) * SecondsInWeek + BeginningOfPrimeTime)
end

function GetLastDailyReset()
    return(math.floor((Turbine.Engine.GetLocalTime() - BeginningOfPrimeTime)/SecondsInDay) * SecondsInDay + BeginningOfPrimeTime)
end

function GetLastT1Reset()
    local lastReset = GetLastReset()
    if lastReset + (SecondsInDay * 2) > Turbine.Engine.GetLocalTime() then
        return lastReset
    elseif lastReset + (SecondsInDay * 4) > Turbine.Engine.GetLocalTime() then
        return lastReset + SecondsInDay * 2
    elseif lastReset + (SecondsInDay * 7) > Turbine.Engine.GetLocalTime() then 
        return lastReset + SecondsInDay * 4
    end
end

function HandleLootUpdate()
    CheckLootTable()
    local lastWeeklyReset = GetLastReset()
    local lastT1Reset = GetLastT1Reset()
    local lastDailyReset = GetLastDailyReset()

    if GetLastReset() > LootLocks.LastReset then
        ClearWeeklyLootTable()
    else
        if GetLastT1Reset() > LootLocks.LastT1Reset then
            ClearT1RaidLootTable()
        end

        if GetLastDailyReset() > LootLocks.LastDailyReset then
            ClearDailyLootTable()
        end
    end
end

function ClearWeeklyLootTable()   
    -- Clear table
    LootLocks = {}

    -- Initialize values
    LootLocks.Locks = {}
    LootLocks.RaidLocks = {}
    LootLocks.WeeklyLocks = {}
    LootLocks.Characters = {}
    ClearT1RaidLootTable()
    ClearDailyLootTable()

    -- Set last resets
    LootLocks.LastReset = GetLastReset()

    ChatOutput("Cleared Weekly Locks")
end

function ClearT1RaidLootTable()
    LootLocks.T1RaidLocks = {}
    LootLocks.LastT1Reset = GetLastT1Reset()

    ChatOutput("Cleared T1 Raid Locks")
end

function ClearDailyLootTable()
    LootLocks.DailyLocks = {}
    LootLocks.T1Locks = {}
    LootLocks.LastDailyReset = GetLastDailyReset()

    ChatOutput("Cleared Daily Locks")
end

function CheckLootTable()
    local tempLoot = LootLocks

    LootLocks = {}
    
    -- Instances
    LootLocks.Locks = {}
    LootLocks.T1Locks = {}

    -- Raids
    LootLocks.RaidLocks = {}
    LootLocks.T1RaidLocks = {}

    -- Quests
    LootLocks.WeeklyLocks = {}
    LootLocks.DailyLocks = {}
    LootLocks.Characters = {}

    -- Reset times
    LootLocks.LastT1Reset = BeginningOfPrimeTime
    LootLocks.LastDailyReset = BeginningOfPrimeTime
    LootLocks.LastReset = BeginningOfPrimeTime

    for key, value in pairs(LootLocks) do
        if tempLoot[key] ~= nil then
            LootLocks[key] = tempLoot[key]
        end
    end
end

function TimeDeltaToString(time)
    local minutes = math.floor(time / 60)

    local seconds = math.floor(time % 60)
    if seconds < 10 then
        seconds = "0" .. seconds
    end

    local milliSeconds = time - math.floor(time)
    
    if milliSeconds == 0 then milliSeconds = "0.0000" end

    return(minutes .. ":" .. seconds .. "." .. string.sub(milliSeconds, 3, 4))
end

function CombatTimeDeltaToString(time)
    local minutes = math.floor(time / 60)

    local seconds = math.floor(time % 60)
    if seconds < 10 then
        seconds = "0" .. seconds
    end

    return(minutes .. ":" .. seconds)
end

function FormatTimeString(time)
    local minutes = math.floor(time / 60)
    local seconds = string.format("%02.f", math.floor(time % 60))
    local milliseconds = string.format("%02.f", math.floor(time * 100 % 100))

    return(minutes .. ":" .. seconds .. "." .. milliseconds)
end

function FormatTimeStringNoMilli(time)
    local minutes = math.floor(time / 60)
    local seconds = string.format("%02.f", math.floor(time % 60))

    return(minutes .. ":" .. seconds)
end


function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
    return callback;
end

function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end

function math.clamp(value, lower, upper)
    return math.max(lower, math.min(upper, value))
end

function BoolToString(bool) if bool then return ("True") else return("False") end end