WindowHandler = class(Turbine.UI.Control)
function WindowHandler:Constructor()
    Turbine.UI.Control.Constructor(self);
    self:SetWantsKeyEvents(true)

    -- Window collection
    Windows = {}

    -- Check for reload data
    ReloadStateData = LoadTable(Turbine.DataScope.Server, "PrimeReloadData") 

    -- Setup player and party data
    Player = Turbine.Gameplay.LocalPlayer.GetInstance();
    PlayerName = Player:GetName()
    Name = Player:GetName();
    Party = Player:GetParty();
    if Party ~= nil then
        GroupSize = Party:GetMemberCount()
    else
        GroupSize = 1
    end

    -- Create parser
    ChatEventHandler = ChatHandler()
    Windows["ChatEventHandler"] = ChatEventHandler

    -- Handle lootlock data
    LootLocks = LoadTable(Turbine.DataScope.Server, "PrimeLootLocks")
    if LootLocks == nil then
        LootLocks = {}
    end

    --Build current party to not break the plugin
    Group = {}
    GroupIDToName = {}
    PlayerEffectsHandler = PlayerEffectEvents()
    PlayerEffectsHandler:SetupOwnEvents()
    Windows["PlayerEffectHandler"] = PlayerEffectsHandler
    GroupEffectsHandler = GroupEffectEvents()
    Windows["GroupEffectsHandler"] = GroupEffectsHandler
    SkillEventHandler = Handlers.PlayerSkillEvents()
    

    --SetGroup() -- TODO ADD NEW BINDING
    --SetEffects()

    -- Create the global upate handler
    Updater = Handlers.UpdateHandler()
    Windows["Updater"] = Updater

    -- Countdown window
    countdownWindow = CountdownWindow()
    Windows["Countdown"] = countdownWindow

    -- Combat timer
    if Settings.CTEnabled then 
        combatTimer = Timers.CombatWindow() 
        Windows["Combat Timer"] = combatTimer
    end

    -- Alert, dnf tracker and effects
    -- ALert window

    if Settings.AEnabled then 
        rezTracker = DNFTracker() 
        Windows["DNF Tracker"] = rezTracker
    end

    --EffectsWindow

    EventWindows = Alerts.EventWindowsHandler()

    -- Skill & boss timers
    -- Boss Timers

    if Settings.SPEnabled then 
        skillTimers = SkillTools.SkillTimers(0)
        Windows["Cooldown Timers"] = skillTimers
    end 

    if Settings.SPEnabled then 
        activeTimers = SkillTools.SkillTimers(1) 
        Windows["Active Timers"] = activeTimers
    end

    SkillUsageTracker = SkillTools.SkillTrackers(GroupSize <= 3)
    Windows["Skills Tracker"] = SkillUsageTracker

    -- Note Window
    noteWindow = NoteWindow()
    Windows["Note Window"] = noteWindow

    -- Leader & Log Window
    if Settings.LWEnabled then 
        leaderWindow = LeaderWindow() 
        Windows["Leader Window"] = leaderWindow
    end

    if Settings.LEnabled then 
        logWindow = LogWindow() 
        Windows["Log Window"] = logWindow
    end 

    -- Loot, Stat window & dungeon window
    if Settings.LOEnabled then 
        lootWindow = LootWindow() 
        Windows["Loot Window"] = lootWindow
    end
    Statwindow = nil --Stats()
    DungeonWindow = nil --DungeonLog()

    -- Options button & reload windows
    reloadWindow = ReloadWindow()
    Windows["Reload Window"] = reloadWindow

    optionsButton = OptionsButton()
    Windows["Options Button"] = optionsButton

    MasterOptions = nil

    -- Skill window
    --skillWindow = SkillWindow()
    --Windows["Skill Window"] = skillWindow

    -- Improved ready check windows
    ReadyHandler = ReadyCheckHandler()
    Windows["Ready Handler"] = ReadyHandler

    RButton = ReloadButton()
    Windows["Reload Button"] = RButton


    Patches = nil --PatchNotesWindow()

    -- Sound player
    Sounds = SoundPlayer()
    Windows["Sound Player"] = Sounds

    -- Swap window
    SwappyWarner = SwapWarner()
    Windows["Swap Warner"] = SwappyWarner

        -- Role Handler
    Roles = Handlers.RoleHandler()
    
    PluginsManager = PluginManager()
    Windows["Plugins Manager"] = PluginsManager

    
    -- Target Window
    if Settings.TEnabled then 
        Target = TargetWindow()
        Windows["Target Window"] = Target
    end

    Auras = AuraCollection.AuraHandler()
    Windows["Aura Handler"] = Auras


    -- Instance Hanlder
    InstanceController = InstanceHub()
    Windows["Instance Controller"] = InstanceController

    -- Skill Alert Timers
    if Settings.SKAEnabled or Settings.SKTEnabled then 
        PVPCDTracker = PvpTimers()
        Windows["PVP CD Tracker"] = PVPCDTracker
    end

    -- Target Box
    --if Settings.TBEnabled then 
    --    targetBox = TargetBox(GroupSize <= 3)
    --    Windows["Target Box"] = targetBox
    --end

    -- Create buff windows from saved settings
    HandleBuffWindows()

    -- Handle reloading when notes and countdown exists
    if ReloadStateData ~= nil then
        if ReloadStateData.Note ~= nil then
            if ReloadStateData.Note ~= "" then 
                noteWindow.noteWindow:SetText(ReloadStateData.Note)
                ChatOutput("<rgb=#004d00>Note window repopulated after restart, type /prime noteClear to clear the note window if this is needed.</rgb>")
            end
        end
        local cd = ReloadStateData.Countdown
        if ReloadStateData.Countdown ~= nil and cd > 0 then
            countdownWindow:Countdown(cd)
        end
    end

    -- Load the plugin manager options
    import "PrimePlugins.RaidTools.Options"

    -- Setup all group events
    --SetTriggers() -- TODO ADD NEW BINDING

    -- Initialize commands
    SetCommands()

    if Settings.SkillbarPopups then
        SkillPopups =  SkillTools.SkillPopupWindow()
    end

    -- Class tools
    ClassTools = ClassSpecificTools.ClassToolHandler()
    Windows["Class Tools"] = ClassTools

    -- Create skill overlays from data
    SkillsHandler = SkillTools.SkillBarHandler()
    Windows["SkillBarHandler"] = SkillsHandler
    if Settings.SkillOverlayToken == 1 then
        SkillsHandler:ConvertLegacySave()
        Settings.SkillOverlayToken = 2
    else
        SkillsHandler:CreateSkillBarFromDictionary()
    end

    -- Handle first time use current version
    Setup = nil
    if Settings.FirstTime then
        Setup = FirstTimerOptions()
        Windows["FirstTimeSetup"] = Setup
    end

    DataGrabberTool = DataGrab()

    -- Import test env
    --import "PrimePlugins.RaidTools.TestFile"

    -- Set back to normal state
    Settings.ReloadData = false

    self.Hide = false

    self.ShowState = {}

    self.KeyDown = function(sender, args)
        if args.Action == 268435635 then
            if self.Hide then 
                self:ShowWindows()
            else
                self:HideWindows()
            end
        end
    end
end

function WindowHandler:ShowWindows()
    self.Hide = false
    for _, value in pairs(self.ShowState) do
        value:SetVisible(true)
    end
    self.ShowState = {}
end

function WindowHandler:HideWindows()
    self.Hide = true
    for key, value in pairs(Windows) do
        if value:IsVisible() then
            self.ShowState[key] = value
        end 
        value:SetVisible(false)
    end
end

function WindowHandler:CreatePatchWindow()
    if not(Patches) then
        Patches = PatchNotesWindow()
    else
        Patches:SetVisible(true)
    end
end

function WindowHandler:UnloadWindowEvents()
    Windows["PlayerEffectHandler"]:RemoveCallbacks()
    Windows["GroupEffectsHandler"]:RemoveCallbacks()
    Windows["ChatEventHandler"]:RemoveCallbacks()
    Windows["Swap Warner"]:RemoveCallbacks()
    --Windows["Skill Window"]:RemoveCallbacks()
    if Windows["Target Window"] then Windows["Target Window"]:RemoveCallbacks() end
    --if Windows["Target Box"] then Windows["Target Box"]:RemoveCallbacks() end
    if Windows["Combat Timer"] then Windows["Combat Timer"]:RemoveCallbacks() end

    Windows["Aura Handler"]:Close()
    Windows["Class Tools"]:Unload()
    Windows["Instance Controller"]:Unload()
    DataGrabberTool:Save()
    for key, value in pairs(BuffWindowObjects) do
        value:Unload()
    end

    if messageWindow then messageWindow:Unload() end
    if effectsWindow then effectsWindow:Unload() end
    if PVPCDTracker then PVPCDTracker:Unload() end
    EventWindows:Unload()
    SkillEventHandler:RemoveCallbacks()
end

function HandleBuffWindows()
    BuffWindows = LoadTable(Turbine.DataScope.Character, "PrimeBuffWindows")
    
    BuffWindowObjects = {}

    if BuffWindows == nil then
        BuffWindows = {}
        return 
    end

    for key, value in pairs(BuffWindows) do
        local buff = SkillTools.BuffTracker(value)
        BuffWindowObjects[key] = buff
    end
end

WindowsHandlerController = WindowHandler()

import "PrimePlugins.RaidTools.MoveHandler"