OptionOptions = class(Turbine.UI.Window)
function OptionOptions:Constructor()
    local width = 1100
    local height = 570
    local x = 500 - 125
    local y = 200
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Window.Constructor(self);
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetPosition(x,y);
    self:SetWantsKeyEvents(true)
    self:SetText("Option Options")
    self:SetVisible(true)
    self:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

    self.reloadStatus = false

    self.VisibleChanged = function()
        if self.reloadStatus then ReloadPlugin() end
    end

    XIcon = "PrimePlugins/RaidTools/Images/X.tga"
    MoveIcon = "PrimePlugins/RaidTools/Images/Move.tga"

    -- Move logic
    self.Moving = false

    self.MouseDown = function(sender,args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end
  
    self.MouseUp = function()
        self.Moving = false
    end
  
    self.MouseMove = function(sender,args)
        if self.Moving then
            local newLeft = self:GetLeft() - (self.MoveX - args.X)
            local newTop = self:GetTop() - (self.MoveY - args.Y)
            self:SetPosition(newLeft,newTop)
        end
    end

    -- Options Container
    local ColorBackground = Turbine.UI.Control()
    ColorBackground:SetParent(self)
    ColorBackground:SetPosition(10, 10)
    ColorBackground:SetSize(1080, 550)
    ColorBackground:SetMouseVisible(false)
    ColorBackground:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    local EdgeColour = Turbine.UI.Control()
    EdgeColour:SetParent(self)
    EdgeColour:SetPosition(550 - 50, 15)
    EdgeColour:SetSize(100, 30)
    EdgeColour:SetMouseVisible(false)
    EdgeColour:SetBackColor(Turbine.UI.Color(0, 0, 0))

    local MoveButton = Turbine.UI.Label()
    MoveButton:SetParent(self)
    MoveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    MoveButton:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    MoveButton:SetFontStyle(Turbine.UI.FontStyle.Outline)
    MoveButton:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    MoveButton:SetForeColor(Turbine.UI.Color(1, 1, 1))
    MoveButton:SetText("Options")
    MoveButton:SetPosition(550 - 50 + 1, 15 + 1)
    MoveButton:SetSize(100 - 2, 30 - 2)
    MoveButton:SetMouseVisible(false)
    MoveButton:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

    -- Reload button
    local ReloadBackground = Turbine.UI.Control()
    ReloadBackground:SetParent(self)
    ReloadBackground:SetPosition(10, 10)
    ReloadBackground:SetSize(35, 35)
    ReloadBackground:SetMouseVisible(false)
    ReloadBackground:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

    local Image = Turbine.UI.Label()
    Image:SetParent(ReloadBackground)
    Image:SetPosition(0, 0)
    Image:SetSize(25, 25)
    Image:SetBackColor(Turbine.UI.Color(0, 102/255, 0))
    Image:SetWantsKeyEvents(false)
    Image:SetMouseVisible(false)

    local Button = Turbine.UI.Label()
    Button:SetParent(ReloadBackground)
    Button:SetPosition(0, 0)
    Button:SetSize(25, 25)
    Button:SetBackground(MoveIcon)
    Button:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    Button:SetBlendMode(Turbine.UI.BlendMode.Overlay)
    Button:SetWantsKeyEvents(false)
    Button:SetMouseVisible(true)

    Button.MouseClick = function()
        ReloadPlugin()
    end

    -- Close Button
    local CloseBackground = Turbine.UI.Control()
    CloseBackground:SetParent(self)
    CloseBackground:SetPosition(1100 - 45, 10)
    CloseBackground:SetSize(35, 35)
    CloseBackground:SetMouseVisible(false)
    CloseBackground:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

    local Image = Turbine.UI.Label()
    Image:SetParent(CloseBackground)
    Image:SetPosition(10, 0)
    Image:SetSize(25, 25)
    Image:SetBackColor(Turbine.UI.Color(153/255, 0, 0))
    Image:SetWantsKeyEvents(false)
    Image:SetMouseVisible(false)

    local Button = Turbine.UI.Label()
    Button:SetParent(CloseBackground)
    Button:SetPosition(10, 0)
    Button:SetSize(25, 25)
    Button:SetBackground(XIcon)
    Button:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    Button:SetBlendMode(Turbine.UI.BlendMode.Overlay)
    Button:SetWantsKeyEvents(false)
    Button:SetMouseVisible(true)

    Button.MouseClick = function()
        self:SetVisible(false)
    end

    local OptionsContainer = Turbine.UI.Control()
    OptionsContainer:SetParent(self)
    OptionsContainer:SetPosition(300, 50)
    OptionsContainer:SetSize(750, 480)
    OptionsContainer:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

    OptionsDic = {} 

    local VitalOptions = VitalOptionsWindow(OptionsContainer)
    OptionsDic["Player"] = VitalOptions

    local TargetOptions = TargetOptionsWindow(OptionsContainer)
    OptionsDic["Target"] = TargetOptions

    local GlobalOptions = GlobalOptionsWindow(OptionsContainer)
    OptionsDic["Global"] = GlobalOptions

    local ClassicOptions = ClassicOptionwsWindow(OptionsContainer)
    OptionsDic["Classic"] = ClassicOptions

    ButtonsDic = {}

    SortedNames = {}
    for key, value in pairs(OptionsDic) do table.insert( SortedNames, key ) end
    table.sort( SortedNames )

    local index = 0
    for key, value in pairs(SortedNames) do
        local EdgeColour = Turbine.UI.Control()
        EdgeColour:SetParent(self)
        EdgeColour:SetPosition(50, 50 + index * 32)
        EdgeColour:SetSize(250, 32)
        EdgeColour:SetMouseVisible(true)
        EdgeColour:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

        local MoveButton = Turbine.UI.Label()
        MoveButton:SetParent(self)
        MoveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
        MoveButton:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
        MoveButton:SetFontStyle(Turbine.UI.FontStyle.Outline)
        MoveButton:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
        MoveButton:SetForeColor(Turbine.UI.Color(1, 1, 1))
        MoveButton:SetText(value)
        MoveButton:SetPosition(50 + 1, 50 + index * 32 + 1)
        MoveButton:SetSize(250 - 2, 32 - 2)
        MoveButton:SetMouseVisible(true)
        MoveButton:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))
        MoveButton.MouseClick = function()
            for button, _ in pairs(ButtonsDic) do
                button:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))
            end
            MoveButton:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

            for _, optionPage in pairs(OptionsDic) do
                optionPage:SetVisible(false)
            end
            OptionsDic[value]:SetVisible(true)
        end

        ButtonsDic[MoveButton] = true

        index = index + 1
    end
end

function OptionOptions:ChangesMade()
    self.reloadStatus = true
end

VitalOptionsWindow = class(Turbine.UI.Control)
function VitalOptionsWindow:Constructor(parent)
    local width = 750
    local height = 500
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Control.Constructor(self);
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetParent(parent)
    self:SetPosition(0,0);
    self:SetWantsKeyEvents(true)
    self:SetVisible(false)

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(self)
    SingleBackground:SetPosition(90 - 2, 40 - 2)
    SingleBackground:SetSize(420 + 4, 420 + 4)
    SingleBackground:SetBackColor(Turbine.UI.Color(0, 0, 0))

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(self)
    SingleBackground:SetPosition(90, 40)
    SingleBackground:SetSize(420, 420)
    SingleBackground:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    local DescriptionLabel = Turbine.UI.Label()
    DescriptionLabel:SetParent(self)
    DescriptionLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
    DescriptionLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    DescriptionLabel:SetText("Most UI options will also affect target vitals UI when enabled")
    DescriptionLabel:SetPosition(90, 40)
    DescriptionLabel:SetSize(420, 30)

    local EffectBox1 = Turbine.UI.Lotro.CheckBox();
    EffectBox1:SetParent(SingleBackground)
    EffectBox1:SetPosition(10, 50);
    EffectBox1:SetSize(200,20);
    EffectBox1:SetChecked(Settings.EffectVersion == 1)
    EffectBox1:SetText("Lotro Effect Style");
    EffectBox1:SetVisible(true);

    local EffectBox2 = Turbine.UI.Lotro.CheckBox();
    EffectBox2:SetParent(SingleBackground)
    EffectBox2:SetPosition(10, 70);
    EffectBox2:SetSize(200,20);
    EffectBox2:SetChecked(Settings.EffectVersion == 2)
    EffectBox2:SetText("WoW Effect Style");
    EffectBox2:SetVisible(true);

    local EffectBox3 = Turbine.UI.Lotro.CheckBox();
    EffectBox3:SetParent(SingleBackground)
    EffectBox3:SetPosition(210, 90);
    EffectBox3:SetSize(200,20);
    EffectBox3:SetChecked(Settings.EffectVersion == 3)
    EffectBox3:SetText("No Effect Tracking");
    EffectBox3:SetVisible(true);

    EffectBox1.CheckedChanged = function()
        if EffectBox1:IsChecked() then
            Settings.EffectVersion = 1
            EffectBox2:SetChecked(false)
            EffectBox3:SetChecked(false)
        end
        Options:ChangesMade()
    end

    EffectBox2.CheckedChanged = function()
        if EffectBox2:IsChecked() then
            Settings.EffectVersion = 2
            EffectBox1:SetChecked(false)
            EffectBox3:SetChecked(false)
        end
        Options:ChangesMade()
    end

    EffectBox3.CheckedChanged = function()
        if EffectBox3:IsChecked() then
            Settings.EffectVersion = 3
            EffectBox1:SetChecked(false)
            EffectBox2:SetChecked(false)
        end
        Options:ChangesMade()
    end

    local EffectBox = Turbine.UI.Lotro.CheckBox();
    EffectBox:SetParent(SingleBackground)
    EffectBox:SetPosition(10, 110);
    EffectBox:SetSize(200,20);
    EffectBox:SetChecked(Settings.ShowPowerNumbers)
    EffectBox:SetText("Show power numbers");
    EffectBox:SetVisible(true);

    EffectBox.CheckedChanged = function()
        Settings.ShowPowerNumbers = EffectBox:IsChecked()
        Options:ChangesMade()
    end

    local EffectBox = Turbine.UI.Lotro.CheckBox();
    EffectBox:SetParent(SingleBackground)
    EffectBox:SetPosition(10, 130);
    EffectBox:SetSize(200,20);
    EffectBox:SetChecked(Settings.ShowName)
    EffectBox:SetText("Show name");
    EffectBox:SetVisible(true);

    EffectBox.CheckedChanged = function()
        Settings.ShowName = EffectBox:IsChecked()
        Options:ChangesMade()
    end

    local EffectBox = Turbine.UI.Lotro.CheckBox();
    EffectBox:SetParent(SingleBackground)
    EffectBox:SetPosition(10, 150);
    EffectBox:SetSize(200,20);
    EffectBox:SetChecked(Settings.OnlyShowCurrent)
    EffectBox:SetText("Only show current amount");
    EffectBox:SetVisible(true);

    EffectBox.CheckedChanged = function()
        Settings.OnlyShowCurrent = EffectBox:IsChecked()
        Options:ChangesMade()
    end


    local ScaleBox = Turbine.UI.Lotro.TextBox()
    ScaleBox:SetParent(SingleBackground)
    ScaleBox:SetSize(50, 30)
    ScaleBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    ScaleBox:SetPosition(110, 175)
    ScaleBox:SetText(Settings.IconScale)

    local ScaleButton = Turbine.UI.Lotro.Button()
    ScaleButton:SetParent(SingleBackground)
    ScaleButton:SetSize(100, 30)
    ScaleButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    ScaleButton:SetMouseVisible(false)
    ScaleButton:SetPosition(10, 180)
    ScaleButton:SetText("Set Scale")

    ScaleButton.Click = function()
        if ScaleBox:GetText() ~= "" and ScaleBox:GetText() ~= nil and ScaleBox:GetText() ~= Settings.SkillTimerScale then
            Settings.IconScale = tonumber(ScaleBox:GetText())
            Options:ChangesMade()
        end
    end

    local EffectBox = Turbine.UI.Lotro.CheckBox();
    EffectBox:SetParent(SingleBackground)
    EffectBox:SetPosition(10, 200);
    EffectBox:SetSize(200,20);
    EffectBox:SetChecked(Settings.ShowCountdown)
    EffectBox:SetText("Show duration on effects");
    EffectBox:SetVisible(true);

    EffectBox.CheckedChanged = function()
        Settings.ShowCountdown = EffectBox:IsChecked()
        Options:ChangesMade()
    end

    local EffectBox5 = Turbine.UI.Lotro.CheckBox();
    EffectBox5:SetParent(SingleBackground)
    EffectBox5:SetPosition(10, 240);
    EffectBox5:SetSize(200,20);
    EffectBox5:SetChecked(Settings.ColourMode == 1)
    EffectBox5:SetText("Class colours");
    EffectBox5:SetVisible(true);

    local EffectBox6 = Turbine.UI.Lotro.CheckBox();
    EffectBox6:SetParent(SingleBackground)
    EffectBox6:SetPosition(10, 260);
    EffectBox6:SetSize(200,20);
    EffectBox6:SetChecked(Settings.ColourMode == 2)
    EffectBox6:SetText("Base colours");
    EffectBox6:SetVisible(true);

    
    local EffectBox6a = Turbine.UI.Lotro.CheckBox();
    EffectBox6a:SetParent(SingleBackground)
    EffectBox6a:SetPosition(150, 260);
    EffectBox6a:SetSize(200,20);
    EffectBox6a:SetChecked(Settings.ColourMode == 3)
    EffectBox6a:SetText("Gray colours");
    EffectBox6a:SetVisible(true);

    EffectBox5.CheckedChanged = function()
        if EffectBox5:IsChecked() then
            Settings.ColourMode = 1
            EffectBox6:SetChecked(false)
            EffectBox6a:SetChecked(false)
        end
        Options:ChangesMade()
    end

    EffectBox6.CheckedChanged = function()
        if EffectBox6:IsChecked() then
            Settings.ColourMode = 2
            EffectBox5:SetChecked(false)
            EffectBox6a:SetChecked(false)
        end
        Options:ChangesMade()
    end

    EffectBox6a.CheckedChanged = function()
        if EffectBox6a:IsChecked() then
            Settings.ColourMode = 3
            EffectBox5:SetChecked(false)
            EffectBox6:SetChecked(false)
        end
        Options:ChangesMade()
    end

    local EffectBox7 = Turbine.UI.Lotro.CheckBox();
    EffectBox7:SetParent(SingleBackground)
    EffectBox7:SetPosition(10, 300);
    EffectBox7:SetSize(200,20);
    EffectBox7:SetChecked(Settings.Layout == 1)
    EffectBox7:SetText("Big layout");
    EffectBox7:SetVisible(true);

    local EffectBox8 = Turbine.UI.Lotro.CheckBox();
    EffectBox8:SetParent(SingleBackground)
    EffectBox8:SetPosition(10, 320);
    EffectBox8:SetSize(200,20);
    EffectBox8:SetChecked(Settings.Layout == 2)
    EffectBox8:SetText("Slim layout");
    EffectBox8:SetVisible(true);

    local EffectBox8a = Turbine.UI.Lotro.CheckBox();
    EffectBox8a:SetParent(SingleBackground)
    EffectBox8a:SetPosition(150, 320);
    EffectBox8a:SetSize(200,20);
    EffectBox8a:SetChecked(Settings.Layout == 3)
    EffectBox8a:SetText("Slim split layout");
    EffectBox8a:SetVisible(true);

    EffectBox7.CheckedChanged = function()
        if EffectBox7:IsChecked() then
            Settings.Layout = 1
            EffectBox8:SetChecked(false)
            EffectBox8a:SetChecked(false)
        end
        Options:ChangesMade()
    end

    EffectBox8.CheckedChanged = function()
        if EffectBox8:IsChecked() then
            Settings.Layout = 2
            EffectBox7:SetChecked(false)
            EffectBox8a:SetChecked(false)
        end
        Options:ChangesMade()
    end

    EffectBox8a.CheckedChanged = function()
        if EffectBox8a:IsChecked() then
            Settings.Layout = 3
            EffectBox7:SetChecked(false)
            EffectBox8:SetChecked(false)
        end
        Options:ChangesMade()
    end

    local EffectBox9 = Turbine.UI.Lotro.CheckBox();
    EffectBox9:SetParent(SingleBackground)
    EffectBox9:SetPosition(10, 360);
    EffectBox9:SetSize(200,20);
    EffectBox9:SetChecked(Settings.Text == 1)
    EffectBox9:SetText("Text Top/Bottom");
    EffectBox9:SetVisible(true);

    local EffectBox10 = Turbine.UI.Lotro.CheckBox();
    EffectBox10:SetParent(SingleBackground)
    EffectBox10:SetPosition(10, 380);
    EffectBox10:SetSize(200,20);
    EffectBox10:SetChecked(Settings.Text == 2)
    EffectBox10:SetText("Text Left/Right");
    EffectBox10:SetVisible(true);

    EffectBox9.CheckedChanged = function()
        if EffectBox9:IsChecked() then
            Settings.Text = 1
            EffectBox10:SetChecked(false)
        end
        Options:ChangesMade()
    end

    EffectBox10.CheckedChanged = function()
        if EffectBox10:IsChecked() then
            Settings.Text = 2
            EffectBox9:SetChecked(false)
        end
        Options:ChangesMade()
    end

    local EffectBox11 = Turbine.UI.Lotro.CheckBox();
    EffectBox11:SetParent(SingleBackground)
    EffectBox11:SetPosition(10, 400);
    EffectBox11:SetSize(400,20);
    EffectBox11:SetChecked(Settings.ResourceText)
    EffectBox11:SetText("Show Resource Number (Focus/Fervour/etc)");
    EffectBox11:SetVisible(true);

    EffectBox11.CheckedChanged = function()
        Settings.ResourceText = EffectBox11:IsChecked()
        Options:ChangesMade()
    end

    local EffectBox12 = Turbine.UI.Lotro.CheckBox();
    EffectBox12:SetParent(SingleBackground)
    EffectBox12:SetPosition(210, 110);
    EffectBox12:SetSize(200,20);
    EffectBox12:SetChecked(Settings.BuffTracking)
    EffectBox12:SetText("Buff Tracking");
    EffectBox12:SetVisible(true);

    EffectBox12.CheckedChanged = function()
        Settings.BuffTracking = EffectBox12:IsChecked()
        Options:ChangesMade()
    end

    local EffectBox13 = Turbine.UI.Lotro.CheckBox();
    EffectBox13:SetParent(SingleBackground)
    EffectBox13:SetPosition(210, 130);
    EffectBox13:SetSize(200,20);
    EffectBox13:SetChecked(Settings.DebuffTracking)
    EffectBox13:SetText("Debuff Tracking");
    EffectBox13:SetVisible(true);

    EffectBox13.CheckedChanged = function()
        Settings.DebuffTracking = EffectBox13:IsChecked()
        Options:ChangesMade()
    end

    local EffectBox14 = Turbine.UI.Lotro.CheckBox();
    EffectBox14:SetParent(SingleBackground)
    EffectBox14:SetPosition(150, 240);
    EffectBox14:SetSize(400,20);
    EffectBox14:SetChecked(Settings.ColourChanging)
    EffectBox14:SetText("Change colour based on health");
    EffectBox14:SetVisible(true);

    EffectBox14.CheckedChanged = function()
        Settings.ColourChanging = EffectBox14:IsChecked()
        Options:ChangesMade()
    end
end



TargetOptionsWindow = class(Turbine.UI.Control)
function TargetOptionsWindow:Constructor(parent)
    local width = 750
    local height = 500
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Control.Constructor(self);
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetParent(parent)
    self:SetPosition(0,0);
    self:SetWantsKeyEvents(true)
    self:SetVisible(false)

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(self)
    SingleBackground:SetPosition(90 - 2, 40 - 2)
    SingleBackground:SetSize(420 + 4, 420 + 4)
    SingleBackground:SetBackColor(Turbine.UI.Color(0, 0, 0))

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(self)
    SingleBackground:SetPosition(90, 40)
    SingleBackground:SetSize(420, 420)
    SingleBackground:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    local DescriptionLabel = Turbine.UI.Label()
    DescriptionLabel:SetParent(self)
    DescriptionLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    DescriptionLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    DescriptionLabel:SetText("Base Settings")
    DescriptionLabel:SetPosition(90, 40)
    DescriptionLabel:SetSize(220, 30)

    local EffectBox = Turbine.UI.Lotro.CheckBox();
    EffectBox:SetParent(SingleBackground)
    EffectBox:SetPosition(10, 50);
    EffectBox:SetSize(200,20);
    EffectBox:SetChecked(Settings.TargetEnabled)
    EffectBox:SetText("Enabled");
    EffectBox:SetVisible(true);

    EffectBox.CheckedChanged = function()
        Settings.TargetEnabled = EffectBox:IsChecked()
        Options:ChangesMade()
    end

    local DescriptionLabel = Turbine.UI.Label()
    DescriptionLabel:SetParent(SingleBackground)
    DescriptionLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
    DescriptionLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    DescriptionLabel:SetText("Effect tracking on targets is not accurate due to API limitations and might impact client performance")
    DescriptionLabel:SetPosition(10, 70)
    DescriptionLabel:SetSize(370, 40)

    local EffectBox1 = Turbine.UI.Lotro.CheckBox();
    EffectBox1:SetParent(SingleBackground)
    EffectBox1:SetPosition(10, 110);
    EffectBox1:SetSize(700,20);
    EffectBox1:SetChecked(Settings.TargetBuffTracking)
    EffectBox1:SetText("Effect tracking ");
    EffectBox1:SetVisible(true);

    EffectBox1.CheckedChanged = function()
        Settings.TargetBuffTracking = EffectBox1:IsChecked()
        Options:ChangesMade()
    end

    local EffectBox2 = Turbine.UI.Lotro.CheckBox();
    EffectBox2:SetParent(SingleBackground)
    EffectBox2:SetPosition(10, 130);
    EffectBox2:SetSize(700,20);
    EffectBox2:SetChecked(Settings.BuffTracking)
    EffectBox2:SetText("Buff tracking ");
    EffectBox2:SetVisible(true);

    EffectBox2.CheckedChanged = function()
        Settings.BuffTracking = EffectBox2:IsChecked()
        Options:ChangesMade()
    end

    local EffectBox3 = Turbine.UI.Lotro.CheckBox();
    EffectBox3:SetParent(SingleBackground)
    EffectBox3:SetPosition(10, 150);
    EffectBox3:SetSize(700,20);
    EffectBox3:SetChecked(Settings.DebuffTracking)
    EffectBox3:SetText("Debuff tracking ");
    EffectBox3:SetVisible(true);

    EffectBox3.CheckedChanged = function()
        Settings.DebuffTracking = EffectBox3:IsChecked()
        Options:ChangesMade()
    end



end

GlobalOptionsWindow = class(Turbine.UI.Control)
function GlobalOptionsWindow:Constructor(parent)
    local width = 750
    local height = 500
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Control.Constructor(self);
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetParent(parent)
    self:SetPosition(0,0);
    self:SetWantsKeyEvents(true)
    self:SetVisible(false)

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(self)
    SingleBackground:SetPosition(90 - 2, 40 - 2)
    SingleBackground:SetSize(420 + 4, 420 + 4)
    SingleBackground:SetBackColor(Turbine.UI.Color(0, 0, 0))

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(self)
    SingleBackground:SetPosition(90, 40)
    SingleBackground:SetSize(420, 420)
    SingleBackground:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    local DescriptionLabel = Turbine.UI.Label()
    DescriptionLabel:SetParent(self)
    DescriptionLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    DescriptionLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    DescriptionLabel:SetText("Base Settings")
    DescriptionLabel:SetPosition(90, 40)
    DescriptionLabel:SetSize(220, 30)

    local EffectBox = Turbine.UI.Lotro.CheckBox();
    EffectBox:SetParent(SingleBackground)
    EffectBox:SetPosition(10, 50);
    EffectBox:SetSize(200,20);
    EffectBox:SetChecked(Settings.PercentageShown)
    EffectBox:SetText("Percentage");
    EffectBox:SetVisible(true);

    EffectBox.CheckedChanged = function()
        Settings.PercentageShown = EffectBox:IsChecked()
        Options:ChangesMade()
    end

    local EffectBox1 = Turbine.UI.Lotro.CheckBox();
    EffectBox1:SetParent(SingleBackground)
    EffectBox1:SetPosition(10, 90);
    EffectBox1:SetSize(200,20);
    EffectBox1:SetChecked(Settings.NumberMethod == 1)
    EffectBox1:SetText("No number formatting");
    EffectBox1:SetVisible(true);

    local EffectBox2 = Turbine.UI.Lotro.CheckBox();
    EffectBox2:SetParent(SingleBackground)
    EffectBox2:SetPosition(10, 110);
    EffectBox2:SetSize(200,20);
    EffectBox2:SetChecked(Settings.NumberMethod == 2)
    EffectBox2:SetText("Normal number formatting");
    EffectBox2:SetVisible(true);

    EffectBox1.CheckedChanged = function()
        if EffectBox1:IsChecked() then
            Settings.NumberMethod = 1
            EffectBox2:SetChecked(false)
        end
        Options:ChangesMade()
    end

    EffectBox2.CheckedChanged = function()
        if EffectBox2:IsChecked() then
            Settings.NumberMethod = 2
            EffectBox1:SetChecked(false)
        end
        Options:ChangesMade()
    end

    local EffectBox3 = Turbine.UI.Lotro.CheckBox();
    EffectBox3:SetParent(SingleBackground)
    EffectBox3:SetPosition(10, 150);
    EffectBox3:SetSize(200,20);
    EffectBox3:SetChecked(Settings.Aura)
    EffectBox3:SetText("Blood aura");
    EffectBox3:SetVisible(true);

    EffectBox3.CheckedChanged = function()
        Settings.Aura = EffectBox3:IsChecked()
        Options:ChangesMade()
    end

end

ClassicOptionwsWindow = class(Turbine.UI.Control)
function ClassicOptionwsWindow:Constructor(parent)
    local width = 750
    local height = 500
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Control.Constructor(self);
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetParent(parent)
    self:SetPosition(0,0);
    self:SetWantsKeyEvents(true)
    self:SetVisible(false)

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(self)
    SingleBackground:SetPosition(90 - 2, 40 - 2)
    SingleBackground:SetSize(420 + 4, 420 + 4)
    SingleBackground:SetBackColor(Turbine.UI.Color(0, 0, 0))

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(self)
    SingleBackground:SetPosition(90, 40)
    SingleBackground:SetSize(420, 420)
    SingleBackground:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    local DescriptionLabel = Turbine.UI.Label()
    DescriptionLabel:SetParent(self)
    DescriptionLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    DescriptionLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    DescriptionLabel:SetText("Base Settings")
    DescriptionLabel:SetPosition(90, 40)
    DescriptionLabel:SetSize(220, 30)

    local EffectBox = Turbine.UI.Lotro.CheckBox();
    EffectBox:SetParent(SingleBackground)
    EffectBox:SetPosition(10, 50);
    EffectBox:SetSize(200,20);
    EffectBox:SetChecked(Settings.ClassicEnabled)
    EffectBox:SetText("Enable");
    EffectBox:SetVisible(true);

    EffectBox.CheckedChanged = function()
        Settings.ClassicEnabled = EffectBox:IsChecked()
        Options:ChangesMade()
    end

    --Settings.ClassicPVP = nil
    --Settings.ClassicBackground = nil
    --Settings.ClassicCover = nil

    local EffectBox1 = Turbine.UI.Lotro.CheckBox();
    EffectBox1:SetParent(SingleBackground)
    EffectBox1:SetPosition(10, 90);
    EffectBox1:SetSize(200,20);
    EffectBox1:SetChecked(Settings.ClassicOnlyShowCurrent)
    EffectBox1:SetText("Only show current value");
    EffectBox1:SetVisible(true);

    EffectBox1.CheckedChanged = function()
        Settings.ClassicOnlyShowCurrent = EffectBox1:IsChecked()
        Options:ChangesMade()
    end

    local EffectBox2 = Turbine.UI.Lotro.CheckBox();
    EffectBox2:SetParent(SingleBackground)
    EffectBox2:SetPosition(10, 110);
    EffectBox2:SetSize(200,20);
    EffectBox2:SetChecked(Settings.ClassicPercentage)
    EffectBox2:SetText("Show percentage");
    EffectBox2:SetVisible(true);

    EffectBox2.CheckedChanged = function()
        Settings.ClassicPercentage = EffectBox2:IsChecked()
        Options:ChangesMade()
    end

    -- pvp icon
    local PvpButton = Turbine.UI.Lotro.Button()
    PvpButton:SetParent(SingleBackground)
    PvpButton:SetSize(300, 15)
    PvpButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    PvpButton:SetMouseVisible(true)
    PvpButton:SetPosition(100, 200)
    PvpButton:SetText("PVP icon: " .. tostring(Settings.ClassicPVP))
    
    local menu = Turbine.UI.ContextMenu()
    local menuItems = menu:GetItems()

    local sortedKeys2 = {}
    for key, value in pairs(PvpIcons) do table.insert( sortedKeys2, key ) end
    table.sort( sortedKeys2 )
    
    for key, name in pairs(sortedKeys2) do
        local menuItem = Turbine.UI.MenuItem( name )

        menuItem.Click = function()
            Settings.ClassicPVP = name
            PvpButton:SetText("PVP icon: " .. tostring(name))
        end
        
        menuItems:Add(menuItem)
    end

    PvpButton.Click = function()
        menu:ShowMenu()
    end

    -- Character cover
    local CoverButton = Turbine.UI.Lotro.Button()
    CoverButton:SetParent(SingleBackground)
    CoverButton:SetSize(300, 15)
    CoverButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    CoverButton:SetMouseVisible(true)
    CoverButton:SetPosition(100, 220)
    CoverButton:SetText("Cover: " .. tostring(Settings.ClassicCover))
    
    local menu = Turbine.UI.ContextMenu()
    local menuItems = menu:GetItems()

    local sortedKeys2 = {}
    for key, value in pairs(Covers) do table.insert( sortedKeys2, key ) end
    table.sort( sortedKeys2 )
    
    for key, name in pairs(sortedKeys2) do
        local menuItem = Turbine.UI.MenuItem( name )

        menuItem.Click = function()
            Settings.ClassicCover = name
            CoverButton:SetText("Cover: " .. tostring(name))
        end
        
        menuItems:Add(menuItem)
    end

    CoverButton.Click = function()
        menu:ShowMenu()
    end

    -- Vital background
    local VitalButton = Turbine.UI.Lotro.Button()
    VitalButton:SetParent(SingleBackground)
    VitalButton:SetSize(300, 15)
    VitalButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    VitalButton:SetMouseVisible(true)
    VitalButton:SetPosition(100, 240)
    VitalButton:SetText("Background: " .. tostring(Settings.ClassicBackground))
    
    local menu = Turbine.UI.ContextMenu()
    local menuItems = menu:GetItems()

    local sortedKeys2 = {}
    for key, value in pairs(Backgrounds) do table.insert( sortedKeys2, key ) end
    table.sort( sortedKeys2 )
    
    for key, name in pairs(sortedKeys2) do
        local menuItem = Turbine.UI.MenuItem( name )

        menuItem.Click = function()
            Settings.ClassicBackground = name
            VitalButton:SetText("Background: " .. tostring(name))
        end
        
        menuItems:Add(menuItem)
    end

    VitalButton.Click = function()
        menu:ShowMenu()
    end

end

plugin.GetOptionsPanel = function( self )

    local options = Turbine.UI.Control()
    options:SetSize(500, 400)

    local MasterOptionsButton = Turbine.UI.Lotro.Button()
    MasterOptionsButton:SetParent(options)
    MasterOptionsButton:SetPosition(100, 100);
    MasterOptionsButton:SetSize(200,15);
    MasterOptionsButton:SetText("Open options panel");
    MasterOptionsButton:SetVisible(true);

    MasterOptionsButton.Click = function()
        if Options == nil then
            Options = OptionOptions()
        else
            Options:SetVisible(not(Options:IsVisible()))
        end
    end

    return options;
end