EventWindowsHandler = class(Turbine.UI.Control)
function EventWindowsHandler:Constructor()
    Turbine.UI.Control.Constructor(self);

    -- Initialize all data
    self:LoadChatData()
    self:LoadSelfEffectData()
    self:LoadGroupEffectData()
    self:LoadChatPlayerCombatData()
    self:LoadChatEnemyCombatData()

    self.playerName = Turbine.Gameplay.LocalPlayer.GetInstance():GetName()

    self.Role = 0

    self.AlertWindows = {}
    self.EffectWindows = {}
    self.TimerWindows = {}
    self.addedCallbacks = {}

    self.chatCallback = {}

    self.Group = GroupEffectsHandler:GetGroup()

    -- Intialize base windows
    if Settings.AEnabled then 
        messageWindow = AlertWindow(Settings.AWidth, Settings.AHeight, Settings.AX, Settings.AY, Settings.ABaseHeight)
        self.AlertWindows[1] = messageWindow
        Windows["Alert Window"] = messageWindow
    end 

    if Settings.EEnabled then 
        effectsWindow = EffectsWindow() 
        self.EffectWindows[1] = effectsWindow
        Windows["Effect Window"] = effectsWindow
    end 

    if Settings.SEnabled then 
        skillCounters = Timers.SkillCounters()
        self.TimerWindows[1] = skillCounters
        Windows["Boss Timers"] = skillCounters
    end 

    -- Initialize additional windows

    -- INitialize chat callbacks
    self.ChatAlert = function(message)
        local name = GetFirstName(message)
        local cleanedLine = GetTextLine(message)

        -- Chat alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 timerWindowID}
        local phraseTable = self.AllChatData[name]
        if phraseTable then 
            for phrase, alertTable in pairs(phraseTable) do
                if string.find(cleanedLine, phrase) then
                    local restrict = alertTable["Restrictions"]
                    -- Double check if we want the alert
                    if restrict and (not(restrict[1]) or restrict[1] == Settings.InstanceVersion) and (not(restrict[2]) or restrict[2] == self.Role) then
                        local alert = alertTable["Alerts"][1]
                        local timer = alertTable["Timers"][1]
                        local timerID = timer[6]
                        if not(timer[6]) then timerID = 1 end
                        -- Alert specific
                        if alert[1] then
                            self.AlertWindows[1]:Message(alert[1], alert[2], alert[3], alert[4])
                        end

                        -- Sound handling
                        if alert[5] then Sounds:PlaySoundEffect(alert[5]) end

                        -- Timer Handling
                        if timer[1] then
                            self.TimerWindows[timerID]:NewTimer(timer[1], timer[2], alert[4], nil, timer[5], timer[3])
                        end
                    end
                end
            end
        end
    end

    self.ChatPlayerCombatAlert = function(skillname)
        -- Chat alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 timerWindowID}
        local alertTable = self.AllChatPlayerCombatData[skillname]
        if alertTable then 
            local restrict = alertTable["Restrictions"]
            -- Double check if we want the alert
            if restrict and (not(restrict[1]) or restrict[1] == Settings.InstanceVersion) and (not(restrict[2]) or restrict[2] == self.Role) then
                local alert = alertTable["Alerts"][1]
                local timer = alertTable["Timers"][1]
                local timerID = timer[6]
                if not(timer[6]) then timerID = 1 end
                -- Alert specific
                if alert[1] then
                    self.AlertWindows[1]:Message(alert[1], alert[2], alert[3], alert[4])
                end

                -- Sound handling
                if alert[5] then Sounds:PlaySoundEffect(alert[5]) end

                -- Timer Handling
                if timer[1] then
                    self.TimerWindows[timerID]:NewTimer(timer[1], timer[2], timer[4], nil, timer[5], timer[3])
                end
            end
        end
    end

    self.ChatEnemyCombatAlert = function(skillname)
        -- Chat alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 timerWindowID}
        local alertTable = self.AllChatEnemyCombatData[skillname]
        if alertTable then 
            local restrict = alertTable["Restrictions"]
            -- Double check if we want the alert
            if restrict and (not(restrict[1]) or restrict[1] == Settings.InstanceVersion) and (not(restrict[2]) or restrict[2] == self.Role) then
                local alert = alertTable["Alerts"][1]
                local timer = alertTable["Timers"][1]
                local timerID = timer[6]
                if not(timer[6]) then timerID = 1 end
                -- Alert specific
                if alert[1] then
                    self.AlertWindows[1]:Message(alert[1], alert[2], alert[3], alert[4])
                end

                -- Sound handling
                if alert[5] then Sounds:PlaySoundEffect(alert[5]) end

                -- Timer Handling
                if timer[1] then
                    self.TimerWindows[timerID]:NewTimer(timer[1], timer[2], timer[4], nil, timer[5], timer[3])
                end
            end
        end
    end

    self.chatCallback["StandardChat"] = AddCallback(ChatEventHandler, "StandardChat", self.ChatAlert)
    self.chatCallback["EnemyCombat"] = AddCallback(ChatEventHandler, "EnemyCombat", self.ChatEnemyCombatAlert)
    self.chatCallback["PlayerCombat"] = AddCallback(ChatEventHandler, "PlayerCombat", self.ChatPlayerCombatAlert)

     -- All effect added alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 5_countdown, 7 instanceID, 8 role 9 timer duration 10 timer text, 11 timer icon, 12 unique, 13 loop, 14 tiering effect}

    self.PlayerEffectWindowHandling = function(effect)
        local alertTable = nil
        
        if string.match(effect:GetName(), "%d") then
            alertTable = self.AllSelfEffectData[effect:GetName()]
            if not(alertTable) then
                alertTable = self.AllSelfEffectData[string.gsub(effect:GetName(), "%d", "")]
                if alertTable and not(alertTable["Restrictions"][3]) then return end
            end
        else
            alertTable = self.AllSelfEffectData[effect:GetName()]
        end


        if alertTable then
            local restrict = alertTable["Restrictions"]
            if restrict and (not(restrict[1]) or restrict[1] == Settings.InstanceVersion) and (not(restrict[2]) or restrict[2] == self.Role) then

                local alert = alertTable["Alerts"][1]
                local timer = alertTable["Timers"][1]

                if alert[1] and effectsWindow then
                    if alert[5] and Sounds then Sounds:PlaySoundEffect(alert[5]) end
                    effectsWindow:Message(alert[1], alert[2], alert[3], alert[4], effect:GetName(), alert[6])
                end

                if timer[1] and skillCounters then

                    local effectInput = nil
                    local icon = timer[3]
                    if timer[3] == true then 
                        if effect then 
                            effectInput = effect 
                            icon = nil
                        else
                            icon = effect:GetIcon()
                        end
                    end

                    skillCounters:NewTimer(timer[1], string.gsub(timer[2], "<name>", self.playerName), timer[4], effectInput, timer[5], icon, restrict[3], self.playerName)
                end
            end
        end
    end

    self.PlayerGroupEffectWindowHandling = function(effect)
        local alertTable = nil
        
        if string.match(effect:GetName(), "%d") then
            alertTable = self.AllGroupEffectData[effect:GetName()]
            if not(alertTable) then
                alertTable = self.AllGroupEffectData[string.gsub(effect:GetName(), "%d", "")]
                if alertTable and not(alertTable["Restrictions"][3]) then return end
            end
        else
            alertTable = self.AllGroupEffectData[effect:GetName()]
        end

        if alertTable then
            local restrict = alertTable["Restrictions"]
            if restrict and (not(restrict[1]) or restrict[1] == Settings.InstanceVersion) and (not(restrict[2]) or restrict[2] == self.Role) then

                local alert = alertTable["Alerts"][1]
                local timer = alertTable["Timers"][1]

                if alert[1] and messageWindow then
                    if alert[5] and Sounds then Sounds:PlaySoundEffect(alert[5]) end
                    local icon = alert[3]
                    if alert[3] == true and effect then
                        icon = effect:GetIcon()
                    end
                    local stackName = ""
                    if string.match(effect:GetName(), "%d") then
                        stackName = string.match(effect:GetName(), "%d") .. " - "
                    end
                    messageWindow:Message(alert[1], stackName.. alert[2] .. "<rgb=#FF0000>" .. self.playerName .. "</rgb>", icon, alert[4], restrict[3], effect, self.playerName)
                end

                if timer[1] then

                    local effectInput = nil
                    local icon = timer[3]
                    if timer[3] == true then 
                        if effect then 
                            effectInput = effect 
                            icon = nil
                        else
                            icon = effect:GetIcon()
                        end
                    end

                    skillCounters:NewTimer(timer[1], string.gsub(timer[2], "<name>", self.playerName), timer[4], effectInput, timer[5], icon, restrict[3], self.playerName)
                end
            end
        end
    end

    self.addedCallbacks["effectsWindow"] = AddCallback(PlayerEffectsHandler, "EffectAdded", self.PlayerEffectWindowHandling)
    self.addedCallbacks["groupWindow"] = AddCallback(PlayerEffectsHandler, "EffectAdded", self.PlayerGroupEffectWindowHandling)


    -----------------------------------
    -- Group
    -----------------------------------
    self.GroupEffectHandling = function(args)
        local effect = args["effect"]
        local alertTable = nil
        
        if string.match(effect:GetName(), "%d") then
            alertTable = self.AllGroupEffectData[effect:GetName()]
            if not(alertTable) then
                alertTable = self.AllGroupEffectData[string.gsub(effect:GetName(), "%d", "")]
                if alertTable and not(alertTable["Restrictions"][3]) then return end
            end
        else
            alertTable = self.AllGroupEffectData[effect:GetName()]
        end

        if alertTable then
            local restrict = alertTable["Restrictions"]
            if (not(restrict[1]) or restrict[1] == Settings.InstanceVersion) and (not(restrict[2]) or restrict[2] == self.Role) then

                local alert = alertTable["Alerts"][1]
                local timer = alertTable["Timers"][1]

                if alert[1] then
                    if alert[5] and Sounds then Sounds:PlaySoundEffect(alert[5]) end
                    local name = self.Group[args["groupID"]].player:GetName()

                    local icon = alert[3]
                    if alert[3] == true then 
                        icon = effect:GetIcon()
                    end

                    local stackName = ""
                    if string.match(effect:GetName(), "%d") then
                        stackName = string.match(effect:GetName(), "%d") .. " - "
                    end
                    messageWindow:Message(alert[1], stackName  .. alert[2] .. "<rgb=#FF0000>" .. name .. "</rgb>", icon, alert[4], restrict[3], effect, name)
                end

                if timer[1] then

                    local effectInput = nil
                    local icon = timer[3]
                    if timer[3] == true then 
                        if effect then 
                            effectInput = effect 
                            icon = nil
                        else
                            icon = effect:GetIcon()
                        end
                    end

                    skillCounters:NewTimer(timer[1], string.gsub(timer[2], "<name>", self.Group[args["groupID"]].player:GetName()), timer[11], effectInput, timer[12], icon, restrict[3], self.Group[args["groupID"]].player:GetName())
                end
            end
        end

        -- [1] Duration [2] Message [3] Icon (Special/Game/None) [4] Countdown [5] SoundEffect [6] instanceID [7] role [8] timerduration, [9] timer text, [10] timer icon, [11] unique, [12] loop, [13] tiering effect}
    end
    self.groupCallbacks = {}
    self.groupCallbacks["groupeffectsWindow"] = AddCallback(GroupEffectsHandler, "EffectAdded", self.GroupEffectHandling)
end

function EventWindowsHandler:Unload()
    SaveTable(Turbine.DataScope.Account, "PrimeChatData", self.AllChatData)
    SaveTable(Turbine.DataScope.Account, "PrimeChatPlayerCombatData", self.AllChatPlayerCombatData)
    SaveTable(Turbine.DataScope.Account, "PrimeChatEnemyCombatData", self.AllChatEnemyCombatData)
    SaveTable(Turbine.DataScope.Account, "PrimeSelfEffectData", self.AllSelfEffectData)
    SaveTable(Turbine.DataScope.Account, "PrimeGroupEffectData", self.AllGroupEffectData)
    for key, value in pairs(self.addedCallbacks) do
        RemoveCallback(PlayerEffectsHandler, "EffectAdded", value)
    end
    for key, value in pairs(self.groupCallbacks) do
        RemoveCallback(GroupEffectsHandler, "EffectAdded", value)
    end
    for key, value in pairs(self.chatCallback) do
        RemoveCallback(ChatEventHandler, key, value)
    end
end

function EventWindowsHandler:ResetData()
    ChatOutput("Resetting Alert & Timer Data")
    self.AllChatData = self:GetBaseChatData()
    ChatOutput("Reset Say Chat data")
    self.AllChatPlayerCombatData = self:GetBaseChatPlayerCombatData()
    ChatOutput("Reset Player Combat Chat data")
    self.AllChatEnemyCombatData = self:GetBaseChatEnemyCombatData()
    ChatOutput("Reset Player Enemy Chat data")
    self.AllSelfEffectData = self:GetBaseSelfEffectData()
    ChatOutput("Reset Self Effect data")
    self.AllGroupEffectData = self:GetBaseGroupEffectData()
    ChatOutput("Reset Group Effect data")
    ChatOutput("Succesfully Reset Alert & Timer Data")
    ReloadPlugin()
end

function EventWindowsHandler:UpdateBase()
    ChatOutput("Updating Alert & Timer Data")
    self:UpdateTable(self.AllChatData, self:GetBaseChatData())
    ChatOutput("Updating Say Chat data")
    self:UpdateTable(self.AllChatPlayerCombatData, self:GetBaseChatPlayerCombatData())
    ChatOutput("Updating Player Combat Chat data")
    self:UpdateTable(self.AllChatEnemyCombatData, self:GetBaseChatEnemyCombatData())
    ChatOutput("Updating Player Enemy Chat data")
    self:UpdateTable(self.AllSelfEffectData, self:GetBaseSelfEffectData())
    ChatOutput("Updating Self Effect data")
    self:UpdateTable(self.AllGroupEffectData, self:GetBaseGroupEffectData())
    ChatOutput("Updating Group Effect data")
    ChatOutput("Succesfully Updated Alert & Timer Data")
end

function EventWindowsHandler:UpdateTable(base, updating)
    for key, value in pairs(updating) do
        base[key] = value
    end
end

function EventWindowsHandler:SetRole(role)
    self.Role = role
end

-- Data loading functions
function EventWindowsHandler:LoadChatData()
    self.AllChatData = LoadTable(Turbine.DataScope.Account, "PrimeChatData")

    if not(self.AllChatData) then
        self.AllChatData = self:GetBaseChatData()
    end
end

function EventWindowsHandler:LoadChatPlayerCombatData()
    self.AllChatPlayerCombatData = LoadTable(Turbine.DataScope.Account, "PrimeChatPlayerCombatData")

    if not(self.AllChatPlayerCombatData) then
        self.AllChatPlayerCombatData = self:GetBaseChatPlayerCombatData()
    end
end

function EventWindowsHandler:LoadChatEnemyCombatData()
    self.AllChatEnemyCombatData = LoadTable(Turbine.DataScope.Account, "PrimeChatEnemyCombatData")

    if not(self.AllChatEnemyCombatData) then
        self.AllChatEnemyCombatData = self:GetBaseChatEnemyCombatData()
    end
end

function EventWindowsHandler:LoadSelfEffectData()
    self.AllSelfEffectData = LoadTable(Turbine.DataScope.Account, "PrimeSelfEffectData")
    if not(self.AllSelfEffectData) then
        self.AllSelfEffectData = self:GetBaseSelfEffectData()
    end
end

function EventWindowsHandler:LoadGroupEffectData()
    self.AllGroupEffectData = LoadTable(Turbine.DataScope.Account, "PrimeGroupEffectData")
    if not(self.AllGroupEffectData) then
        self.AllGroupEffectData = self:GetBaseGroupEffectData()
    end
end

--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------

function EventWindowsHandler:AddChatAlert(source, trigger, alert)
    if not(self.AllChatData[source]) then self.AllChatData[source] = {} end
    self.AllChatData[source][trigger] = alert
end

function EventWindowsHandler:AddSelfEffectAlert(key, value)
    self.AllSelfEffectData[key] = value
end

function EventWindowsHandler:AddGroupEffectAlert(key, value)
    self.AllGroupEffectData[key] = value
end

function EventWindowsHandler:AddChatPlayerCombatAlert(trigger, alert)
    self.AllChatPlayerCombatData[trigger] = alert
end

function EventWindowsHandler:AddChatEnemyCombatAlert(trigger, alert)
    self.AllChatEnemyCombatData[trigger] = alert
end

function EventWindowsHandler:GetCurrentChatData()
    return self.AllChatData
end

function EventWindowsHandler:GetCurrentSelfEffectData()
    return self.AllSelfEffectData
end

function EventWindowsHandler:GetCurrentGroupEffectData()
    return self.AllGroupEffectData
end

function EventWindowsHandler:GetCurrentChatPlayerCombatData()
    return self.AllChatPlayerCombatData
end

function EventWindowsHandler:GetCurrentChatEnemyCombatData()
    return self.AllChatEnemyCombatData
end

function EventWindowsHandler:GetBaseChatData()
    -- Chat alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 windowID}
    return 
        {
            ["Groz"] = 
            {
                ["Crush them to bits."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Smash, smash, smash."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["This will hurt."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Smash those little things."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Smash all of them to bits."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Incompetent oafs! Crush and smash them!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "RP",
                            [3.000000] = "PrimePlugins/RaidTools/Images/RolePlay.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {

                    }
                }
            },
            ["The Shadowed King"] = 
            {
                ["Atmên ambâkh tabi ablâkmâ"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 2.000000,
                            [2.000000] = "AOE",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "AirHorn"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 45.000000,
                            [2.000000] = "Shadow Force",
                            [3.000000] = "PrimePlugins/RaidTools/Images/ShadowFace.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 36.000000
                    }
                }
            },
            ["Remmenaeg"] = 
            {
                ["Welcome. You have made it quite far, but now your journey has come to its end."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Left = Bats! Right = Wights! Far = Spirit!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Captain Zabothak"] = 
            {
                ["Show me your strength! Haha!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Reflect!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/Logo.tga",
                            [4.000000] = true,
                            [5.000000] = "alarmbeep"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 35.000000,
                            [2.000000] = "Reflect!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/Logo.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 1.000000
                    }
                },
                ["You weaklings cannot defeat ME!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Reflect!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/Logo.tga",
                            [4.000000] = true,
                            [5.000000] = "alarmbeep"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 35.000000,
                            [2.000000] = "Reflect!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/Logo.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 1.000000
                    }
                },
                ["Slaves! Make yourselves useful!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Adds",
                            [3.000000] = "PrimePlugins/RaidTools/Images/Goblin.tga",
                            [4.000000] = false,
                            [5.000000] = "Adds"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 1.000000
                    }
                },
                ["Get out here, you worthless goblins!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Adds",
                            [3.000000] = "PrimePlugins/RaidTools/Images/Goblin.tga",
                            [4.000000] = false,
                            [5.000000] = "Adds"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 1.000000
                    }
                }
            },
            ["Brúnek I Clovenbow"] = 
            {
                ["Taste my axes"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Interrupt Clovenbrow",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzz_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "Clovenbrow axes (Min)",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["How would you like"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Interrupt Clovenbrow",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzz_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "Clovenbrow axes (Min)",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Khúrthak the Rent"] = 
            {
                ["Crush"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Dropping puddle!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/PuddleSmall.jpg",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzz_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 14.000000
                    }
                },
                ["Away"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Frontal",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 14.000000
                    }
                },
                ["strength"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Remove damage buff",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 14.000000
                    }
                }
            },
            ["Shatarkha"] = 
            {
                ["Chill them to the core."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Frontal!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "runout"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["May your blood run cold."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Horn",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "defensive"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                }
            },
            ["Blogmal Troll-keeper"] = 
            {
                ["Hah hah hah! Now Master not near to tell me be careful!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You dead now, little squishies, me swing real hard this time!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I will put an end to you, vermin!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Interrupt Slave-master",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["No problem smashing when Master far away!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Igash the Fanatic"] = 
            {
                ["Let the darkness crush you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 2.500000,
                            [2.000000] = "AoE stun!!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = 2.500000
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Kúzek Squint-Eye"] = 
            {
                ["Grrr."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 10.000000,
                            [2.000000] = "Squint-eye stun",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzz_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 40.000000,
                            [2.000000] = "Squint-eye stun",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Ránulur, Chieftain of Wargs"] = 
            {
                ["Children! To me!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Adds!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/WolfIcon.tga",
                            [4.000000] = false,
                            [5.000000] = "Adds"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 45.000000,
                            [2.000000] = "Adds",
                            [3.000000] = "PrimePlugins/RaidTools/Images/WolfIcon.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 7.000000
                    }
                },
                ["Your father needs you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Adds!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/WolfIcon.tga",
                            [4.000000] = false,
                            [5.000000] = "Adds"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 45.000000,
                            [2.000000] = "Adds",
                            [3.000000] = "PrimePlugins/RaidTools/Images/WolfIcon.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 7.000000
                    }
                }
            },
            ["Cargaraf"] = 
            {
                ["Away with you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Move away from Woman!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["The Mistress of Pestilence"] = 
            {
                ["Your ignorance will be your final undoing!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Beware Reflect | CC limrafn to Kill",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Let darkness aid me!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Goblin phase",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Impressive. Your thoughts are clear but you still do not see."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Overseer phase",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Flâgît"] = 
            {
                ["You can not stop Her Army! I will not allow it!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Bubble phase!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["The Mistress' Army is too strong! We were bred stronger and tougher!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Beware of eyes",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["The light... it dims... what are you doing? I must protect myself!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "FASTER BUBBLES",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Blagh"] = 
            {
                ["Brother! We are together, let us strike in unison!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Split trolls!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I am here, Brother! You take the weak one and I'll take the ugly one!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Split trolls!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I am with you Brother! We shall punish them together!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Split trolls!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["His death will mean your death!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Mushroom phase",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Manôzagar"] = 
            {
                ["shadow of the"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 7.000000,
                            [2.000000] = "Aoe distributed",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = true,
                            [5.000000] = "alarmdouble_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 12.000000
                    }
                },
                ["None can stand before me!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 7.000000,
                            [2.000000] = "Frontal",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzz_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 12.000000
                    }
                },
                ["pests!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Group knockup",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzz_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 12.000000
                    }
                }
            },
            ["Shaktur"] = 
            {
                ["Don't run!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Frontal",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 33.000000,
                            [2.000000] = "AoE!",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 11.000000
                    }
                },
                ["maggots"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Frontal",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 32.000000,
                            [2.000000] = "Maggots",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 11.000000
                    }
                }
            },
            ["Vethúg Wintermind"] = 
            {
                ["Freeze, insects"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Check for BLUE eye",
                            [4.000000] = true,
                            [5.000000] = "alarmdouble_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 50.000000,
                            [2.000000] = "Blue Eye",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Dúrchest"] = 
            {
                ["Such a shame to come so far only to fail."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Remove Corruptions on boss",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Cowards! Cheats! We will not be lured away from our abode so easily!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Seems you dropped the soap! Have fun!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You! Finish these puny creatures!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Add Entering",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Soldier! Front and centre!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Add Entering",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Karazgar the Weeping Warrior"] = 
            {
                ["Finish them"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Check for BLUE eye",
                            [4.000000] = true,
                            [5.000000] = "alarmdouble_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 50.000000,
                            [2.000000] = "Blue Eye",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["meet your end"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Check for PURPLE eye",
                            [4.000000] = true,
                            [5.000000] = "alarmdouble_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 40.000000,
                            [2.000000] = "Purple Eye",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["By my Morgul Steel!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Check for PURPLE eye",
                            [4.000000] = true,
                            [5.000000] = "alarmdouble_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 40.000000,
                            [2.000000] = "Purple Eye",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Zholuga"] = 
            {
                ["Mistress, lend me your courage!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "DO NOT touch the runestones",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Rung"] = 
            {
                ["Brother! We are together, let us strike in unison!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Split trolls!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I am here, Brother! You take the weak one and I'll take the ugly one!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Split trolls!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I am with you Brother! We shall punish them together!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Split trolls!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["His death will mean your death!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Mushroom phase",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Urcheron"] = 
            {
                ["You will find your end soon enough!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Interupt, Move out of Flames",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Warchief Búrzghâsh"] = 
            {
                ["Whelps! You have come far enough. This is the end for you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Tier 2? Killing goblins boosts dmg of boss",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Gultháva, Maiden of Gladden"] = 
            {
                ["This place the dwarves called"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 38.000000,
                            [2.000000] = "RP",
                            [3.000000] = "PrimePlugins/RaidTools/Images/RolePlay.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {

                    }
                }
            },
            ["Udúnion"] = 
            {
                ["Burn!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Horn of Anor - FAR SIDE!",
                            [4.000000] = 6.000000
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Shadow take you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Staff of Dark Shadows - LEFT SIDE!",
                            [4.000000] = 6.000000
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Come dark fire!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Scroll of Ruin - RIGHT SIDE!",
                            [4.000000] = 6.000000
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Carchrien"] = 
            {
                ["Mmm come to play?"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Careful of bat stuns!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Hahahaha"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Careful of bat stuns!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Gothghaash the Firecaller"] = 
            {
                ["Wreathed in flame I'll watch you burn yourself!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 8.000000,
                            [2.000000] = "Melee REFLECT!!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = 8.000000
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Blogmal Slave-master"] = 
            {
                ["Hah hah hah! Now Master not near to tell me be careful!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You dead now, little squishies, me swing real hard this time!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I will put an end to you, vermin!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Interrupt Slave-master",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["No problem smashing when Master far away!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Ulk"] = 
            {
                ["Crush them to bits."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Smash, smash, smash."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["This will hurt."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Smash those little things."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Smash all of them to bits."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Incompetent oafs! Crush and smash them!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "RP",
                            [3.000000] = "PrimePlugins/RaidTools/Images/RolePlay.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {

                    }
                }
            },
            ["Bratha Tasakh, Lady of Many Venoms"] = 
            {
                ["My chosen, embrace them"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Spider spawn",
                            [3.000000] = "PrimePlugins/RaidTools/Images/Spider.tga",
                            [4.000000] = false,
                            [5.000000] = "Adds"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 1.000000
                    }
                }
            },
            ["Gaerdring"] = 
            {
                ["You have entered your new tomb."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Killing crawlers boosts add damage!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["No more talk! Die!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Kill Gaerdring to stop add spawn!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Dargásh"] = 
            {
                ["Come closer!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "BIG AOE",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 25.000000,
                            [2.000000] = "Come closer",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 11.000000
                    }
                },
                ["Contagion"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 9.000000,
                            [2.000000] = "Tank is dropping puddles!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/PuddleSmall.jpg",
                            [4.000000] = true,
                            [5.000000] = "alarmdouble_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "Puddles",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 11.000000
                    }
                },
                ["Minions"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Interrupt Dargash",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 11.000000
                    }
                }
            },
            ["Barz"] = 
            {
                ["Come, slaves. Deal with these worms!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Slave wave [Right]",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I will bring this cave down upon you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Move out of cave-in",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Let us see how you fare against one of my trolls!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Troll wave [Left]",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Quickly! Destroy them! Barz commands it!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Slave-master wave [Left]",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["No power is stronger than Barz!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Random power-drain Keep distance",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I'll gnaw your bones!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Move away!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Azbauz"] = 
            {
                ["They will fall."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt Azbaus",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false,
                            [5.000000] = "kickcast"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Bleed… slowly."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Bleed",
                            [3.000000] = "PrimePlugins/RaidTools/Images/Bloodboil.tga",
                            [4.000000] = false,
                            [5.000000] = "defensive"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                }
            },
            ["Zurm"] = 
            {
                ["Slaves, destroy these intruders, or you will feel my lash!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Slave wave [Left]",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["My trolls will deal with you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Troll wave [Right]",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Fools! Aid your master...destroy them!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Slave-master wave [Right]",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Óiko I the Bellower"] = 
            {
                ["Súmrutu!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Interrupt bellower",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzz_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "Bellower interrupt",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Nardur the Shield"] = 
            {
                ["Give me some room!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 2.500000,
                            [2.000000] = "Knockback incoming!!",
                            [4.000000] = 2.500000
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Rûkhor the Pale Herald"] = 
            {
                ["Your strength shall be your undoing!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Blood Surge",
                            [3.000000] = "PrimePlugins/RaidTools/Images/Blood.tga",
                            [4.000000] = true,
                            [5.000000] = "aesoon"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 1.000000
                    }
                },
                ["How long can you last without your power?"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Interrupt Rúkhor!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = true,
                            [5.000000] = "kickcast"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 1.000000
                    }
                },
                ["Your power becomes mine!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Interrupt Rúkhor!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = true,
                            [5.000000] = "kickcast"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 1.000000
                    }
                },
                ["I will turn your strength against you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Blood Surge",
                            [3.000000] = "PrimePlugins/RaidTools/Images/Blood.tga",
                            [4.000000] = true,
                            [5.000000] = "aesoon"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 1.000000
                    }
                },
                ["Your life-force sustains me!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Life drain",
                            [3.000000] = "PrimePlugins/RaidTools/Images/Bloodboil.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbeep"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 1.000000
                    }
                },
                ["What are you without your power!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Interrupt Rúkhor!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = true,
                            [5.000000] = "kickcast"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 1.000000
                    }
                },
                ["It is over, Zabothak... and you have failed."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 39.000000,
                            [2.000000] = "RP",
                            [3.000000] = "PrimePlugins/RaidTools/Images/RolePlay.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {

                    }
                }
            },
            ["Rúcrog Hoarpelt"] = 
            {
                ["Die by the claws of the Gauredain!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Frontal AoE",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "aesoon"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["My spear flies!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Javelin-Puddle",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "Info"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["Ancestors, I call upon you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Totem Spawn!",
                            [4.000000] = false,
                            [5.000000] = "Adds"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["Show me the strength of your pack!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Frontal AoE",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "aesoon"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["Our ancestors fight with me!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Totem Spawn!",
                            [4.000000] = false,
                            [5.000000] = "Adds"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["Feel the sting of my spear!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Javelin-Puddle",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "Info"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["Fight as one!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "CC Adds!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                }
            },
            ["Ivar"] = 
            {
                ["A new wretch come to fill our ranks, my maiden. And by the smell of living flesh your servant will be tested quickly."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Don't kill Banner-Wights",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["The Blind One "] = 
            {
                ["Darklings, arise!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Darkling phase",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Now, we fight."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Now we fight",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I gather the shadows!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Remove corruptions | CC clones!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You have made it deep into our lair. The Mistress will be most displeased if I allow you any further."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "I hope you brought a damage type",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Isvítha the Gluttonous"] = 
            {
                ["taste"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Check for eye",
                            [4.000000] = false,
                            [5.000000] = "alarmdouble_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 180.000000,
                            [2.000000] = "Purple eye",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["main"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Lmao it is enraged",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["help prepare"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Adds to main tank",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzz"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["scrumptious"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Check for eye",
                            [4.000000] = false,
                            [5.000000] = "alarmdouble_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 180.000000,
                            [2.000000] = "Purple eye",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["play"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "<rgb=#FF9900>Congratulations</rgb>",
                            [4.000000] = false,
                            [5.000000] = "Victory"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["drakes"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Add spawn",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzz"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 140.000000,
                            [2.000000] = "Add spawn",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Prince Durin"] = 
            {
                ["Heroes! To me!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Hide!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "findshelter"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 70.000000,
                            [2.000000] = "Flames",
                            [3.000000] = "PrimePlugins/RaidTools/Images/FireShield.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 7.000000
                    }
                },
                ["Stand with us! We will shield you from the flames!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Hide!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "findshelter"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 70.000000,
                            [2.000000] = "Flames",
                            [3.000000] = "PrimePlugins/RaidTools/Images/FireShield.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 7.000000
                    }
                }
            },
            ["Gulmúk Dampmould"] = 
            {
                ["Witness our might!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Big Knockback",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptIcon.tga",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 33.000000
                    }
                },
                ["Fall before us!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Small Knockback",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptIcon.tga",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 33.000000
                    }
                },
                ["Our corruption runs deep!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Puddle",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 33.000000
                    }
                }
            },
            ["Dolguzigir, the Dark Archivist"] = 
            {
                ["double-edged"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Spirit spawn",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "AirHorn"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 15.000000
                    }
                },
                ["fires of rage"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Spirit spawn",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "AirHorn"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 15.000000
                    }
                },
                ["Eeeaaa"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "<rgb=#FF9900>Congratulations</rgb>",
                            [4.000000] = false,
                            [5.000000] = "Victory"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 15.000000
                    }
                },
                ["memories against"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Throw",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "AirHorn"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 15.000000
                    }
                }
            },
            ["Megoriath"] = 
            {
                ["Welcome to my lair, friends. I have had quite a few visitors recently...."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Stepping Stones on LAGTRO What could go wrong?",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Talug's Fire-fist"] = 
            {
                ["Mind your feet…."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "ARCHER PLACING TRAP!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Aaaiiiieee!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Kill fire-first next to boss, then drag into trap!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Alagossír"] = 
            {
                ["Can you handle the burn"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Stack up or Line of Sight!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You stray too far, !"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Get in Melee Range!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["The storms are gathering quickly."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Stay In Melee Range!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["The storm's pace hastens. It is only a matter of time now."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Hug left brazier to avoid lightning!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Dóbruz II Stark-heart"] = 
            {
                ["clan is mine"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Interrupt Stark-Heart",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzz_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "Stark heart interrupt",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You will not harm"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Interrupt Stark-Heart",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzz_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "Stark heart interrupt",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Îstum"] = 
            {
                ["You are too strong! Too many! It is good I am here to stop you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Middle worm spawn",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["What is this? The Mistress ordered me not to fail! Death to all!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Corner worm spawn",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Brántokh II the Sunderer"] = 
            {
                ["Cower before the might"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 40.000000,
                            [2.000000] = "Sunderer aoe",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {

                    }
                },
                ["bring this mountain"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 40.000000,
                            [2.000000] = "Sunderer aoe",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {

                    }
                }
            },
            ["Storm of the Witch-King"] = 
            {
                ["You will not escape."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 8.000000,
                            [2.000000] = "Avoid Wave!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = 8.000000,
                            [5.000000] = "AirHorn"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["Your end draws near, mortal."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 8.000000,
                            [2.000000] = "Avoid Wave!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = 8.000000,
                            [5.000000] = "AirHorn"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                }
            },
            ["Dóbruz IV the Unheeding"] = 
            {
                ["I challenge YOU"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 35.000000,
                            [2.000000] = "Unheeding challenge",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {

                    }
                },
                ["You look like a weakling"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 35.000000,
                            [2.000000] = "Unheeding challenge",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {

                    }
                }
            },
            ["Vrarz the Cook"] = 
            {
                ["Would you like that pan-seared?"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Stun & Wound Incoming!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Have you tried the special?"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 8.000000,
                            [2.000000] = "INTERRUPT or move away!!!",
                            [4.000000] = 8.000000
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Agath-kali, the Gaunt-woman"] = 
            {
                ["Your mind shall break!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Spread for fear!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = true,
                            [5.000000] = "Spread"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 12.000000
                    }
                },
                ["Drink, Morloth! Drink and be renewed!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Root bubble",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 12.000000
                    }
                },
                ["Wither... and die!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Enraged",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmdouble_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 12.000000
                    }
                },
                ["You shall serve"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Spirit",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = true,
                            [5.000000] = "runaway"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 12.000000
                    }
                }
            },
            ["Gothmog"] = 
            {
                ["The fires of Mordor come for you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "MORGUL FLAMES!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 8.000000
                    }
                }
            },
            ["Aigrith the Ashen"] = 
            {
                ["Burn to cinders!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Remove Corruption!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/FireShield.tga",
                            [4.000000] = false,
                            [5.000000] = "dontdie"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["You will observe our pact, spirit!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Add",
                            [4.000000] = false,
                            [5.000000] = "Adds"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["By the will of Angmar!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Avoid Frontal",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "aesoon"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["I bind you to my will, spirit!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Add",
                            [4.000000] = false,
                            [5.000000] = "Adds"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["By the power of the Witch-king!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Avoid Frontal",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "aesoon"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["Burn to ash!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Remove Corruption!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/FireShield.tga",
                            [4.000000] = false,
                            [5.000000] = "dontdie"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["I call to the flames!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Cleanse in Water!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = 5.000000,
                            [5.000000] = "findshelter"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                },
                ["The flames will burn you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Cleanse in Water!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = 5.000000,
                            [5.000000] = "findshelter"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 32.000000
                    }
                }
            },
            ["Naruhel"] = 
            {
                ["My strength weakens, surround me nature and defend me from my attackers."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Adds",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "Adds"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I will not fall! This forest is my home, where are my defenders?"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Adds",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "Adds"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["My bonds with the swamp weaken, come water protect me from these intruders."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Adds",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "Adds"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Tarkrîp Cave-troll"] = 
            {
                ["Hah hah hah! Now Master not near to tell me be careful!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You dead now, little squishies, me swing real hard this time!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I will put an end to you, vermin!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Interrupt Slave-master",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["No problem smashing when Master far away!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["General Talug"] = 
            {
                ["Mind your legs!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "INTERRUPT or move away!!!",
                            [4.000000] = 4.000000
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You have arrived at the perfect time!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Phase 1, Pull Into traps",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You have the honour of witnessing the power of my warriors before Lothlórien!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Phase 2, Kill fire-first next to him and then trap him",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Show them how you'll kill those wretched Elf Archers!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Phase 3, Nuke him!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Groz-and-Ulk"] = 
            {
                ["Crush them to bits."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Smash, smash, smash."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["This will hurt."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Smash those little things."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Smash all of them to bits."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["Incompetent oafs! Crush and smash them!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "RP",
                            [3.000000] = "PrimePlugins/RaidTools/Images/RolePlay.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {

                    }
                }
            },
            ["Skorgrím"] = 
            {
                ["You cannot withstand my full might"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Hammer!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = true,
                            [5.000000] = "AirHorn"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 15.000000
                    }
                }
            },
            ["Brogadan"] = 
            {
                ["Greetings, intruders. You are a bit late for the festivities, but perhaps I can fill you in."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Kill Limrafn, Avoid Flames!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Moria"] = 
            {
                ["Intruders!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Use uniforms to not alert the guards!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["An intruder! Get him!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Put on his uniform to be invisible!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["How dare you come here!?"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Put on his uniform to be invisible!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Sound the alarm!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Use uniforms to not alert the guards!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["To arms!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Use uniforms to not alert the guards!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You're no Orc!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Put on his uniform to be invisible!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Do you think me a fool?"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Put on his uniform to be invisible!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You can't fool me… !"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Put on his uniform to be invisible!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You shall not pass!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Put on his uniform to be invisible!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Hey! You're not an officer!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Put on his uniform to be invisible!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Prepare the Warband Manoeuvre!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "PREPARE THE WARBAND MANOEUVRE!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Atli Spider-bane"] = 
            {
                ["Legolas! Over here!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "RP",
                            [3.000000] = "PrimePlugins/RaidTools/Images/RolePlay.tga"
                        }
                    },
                    ["Restrictions"] = 
                    {

                    }
                }
            },
            ["Umbarrim Sorcerer"] = 
            {
                ["You will betray your friends this day, !"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Betray target, Move Out of Group!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Burn, !"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Burn Target, Move Into Group",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Glathlírel "] = 
            {
                ["When I open this seal, the true test of your will shall begin!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Time to wake up",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Strangers... you were sent to me by the Eldgang?"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Enjoy the RP",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Dúnlang"] = 
            {
                ["Welcome, welcome are you new recruits for the Red-maid's cause?"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Don't kill adds!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Hrímil Frost-heart"] = 
            {
                ["glacier"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Knockback",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzzer"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 70.000000,
                            [2.000000] = "Knockback",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["powerless"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 10.000000,
                            [2.000000] = "Hide",
                            [4.000000] = true,
                            [5.000000] = "runaway"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 90.000000,
                            [2.000000] = "Ice-breath",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Begone"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Knockback",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzzer"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 70.000000,
                            [2.000000] = "Knockback",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Blogmal Cave-troll"] = 
            {
                ["Hah hah hah! Now Master not near to tell me be careful!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You dead now, little squishies, me swing real hard this time!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I will put an end to you, vermin!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Interrupt Slave-master",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["No problem smashing when Master far away!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Narkhor the Flensed"] = 
            {
                ["flay you until nothing"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Frontal!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzzer"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 12.000000
                    }
                },
                ["The blood of the Unhoused is mine to command!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Add spawn",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmdouble_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 12.000000
                    }
                },
                ["flesh limits you"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Triple hit!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzz_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 12.000000
                    }
                }
            },
            ["Tarkrîp Slave-master"] = 
            {
                ["Hah hah hah! Now Master not near to tell me be careful!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You dead now, little squishies, me swing real hard this time!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I will put an end to you, vermin!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Interrupt Slave-master",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["No problem smashing when Master far away!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Commander Greb"] = 
            {
                ["No more games! Prepare to fall to my flaming axe!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "DONT APPLY MELEE DOTS!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Now you shall witness my defences..."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Adds Entering!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Ha! I'll return your puny blows!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 30.000000,
                            [2.000000] = "200% MELEE REFLECT!!!!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = 30.000000,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["The General will avenge me!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Collect Key from chest!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Gorothúl"] = 
            {
                ["Let terror take you!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "INCOMING AOE FEAR!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["From this point forward, I will teach you the true meaning of terror!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Dont Kill Guldur-Spirits",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Maugrush"] = 
            {
                ["Blood shall cover the field."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Interrupt Maugrash",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = false,
                            [5.000000] = "kickcast"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["My blade will bathe in blood."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Bleed",
                            [3.000000] = "PrimePlugins/RaidTools/Images/Bloodboil.tga",
                            [4.000000] = false,
                            [5.000000] = "defensive"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                },
                ["All those before me shall fall."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Frontal!",
                            [4.000000] = false,
                            [5.000000] = "runout"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 10.000000
                    }
                }
            },
            ["Óiko II Rill-Seeker"] = 
            {
                ["I seek the"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 40.000000,
                            [2.000000] = "Rill-seeker aoe",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {

                    }
                },
                ["Get out of my way!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 40.000000,
                            [2.000000] = "Rill-seeker aoe",
                            [3.000000] = 1090551617.000000
                        }
                    },
                    ["Restrictions"] = 
                    {

                    }
                }
            },
            ["Thadúr"] = 
            {
                ["I grow bored of this. My hounds will deal with you."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Kill adds to drop shield!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["Do you mock me with this refusal to die? Allow me to collect myself while you meet my friends."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Kill adds to drop shield!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["This is becoming quite frustrating. I must rest, and then I will deal with you once and for all!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Kill adds to drop shield!",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Rhavameldir"] = 
            {
                ["You think to find Amarthiel here, do you? This keep belongs to us now."] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 5.000000,
                            [2.000000] = "Adds enter on boss HP, and at 1 minute&2minutes",
                            [4.000000] = false
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["Tarkrîp Troll-keeper"] = 
            {
                ["Hah hah hah! Now Master not near to tell me be careful!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["You dead now, little squishies, me swing real hard this time!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["I will put an end to you, vermin!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 6.000000,
                            [2.000000] = "Interrupt Slave-master",
                            [3.000000] = "PrimePlugins/RaidTools/Images/InterruptSmall.jpg",
                            [4.000000] = true,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                },
                ["No problem smashing when Master far away!"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 4.000000,
                            [2.000000] = "Stack on keeper!",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzzer_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = false
                    }
                }
            },
            ["The Witch-king"] = 
            {
                ["Consumed by shadow"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Puddles",
                            [3.000000] = "PrimePlugins/RaidTools/Images/PuddleSmall.jpg",
                            [4.000000] = false,
                            [5.000000] = "alarmbuzz_quiet"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 17.000000
                    }
                },
                ["I fear no one"] = 
                {
                    ["Alerts"] = 
                    {
                        [1.000000] = 
                        {
                            [1.000000] = 3.000000,
                            [2.000000] = "Threat wipe",
                            [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                            [4.000000] = false,
                            [5.000000] = "AirHorn"
                        }
                    },
                    ["Timers"] = 
                    {
                        [1.000000] = 
                        {

                        }
                    },
                    ["Restrictions"] = 
                    {
                        [1.000000] = 17.000000
                    }
                }
            }
        }
    -- Minas Morgul
    -- Harrowing
    --[[
    local AShaktur = {
        ["Don't run!"] = {5, FrontalMessage, AlertIconSmall, false, SoundEffects["Horn"], 11, nil, 33, "AoE!", 0x41007F41},
        ["maggots"] = {5, FrontalMessage, AlertIconSmall, false, SoundEffects["Horn"], 11, nil, 32, "Maggots", 0x41007F41},
    }

    local ADargash = {
        ["Come closer!"] = {5, "BIG AOE", AlertIconSmall, false, SoundEffects["Horn"], 11, nil, 25, "Come closer", 0x41007F41},
        ["Contagion"] = {9, "Tank is dropping puddles!", PuddleIconSmall, true, SoundEffects["Doubleq"], 11, nil, 30, "Puddles", 0x41007F41},
        ["Minions"] = {5, "Interrupt Dargash", InterruptIconSmall, true, SoundEffects["Hornq"], 11, nil, 30, "Interrupt", 0x41007F41}
    }

    -- Roost
    local AKhurtakTheRent = {
        ["Crush"] = {3, Puddle, PuddleIconSmall, false, SoundEffects["Alarmq"], 14, nil},
        ["Away"] = {5, FrontalMessage, AlertIconSmall, false, SoundEffects["Horn"], 14, nil},
        ["strength"] = {5, "Remove damage buff", nil, false, nil, 14, nil}
    }

    -- Anvil
    local EyeMessage = {5, "Check for eye", nil, false, SoundEffects["Doubleq"], false, nil, 180, "Purple eye", 0x41007F41}
    local AIsvitha = {
        ["taste"] = EyeMessage,
        ["drakes"] = {5, "Add spawn", nil, false, SoundEffects["Alarm"], false, nil, 140, "Add spawn", 0x41007F41},
        ["scrumptious"] = EyeMessage,
        ["help prepare"] = {5, "Adds to main tank", AlertIconSmall, false, SoundEffects["Alarm"], false, nil},
        ["main"] = {5, "Lmao it is enraged", nil, false, SoundEffects["Horn"], false, nil},
        ["play"] = {3, "<rgb=#FF9900>Congratulations</rgb>", nil, false, SoundEffects["Victory"], false, nil}

    }

    local AKuzek = {
        ["Grrr."] = {10, "Squint-eye stun", nil, true, SoundEffects["Alarmq"], false, nil, 40, "Squint-eye stun", 0x41007F41}
    }

    local ABrunek = {
        ["Taste my axes"] = {3, "Interrupt Clovenbrow", InterruptIconSmall, false, SoundEffects["Alarmq"], false, nil, 30, "Clovenbrow axes (Min)", 0x41007F41},
        ["How would you like"] = {3, "Interrupt Clovenbrow", InterruptIconSmall, false, SoundEffects["Alarmq"], false, nil, 30, "Clovenbrow axes (Min)", 0x41007F41}
    }

    -- The unheeding & stark heart
    local ADobruzUnheed = {
        ["I challenge YOU"] =  {nil, nil, nil, nil, nil, nil, nil, 35, "Unheeding challenge", 0x41007F41},
        ["You look like a weakling"] =  {nil, nil, nil, nil, nil, nil, nil, 35, "Unheeding challenge", 0x41007F41},
    }

    local ADobruzStark = {
        ["You will not harm"] = {5, "Interrupt Stark-Heart", InterruptIconSmall, false, SoundEffects["Alarmq"], false, nil, 30, "Stark heart interrupt", 0x41007F41},
        ["clan is mine"] = {5, "Interrupt Stark-Heart", InterruptIconSmall, false, SoundEffects["Alarmq"], false, nil, 30, "Stark heart interrupt", 0x41007F41}
    }

    local ASunderer = {
        ["Cower before the might"] =  {nil, nil, nil, nil, nil, nil, nil, 40, "Sunderer aoe", 0x41007F41},
        ["bring this mountain"] =  {nil, nil, nil, nil, nil, nil, nil, 40, "Sunderer aoe", 0x41007F41},
    }

    local AOikoBell = {
        ["Súmrutu!"] = {3, "Interrupt bellower", InterruptIconSmall, false, SoundEffects["Alarmq"], false, nil, 30, "Bellower interrupt", 0x41007F41}
    }

    local AOikoRill = {
        ["Get out of my way!"] = {nil, nil, nil, nil, nil, nil, nil, 40, "Rill-seeker aoe", 0x41007F41},
        ["I seek the"] ={nil, nil, nil, nil, nil, nil, nil, 40, "Rill-seeker aoe", 0x41007F41},
    }

    local AKarazgar = {
        ["Finish them"] = {5, "Check for BLUE eye", nil, true, SoundEffects["Doubleq"], false, nil, 50, "Blue Eye", 0x41007F41},
        ["meet your end"] = {5, "Check for PURPLE eye", nil, true, SoundEffects["Doubleq"], false, nil, 40, "Purple Eye", 0x41007F41},
        ["By my Morgul Steel!"] = {5, "Check for PURPLE eye", nil, true, SoundEffects["Doubleq"], false, nil, 40, "Purple Eye", 0x41007F41}
    }

    local AVethug = {
        ["Freeze, insects"] = {5, "Check for BLUE eye", nil, true, SoundEffects["Doubleq"], false, nil, 50, "Blue Eye", 0x41007F41}
    }
    

    local AHrimil = {
        ["glacier"] = {5, "Knockback", AlertIconSmall, true, SoundEffects["Horn"], false, nil, 70, "Knockback", 0x41007F41},
        ["Begone"] = {5, "Knockback", AlertIconSmall, true, SoundEffects["Horn"], false, nil, 70, "Knockback", 0x41007F41},
        ["powerless"] = {10, "Hide", nil, true, SoundEffects["Run"], false, nil, 90, "Ice-breath", 0x41007F41}, 
    }

    local AManozagar = {
        --["shadows of doubt"] = {11, "Group puddle", PuddleIcon, true},
        ["shadow of the"] = {7, "Aoe distributed", AlertIconSmall, true, SoundEffects["Doubleq"], 12, nil},
        ["pests!"] = {4, "Group knockup", AlertIconSmall, true, SoundEffects["Alarmq"], 12, nil},
        ["None can stand before me!"] = {7, "Frontal", AlertIconSmall, true, SoundEffects["Alarmq"], 12, nil}
    }

    local ANarkhor = {
        ["flay you until nothing"] = {5, "Frontal!", AlertIconSmall, true, SoundEffects["Horn"], 12, nil},
        ["flesh limits you"] = {3, "Triple hit!", AlertIconSmall, true, SoundEffects["Alarmq"], 12, nil},
        --["Bleed, you miserable wretches!"] = {3, "Aura", AlertIconSmall, false, nil},
        ["The blood of the Unhoused is mine to command!"] = {3, "Add spawn", AlertIconSmall, false, SoundEffects["Doubleq"], 12, nil}
    }

    local AKali = {
        --["Offer your blood!"] = {3, "Aura", AlertIconSmall, false, nil},
        ["Your mind shall break!"] = {6, "Spread for fear!", AlertIconSmall, true, SoundEffects["Spread"], 12, nil},
        ["You shall serve"] = {5, "Spirit", AlertIconSmall, true, SoundEffects["Run"], 12, nil},
        ["Drink, Morloth! Drink and be renewed!"] = {3, "Root bubble", AlertIconSmall, false, nil, 12, nil},
        ["Wither... and die!"] = {3, "Enraged", AlertIconSmall, false, SoundEffects["Doubleq"], 12, nil},
        --["And I am... the last"] = {3, "<rgb=#FF9900>Congratulations</rgb>", nil, false, SoundEffects["Victory"]}
    }

    local ADolgulzigir = {
        ["fires of rage"] = {3, "Spirit spawn", AlertIconSmall, false, SoundEffects["AHorn"], 15, nil},
        ["double-edged"] = {3, "Spirit spawn", AlertIconSmall, false, SoundEffects["AHorn"], 15, nil},
        ["memories against"] = {3, "Throw", AlertIconSmall, false, SoundEffects["AHorn"], 15, nil},
        ["Eeeaaa"] = {3, "<rgb=#FF9900>Congratulations</rgb>", nil, false, SoundEffects["Victory"], 15, nil}

    }

    -- The witch king
    local ATheWK = {
        ["Consumed by shadow"] = {3, "Puddles", PuddleIconSmall, false, SoundEffects["Alarmq"], 17, nil},
        ["I fear no one"] = {3, "Threat wipe", AlertIconSmall, false, SoundEffects["AHorn"], 17, nil},
    }
    
    -- The shadowed king
    local ATheSK = {
        ["Atmên ambâkh tabi ablâkmâ"] = {2, "AOE", AlertIconSmall, false, SoundEffects["AHorn"], 36, nil, 45, "Shadow Force", ShadowFace}, 
    }

    -- The blind one
    local ATheBO = {
        ["Darklings, arise!"] = {4, "Darkling phase", nil, false, SoundEffects["AHornq"], false, nil},
        ["Now, we fight."] = {4, "Now we fight", nil, false, nil, false, nil},
        ["You have made it deep into our lair. The Mistress will be most displeased if I allow you any further."] = {4, "I hope you brought a damage type", nil, false, nil, false, nil},
        ["I gather the shadows!"] = {5, "Remove corruptions | CC clones!", nil, false, nil, false, nil},
    }

    -- The mistress of pestilence
    local ATheMOP = {
        ["Your ignorance will be your final undoing!"] = {5, "Beware Reflect | CC limrafn to Kill", nil, false, nil, false, nil},
        ["Let darkness aid me!"] = {5, "Goblin phase", nil, false, nil, false, nil},
        ["Impressive. Your thoughts are clear but you still do not see."] = {5, "Overseer phase", nil, false, nil, false, nil},
    }

    -- The warden
    local ATheW = {
        ["You can not think to threaten me!"] = {3, "Threat Loss", nil, false, nil, false, nil},
        ["Your threat means nothing!"] = {3, "Threat Loss", nil, false, nil, false, nil},
        ["'Let terror take you all!"] = {5, "AOE FEAR", nil, false, nil, false, nil},
    }

    local ASkorgrim = {
        ["You cannot withstand my full might"] = {5, "Hammer!", AlertIconSmall, true, SoundEffects["AHorn"], 15, nil}
    }

    -------------------------------------------------------------------------------------------------------------------------------------
    -- Remmorchant ----------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------
    local ABratha = {
        ["My chosen, embrace them"] = {3, "Spider spawn", SpiderIcon, false, SoundEffects["Adds"], 1, nil}
    }

    local ARukhor = {
        ["I will turn your strength against you!"] = {6, "Blood Surge", BloodIcon, true, SoundEffects["aoe"], 1, nil},
        ["Your strength shall be your undoing!"] = {6, "Blood Surge", BloodIcon, true, SoundEffects["aoe"], 1, nil},
        ["Your life-force sustains me!"] = {3, "Life drain", BloodBoilIcon, false, SoundEffects["Beep"], 1, nil},
        ["Your power becomes mine!"] = {3, "Interrupt Rúkhor!", InterruptIconSmall, true, SoundEffects["Interrupt"], 1, nil},
        ["What are you without your power!"] = {3, "Interrupt Rúkhor!", InterruptIconSmall, true, SoundEffects["Interrupt"], 1, nil},
        ["How long can you last without your power?"] = {3, "Interrupt Rúkhor!", InterruptIconSmall, true, SoundEffects["Interrupt"], 1, nil},
        ["It is over, Zabothak... and you have failed."] = {nil, nil, nil, nil, nil, nil, nil, 39, "RP", RolePlay},
    }
    
    local AZabothrak = {
        ["You weaklings cannot defeat ME!"] = {3, "Reflect!", CJIconSmalll, true, SoundEffects["Beep"], 1, nil, 35, "Reflect!", CJIconSmalll},
        ["Show me your strength! Haha!"] = {3, "Reflect!", CJIconSmalll, true, SoundEffects["Beep"], 1, nil, 35, "Reflect!", CJIconSmalll},
        ["Get out here, you worthless goblins!"] = {5, "Adds", GoblinIcon, false, SoundEffects["Adds"], 1, nil},
        ["Slaves! Make yourselves useful!"] = {5, "Adds", GoblinIcon, false, SoundEffects["Adds"], 1, nil},
    }

    local AAtli = {
        ["Legolas! Over here!"] = {nil, nil, nil, nil, nil, nil, nil, 32, "RP", RolePlay}
    }
    

    -------------------------------------------------------------------------------------------------------------------------------------
    -- Great Barrows --------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------

    local AGaerdring = {
        ["You have entered your new tomb."] = {5, "Killing crawlers boosts add damage!", nil, false, nil, false, nil},
        ["No more talk! Die!"] = {5, "Kill Gaerdring to stop add spawn!", nil, false, nil, false, nil},
    }

    local AThadur = {
        ["I grow bored of this. My hounds will deal with you."] = {3, "Kill adds to drop shield!", nil, false, nil, false, nil},
        ["Do you mock me with this refusal to die? Allow me to collect myself while you meet my friends."] = {3, "Kill adds to drop shield!", nil, false, nil, false, nil},
        ["This is becoming quite frustrating. I must rest, and then I will deal with you once and for all!"] = {3, "Kill adds to drop shield!", nil, false, nil, false, nil},
    }

    -------------------------------------------------------------------------------------------------------------------------------------
    -- Garth Agarwen --------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------
    local AIvar = {
        ["A new wretch come to fill our ranks, my maiden. And by the smell of living flesh your servant will be tested quickly."] = {3, "Don't kill Banner-Wights", nil, false, nil, false, nil},
    }

    local ADunlang = {
        ["Welcome, welcome are you new recruits for the Red-maid's cause?"] = {3, "Don't kill adds!", nil, false, nil, false, nil},
    }

    local AVatar = {
        ["Who comes to disturb the slumber of the Red-maid? Come forward and show yourself."] = {5, "Enjoy the RP :)", nil, false, nil, false, nil},
        ["Let us begin!"] = {3, "Still Awake? Time to fight!", nil, false, nil, false, nil},
    }

    local ANaruhel = {
        -- 'Who calls me forth, who are not my devoted?'
        ["My strength weakens, surround me nature and defend me from my attackers."] = {3, "Adds", AlertIconSmall, false, SoundEffects["Adds"], false, nil},
        ["My bonds with the swamp weaken, come water protect me from these intruders."] = {3, "Adds", AlertIconSmall, false, SoundEffects["Adds"], false, nil},
        ["I will not fall! This forest is my home, where are my defenders?"] = {3, "Adds", AlertIconSmall, false, SoundEffects["Adds"], false, nil},
    }

    -------------------------------------------------------------------------------------------------------------------------------------
    -- Woe of the Willow --------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------
    local AGulmak = {
        -- "Pesky vermin... these woods are ours!"
        ["Witness our might!"] = {3, "Big Knockback", InterruptTargetIcon, false, nil, 33, nil},
        ["Fall before us!"] = {3, "Small Knockback", InterruptTargetIcon, false, nil, 33, nil},
        ["Our corruption runs deep!"] = {3, "Puddle", nil, false, nil, 33, nil},

        -- at old man willow ~ Marigold Boffin says, 'There's Old Man Willow. And... you see that? A sword!'
    }

    -------------------------------------------------------------------------------------------------------------------------------------
    -- Narrowdelve --------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------
    local ARucrog = {
        -- 'Come then, face a chieftain of the Gauredain!'
        ["My spear flies!"] = {5, "Javelin-Puddle", AlertIconSmall, false, SoundEffects["Ping"], 32, nil},
        ["Feel the sting of my spear!"] = {5, "Javelin-Puddle", AlertIconSmall, false, SoundEffects["Ping"], 32, nil},
        ["Show me the strength of your pack!"] = {3, "Frontal AoE", AlertIconSmall, false, SoundEffects["aoe"], 32, nil},
        ["Die by the claws of the Gauredain!"] = {3, "Frontal AoE", AlertIconSmall, false, SoundEffects["aoe"], 32, nil},
        ["Ancestors, I call upon you!"] = {5, "Totem Spawn!", nil, false, SoundEffects["Adds"], 32, nil}, 
        ["Our ancestors fight with me!"] = {5, "Totem Spawn!", nil, false, SoundEffects["Adds"], 32, nil}, 
        ["Fight as one!"] = {3, "CC Adds!", nil, false, nil, 32, nil},   
    }

    local AAigrith = {
        -- 'Outsiders! You will be destroyed at the hands of Aigrith the Ashen! The Witch-King has commanded it!'
        ["Burn to cinders!"] = {5, "Remove Corruption!", FireShield, false, SoundEffects["DontDie"], 32, nil},
        ["Burn to ash!"] = {5, "Remove Corruption!", FireShield, false, SoundEffects["DontDie"], 32, nil},
        ["By the will of Angmar!"] = {5, "Avoid Frontal", AlertIconSmall, false, SoundEffects["aoe"], 32, nil},
        ["By the power of the Witch-king!"] = {5, "Avoid Frontal", AlertIconSmall, false, SoundEffects["aoe"], 32, nil},
        ["You will observe our pact, spirit!"] = {3, "Add", nil, false, SoundEffects["Adds"], 32, nil},
        ["I bind you to my will, spirit!"] = {3, "Add", nil, false, SoundEffects["Adds"], 32, nil},
        ["I call to the flames!"] = {5, "Cleanse in Water!", AlertIconSmall, 5, SoundEffects["hide"], 32, nil},
        ["The flames will burn you!"] = {5, "Cleanse in Water!", AlertIconSmall, 5, SoundEffects["hide"], 32, nil},  

    }

    local AStorm = {
        -- Captain Glamagon shouts, '(NAME), Lothrandir, be ready for anything!'
        ["You will not escape."] = {8, "Avoid Wave!", AlertIconSmall, 8, SoundEffects["AHorn"], 32, nil},
        ["Your end draws near, mortal."] = {8, "Avoid Wave!", AlertIconSmall, 8, SoundEffects["AHorn"], 32, nil},
    }




    -------------------------------------------------------------------------------------------------------------------------------------
    -- Fornost --------------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------

    local AMegoriath = {
        ["Welcome to my lair, friends. I have had quite a few visitors recently...."] = {5, "Stepping Stones on LAGTRO What could go wrong?", nil, false, nil, false, nil},
    }

    local AWarchief = {
        ["Whelps! You have come far enough. This is the end for you!"] = {5, "Tier 2? Killing goblins boosts dmg of boss", nil, false, nil, false, nil},
    }

    local ARhavameldir = {
        ["You think to find Amarthiel here, do you? This keep belongs to us now."] = {5, "Adds enter on boss HP, and at 1 minute&2minutes", nil, false, nil, false, nil},
    }

    local ABrogadan = {
        ["Greetings, intruders. You are a bit late for the festivities, but perhaps I can fill you in."] = {5, "Kill Limrafn, Avoid Flames!", nil, false, nil, false, nil},
    }

    local ARemmenaeg = {
        ["Welcome. You have made it quite far, but now your journey has come to its end."] = {5, "Left = Bats! Right = Wights! Far = Spirit!", nil, false, nil, false, nil},
    }

    -------------------------------------------------------------------------------------------------------------------------------------
    -- Barad Guluran --------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------

    local AUdunion = {
        ["Shadow take you!"] = {6, "Staff of Dark Shadows - LEFT SIDE!", nil, 6, nil, false, nil},
        ["Burn!"] = {6, "Horn of Anor - FAR SIDE!", nil, 6, nil, false, nil},
        ["Come dark fire!"] = {6, "Scroll of Ruin - RIGHT SIDE!", nil, 6, nil, false, nil},
    }

    -------------------------------------------------------------------------------------------------------------------------------------
    -- Rift -----------------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------

    local ATakrip = {
        ["I will put an end to you, vermin!"] = {6, "Interrupt Slave-master", InterruptIconSmall, true, SoundEffects["Hornq"], false, nil},
        ["Hah hah hah! Now Master not near to tell me be careful!"] = {4, "Stack on keeper!", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
        ["You dead now, little squishies, me swing real hard this time!"] = {4, "Stack on keeper!", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
        ["No problem smashing when Master far away!"] = {4, "Stack on keeper!", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
    }

    local ABlogmal = {
        ["I will put an end to you, vermin!"] = {6, "Interrupt Slave-master", InterruptIconSmall, true, SoundEffects["Hornq"], false, nil},
        ["Hah hah hah! Now Master not near to tell me be careful!"] = {4, "Stack on keeper!", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
        ["You dead now, little squishies, me swing real hard this time!"] = {4, "Stack on keeper!", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
        ["No problem smashing when Master far away!"] = {4, "Stack on keeper!", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
    }

    local AZurm = {
        ["My trolls will deal with you!"] = {6, "Troll wave [Right]", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
        ["Slaves, destroy these intruders, or you will feel my lash!"] = {6, "Slave wave [Left]", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
        ["Fools! Aid your master...destroy them!"] = {6, "Slave-master wave [Right]", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
    }

    local ABarz = {
        ["I will bring this cave down upon you!"] = {4, "Move out of cave-in", nil, false, SoundEffects["Hornq"], false, nil},
        ["I'll gnaw your bones!"] = {4, "Move away!", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
        ["Let us see how you fare against one of my trolls!"] = {6, "Troll wave [Left]", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
        ["Come, slaves. Deal with these worms!"] = {6, "Slave wave [Right]", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
        ["Quickly! Destroy them! Barz commands it!"] = {6, "Slave-master wave [Left]", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},
        ["No power is stronger than Barz!"] = {4, "Random power-drain Keep distance", nil, false, SoundEffects["Hornq"], false, nil}
    }

    local AGlathirel = {
        ["Strangers... you were sent to me by the Eldgang?"] = {4, "Enjoy the RP", nil, false, nil, false, nil},
        ["When I open this seal, the true test of your will shall begin!"] = {4, "Time to wake up", nil, false, nil, false, nil},
    }

    -------------------------------------------------------------------------------------------------------------------------------------
    -- Fil Gashan -----------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------

    local ASentry = {
        ["Sound the alarm!"] = {5, "Use uniforms to not alert the guards!", nil, false, nil, false, nil},
        ["To arms!"] = {5, "Use uniforms to not alert the guards!", nil, false, nil, false, nil},
        ["Intruders!"] = {5, "Use uniforms to not alert the guards!", nil, false, nil, false, nil},

        --Chief Guard
        ["You shall not pass!"] = {5, "Put on his uniform to be invisible!", nil, false, nil, false, nil},
        ["You're no Orc!"] = {5, "Put on his uniform to be invisible!", nil, false, nil, false, nil},
        ["You can't fool me… !"] = {5, "Put on his uniform to be invisible!", nil, false, nil, false, nil},
        ["An intruder! Get him!"] = {5, "Put on his uniform to be invisible!", nil, false, nil, false, nil},

        --Lieutenant
        ["How dare you come here!?"] = {5, "Put on his uniform to be invisible!", nil, false, nil, false, nil},
        ["Do you think me a fool?"] = {5, "Put on his uniform to be invisible!", nil, false, nil, false, nil},
        ["Hey! You're not an officer!"] = {5, "Put on his uniform to be invisible!", nil, false, nil, false, nil},
        ["Prepare the Warband Manoeuvre!"] = {3, "PREPARE THE WARBAND MANOEUVRE!", nil, false, nil, false, nil},
    }

    local AVrarz = {
        ["Would you like that pan-seared?"] = {3, "Stun & Wound Incoming!", nil, false, nil, false, nil},
        ["Have you tried the special?"] = {8, "INTERRUPT or move away!!!", nil, 8, nil, false, nil},
    }

    local AGreb = {
        ["No more games! Prepare to fall to my flaming axe!"] = {3, "DONT APPLY MELEE DOTS!", nil, false, nil, false, nil},
        ["Now you shall witness my defences..."] = {3, "Adds Entering!", nil, false, nil, false, nil},
        ["The General will avenge me!"] = {5, "Collect Key from chest!", nil, false, nil, false, nil},
        ["Ha! I'll return your puny blows!"] = {30, "200% MELEE REFLECT!!!!", AlertIconSmall, 30, SoundEffects["Hornq"], false, nil},
    }

    local ATalug = {
        ["You have arrived at the perfect time!"] = {5, "Phase 1, Pull Into traps", nil, false, nil, false, nil},
        ["You have the honour of witnessing the power of my warriors before Lothlórien!"] = {5, "Phase 2, Kill fire-first next to him and then trap him", nil, false, nil, false, nil},
        ["Show them how you'll kill those wretched Elf Archers!"] = {5, "Phase 3, Nuke him!", nil, false, nil, false, nil},
        ["Mind your legs!"] = {4, "INTERRUPT or move away!!!", nil, 4, nil, false, nil},
    }

    local AFang = {
        ["Mind your feet…."] = {3, "ARCHER PLACING TRAP!", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},

        --fire-fist
        ["Aaaiiiieee!"] = {3, "Kill fire-first next to boss, then drag into trap!", nil, false, nil, false, nil},
    }

    -------------------------------------------------------------------------------------------------------------------------------------
    -- Grand Stairs ---------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------

    local ANardur = {
        ["Give me some room!"] = {2.5, "Knockback incoming!!", nil, 2.5, nil, false, nil},
    }
    local AGothghaash = {
        ["Wreathed in flame I'll watch you burn yourself!"] = {8, "Melee REFLECT!!", AlertIconSmall, 8, nil, false, nil},
    }

    local AIgash = {
        ["Let the darkness crush you!"] = {2.5, "AoE stun!!", AlertIconSmall, 2.5, nil, false, nil},
    }


    -------------------------------------------------------------------------------------------------------------------------------------
    -- DN -------------------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------

    local ABlagh = {
        ["Brother! We are together, let us strike in unison!"] = {4, "Split trolls!", nil, false, nil, false, nil},
        ["I am here, Brother! You take the weak one and I'll take the ugly one!"] = {4, "Split trolls!", nil, false, nil, false, nil},
        ["I am with you Brother! We shall punish them together!"] = {4, "Split trolls!", nil, false, nil, false, nil},
        ["His death will mean your death!"] = {4, "Mushroom phase", nil, false, nil, false, nil}
    }

    local ARung = {
        ["Brother! We are together, let us strike in unison!"] = {4, "Split trolls!", nil, false, nil, false, nil},
        ["I am here, Brother! You take the weak one and I'll take the ugly one!"] = {4, "Split trolls!", nil, false, nil, false, nil},
        ["I am with you Brother! We shall punish them together!"] = {4, "Split trolls!", nil, false, nil, false, nil},
        ["His death will mean your death!"] = {4, "Mushroom phase", nil, false, nil, false, nil}
    }

    local AZholunga = {
        ["Mistress, lend me your courage!"] = {4, "DO NOT touch the runestones", nil, false, nil, false, nil}
    }

    local AFlagit = {
        ["The Mistress' Army is too strong! We were bred stronger and tougher!"] = {4, "Beware of eyes", nil, false, nil, false, nil},
        ["You can not stop Her Army! I will not allow it!"] = {4, "Bubble phase!", nil, false, nil, false, nil},
        ["The light... it dims... what are you doing? I must protect myself!"] = {4, "FASTER BUBBLES", nil, false, nil, false, nil}
    }

    local AIstum = {
        ["You are too strong! Too many! It is good I am here to stop you!"] = {4, "Middle worm spawn", nil, false, nil, false, nil},
        ["What is this? The Mistress ordered me not to fail! Death to all!"] = {4, "Corner worm spawn", nil, false, nil, false, nil}
    }


    -------------------------------------------------------------------------------------------------------------------------------------
    -- Sammath Gul ----------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------

    local AUrchir = {
        ["Can you handle the burn"] = {6, "Stack up or Line of Sight!", AlertIconSmall, false, SoundEffects["Hornq"], false, nil},

        --Alagossir
        ["The storms are gathering quickly."] = {5, "Stay In Melee Range!", nil, false, nil, false, nil},
        ["The storm's pace hastens. It is only a matter of time now."] = {5, "Hug left brazier to avoid lightning!", nil, false, nil, false, nil},
        ["You stray too far, !"] = {5, "Get in Melee Range!", AlertIconSmall, false, SoundEffects["Hornq"], false, nil}
    }

    local AGorothul = {
        ["From this point forward, I will teach you the true meaning of terror!"] = {5, "Dont Kill Guldur-Spirits", nil, false, nil, false, nil},
        ["Let terror take you!"] = {3, "INCOMING AOE FEAR!", nil, false, nil, false, nil},      
    }

    -------------------------------------------------------------------------------------------------------------------------------------
    -- Sword Halls ----------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------
        
    local AUrcheron = {
    ["You will find your end soon enough!"] = {5, "Interupt, Move out of Flames", nil, false, nil, false, nil},

    }

    local ACarchrien = {
    ["Hahahaha"] = {5, "Careful of bat stuns!", nil, false, nil, false, nil},
    ["Mmm come to play?"] = {5, "Careful of bat stuns!", nil, false, nil, false, nil},
    }
        
    -------------------------------------------------------------------------------------------------------------------------------------
    -- Dungeons of Dol Guldur -----------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------

    local AGuldur = {
        ["Time to nap!"] = {60, "Sleeping Troll!", nil, 60, nil, false, nil, 60, "Sleeping Troll!", RolePlay},
        ["Maybe here is good place to rest."] = {60, "Sleeping Troll!", nil, 60, nil, false, nil, 60, "Sleeping Troll!", RolePlay},
        ["Why so tired now?"] = {60, "Sleeping Troll!", nil, 60, nil, false, nil, 60, "Sleeping Troll!", RolePlay},
        ["Nightshade... sleep."] = {60, "Sleeping Troll!", nil, 60, nil, false, nil, 60, "Sleeping Troll!", RolePlay},
    }
        
    -------------------------------------------------------------------------------------------------------------------------------------
    -- Barad Guldur ---------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------

    local ADurchest = {
        ["Such a shame to come so far only to fail."] = {5, "Remove Corruptions on boss", nil, false, nil, false, nil},
        ["Cowards! Cheats! We will not be lured away from our abode so easily!"] = {5, "Seems you dropped the soap! Have fun!", nil, false, nil, false, nil},
        ["Soldier! Front and centre!"] = {5, "Add Entering", nil, false, nil, false, nil},
        ["You! Finish these puny creatures!"] = {5, "Add Entering", nil, false, nil, false, nil},
    }

    local AUmbarrim = {
        ["You will betray your friends this day, !"] = {5, "Betray target, Move Out of Group!", nil, false, nil, false, nil},
        ["Burn, !"] = {5, "Burn Target, Move Into Group", nil, false, nil, false, nil},
    }

    local ACargaraf = {
        ["Away with you!"] = {3, "Move away from Woman!", nil, false, nil, false, nil}, 
    }

    -------------------------------------------------------------------------------------------------------------------------------------
    -- Throne ---------------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------
        
    local AGothmog ={
        ["The fires of Mordor come for you!"] = {30, "MORGUL FLAMES!", nil, false, nil, 8, nil}
    }

    -------------------------------------------------------------------------------------------------------------------------------------
    -- AD -------------------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------
    local ARanulur = {
        --["The flames will burn you all!"] = {4, "Run to cover!", nil, false, nil, false, nil},
        ["Children! To me!"] = {4, "Adds!", WolfIcon, false, SoundEffects["Adds"], 7, nil, 45, "Adds", WolfIcon},
        ["Your father needs you!"] = {4, "Adds!", WolfIcon, false, SoundEffects["Adds"], 7, nil, 45, "Adds", WolfIcon},
        
    }

    local APrinceDurin = {
        ["Heroes! To me!"] = {4, "Hide!", AlertIconSmall, false, SoundEffects["hide"], 7, nil, 70, "Flames", FireShield},
        ["Stand with us! We will shield you from the flames!"] = {4, "Hide!", AlertIconSmall, false, SoundEffects["hide"], 7, nil, 70, "Flames", FireShield},
    }

    -------------------------------------------------------------------------------------------------------------------------------------
    -- Stairs ---------------------------------------------------------------------------------------------------------------------------
    -------------------------------------------------------------------------------------------------------------------------------------

    local AShatarkha = {
        ["Chill them to the core."] = {4, "Frontal!", AlertIconSmall, false, SoundEffects["runout"], 10, nil},
        ["May your blood run cold."] = {3, "Horn", AlertIconSmall, false, SoundEffects["Defensive"], 10, nil},
    }

    local AMaugrash = {
        ["My blade will bathe in blood."] = {3, "Bleed", BloodBoilIcon, false, SoundEffects["Defensive"], 10, nil}, --bleed,
        ["All those before me shall fall."] = {3, "Frontal!", nil, false, SoundEffects["runout"], 10, nil}, --frontal
        ["Blood shall cover the field."] = {4, "Interrupt Maugrash", InterruptIconSmall, false, SoundEffects["Interrupt"], 10, nil}
    }

    local AAzbauz = {
        ["They will fall."] = {4, "Interrupt Azbaus", InterruptIconSmall, false, SoundEffects["Interrupt"], 10, nil},
        ["Bleed… slowly."] = {3, "Bleed", BloodBoilIcon, false, SoundEffects["Defensive"], 10, nil}
    }

    --Groz-and-Ulk

    local AGrozUlk = {
        ["This will hurt."] = {4, "Interrupt", InterruptIconSmall, false, nil, 10, nil},
        ["Crush them to bits."] = {4, "Interrupt", InterruptIconSmall, false, nil, 10, nil},
        ["Smash all of them to bits."] = {4, "Interrupt", InterruptIconSmall, false, nil, 10, nil},
        ["Smash, smash, smash."] = {4, "Interrupt", InterruptIconSmall, false, nil, 10, nil},
        ["Smash those little things."] = {4, "Interrupt", InterruptIconSmall, false, nil, 10, nil},
        ["Incompetent oafs! Crush and smash them!"] = {nil, nil, nil, nil, nil, nil, nil, 30, "RP", RolePlay}
    }

    local AGulthava = {
        ["This place the dwarves called"] = {nil, nil, nil, nil, nil, nil, nil, 38, "RP", RolePlay}
    }

    local ARakothas = {
        ["innocent"] = {4, "Interrupt", InterruptIconSmall, false, nil, 8, 3, 10, "TestTimer", 1090551617, true, false, nil},
    }--1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 windowID

    --Smashed Earth - 2

    local AllChats = {
        -- Minas Morgul 3 mans
        ["Shaktur"] = AShaktur,
        ["Dargásh"] = ADargash, -- Dargásh -> sh
        ["Khúrthak the Rent"] = AKhurtakTheRent, -- Khúrthak -> rthak

        -- Anvil
        ["Isvítha the Gluttonous"] = AIsvitha, -- Isvítha -> tha

        ["Kúzek Squint-Eye"] = AKuzek, -- Kúzek -> zek
        ["Brúnek I Clovenbow"] = ABrunek, -- Brúnek -> nek
        ["Dóbruz IV the Unheeding"] = ADobruzUnheed, -- Dóbruz -> bruz
        ["Dóbruz II Stark-heart"] = ADobruzStark,
        ["Óiko I the Bellower"] = AOikoBell, -- Óiko -> iko
        ["Óiko II Rill-Seeker"] = AOikoRill,
        ["Brántokh II the Sunderer"] = ASunderer,

        ["Karazgar the Weeping Warrior"] = AKarazgar,
        ["Vethúg Wintermind"] = AVethug, -- Vethúg -> g

        ["Hrímil Frost-heart"] = AHrimil, --Hrímil -> mil

        ["Manôzagar"] = AManozagar, -- Manôzagar -> zagar
        ["Narkhor the Flensed"] = ANarkhor,
        ["Agath-kali, the Gaunt-woman"] = AKali, --Agath-kali -> kali
        ["Dolguzigir, the Dark Archivist"] = ADolgulzigir,
        ["Skorgrím"] = ASkorgrim,

        ["The Witch-king"] = ATheWK,
        ["The Shadowed King"] = ATheSK,
        ["The Blind One "] = ATheBO,
        ["The Mistress of Pestilence"] = ATheMOP,

        -- Remmorchant
        ["Bratha Tasakh, Lady of Many Venoms"] = ABratha,
        ["Rûkhor the Pale Herald"] = ARukhor,
        ["Captain Zabothak"] = AZabothrak,
        ["Atli Spider-bane"] = AAtli,

        -- Great Barrows
        ["Gaerdring"] = AGaerdring,
        ["Thadúr"] = AThadur,

        -- Garth Agarwen
        ["Ivar"] = AIvar,
        ["Dúnlang"] = ADunlang,
        ["Vatar"] = AVatar,
        ["Naruhel"] = ANaruhel,

        -- Woe of the Willow
        ["Gulmúk Dampmould"] = AGulmak,

        -- Narrowdelve
        ["Rúcrog Hoarpelt"] = ARucrog,
        ["Aigrith the Ashen"] = AAigrith,
        ["Storm of the Witch-King"] = AStorm,

        -- Fornost
        ["Megoriath"] = AMegoriath,
        ["Warchief Búrzghâsh"] = AWarchief,
        ["Rhavameldir"] = ARhavameldir,
        ["Brogadan"] = ABrogadan,
        ["Remmenaeg"] = ARemmenaeg,

        -- Barad Guluran
        ["Udúnion"] = AUdunion,

        -- KK
        ["Gultháva, Maiden of Gladden"] = AGulthava,

        -- Rift
        ["Tarkrîp Slave-master"] = ATakrip, -- Slave master and troll
        ["Tarkrîp Cave-troll"] = ATakrip, -- Slave master and troll
        ["Tarkrîp Troll-keeper"] = ATakrip, -- Slave master and troll
        ["Blogmal Slave-master"] = ABlogmal, -- Slave master and troll
        ["Blogmal Cave-troll"] = ABlogmal, -- Slave master and troll
        ["Blogmal Troll-keeper"] = ABlogmal, -- Slave master and troll

        ["Zurm"] = AZurm,
        ["Barz"] = ABarz,

        --["Frûz"] = AFruz,

        ["Glathlírel "] = AGlathirel,

        -- Fil Gashan
        ["Moria"] = ASentry, --Chief Guard, --Leuitenant,
        ["Vrarz the Cook"] = AVrarz,
        ["Commander Greb"] = AGreb,
        ["General Talug"] = ATalug,
        ["Talug's Fire-fist"] = AFang, --Fire-Fist

        -- Grand Stairs
        ["Nardur the Shield"] = ANardur,
        ["Gothghaash the Firecaller"] = AGothghaash,
        ["Igash the Fanatic"] = AIgash,

        -- DN
        ["Blagh"] = ABlagh,
        ["Rung"] = ARung,

        ["Zholuga"] = AZholunga,
        ["Flâgît"] = AFlagit,

        ["Îstum"] = AIstum,

        -- Sammath Gul
        ["Alagossír"] = AUrchir, --Alagossir
        ["Gorothúl"] = AGorothul,    

        -- Sword Halls
        ["Urcheron"] = AUrcheron,
        ["Carchrien"] = ACarchrien,

        -- Dungeons of Dol Guldur
        --["Guldur"] = AGuldur,

        -- Barad Guldur
        ["Dúrchest"] = ADurchest,
        ["Umbarrim Sorcerer"] = AUmbarrim,
        ["Cargaraf"] = ACargaraf,

        -- Throne of the Dread Terror
        ["Gothmog"] = AGothmog,

        -- 
        ["Ránulur, Chieftain of Wargs"] = ARanulur,
        ["Prince Durin"] = APrinceDurin,

        ["Azbauz"] = AAzbauz,
        ["Maugrush"] = AMaugrash,
        ["Shatarkha"] = AShatarkha,
        ["Groz"] = AGrozUlk,
        ["Ulk"] = AGrozUlk,
        ["Groz-and-Ulk"] = AGrozUlk,

        ["Rakothas"] = ARakothas,
    }

    return AllChats
    --]]
end

function EventWindowsHandler:GetBaseSelfEffectData()
    --[[
     -- All effect added alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 5_countdown, 7 instanceID, 8 role 9 timer duration 10 timer text, 11 timer icon, 12 unique, 13 loop, 14 tiering effect}
    return {
        --["Hemorrhage - 3"] = {10, "Applied hemorrhage tier 3, swap aggro", false, false},
        ["Immolated"] = {3, Wounds, CureIcon, false, SoundEffects["Beepq"], false},
        ["Lingering Frost"] = {2, "Puddle!", PuddleIcon, false, SoundEffects["Horn"], false }, -- TG puddles
        --["Dread"] = {10, "Test", false, false, false },
        
        -- Anvil
        ["Flight"] = {45, "PURPLE EYE!", PurpleEye, true, SoundEffects["Run"], false },
        ["Clinging Frost"] = {60, "Ice drake debuff care",  true, SoundEffects["Alarmq"], false },
        ["Vethúg's Revenge"] = {15, Puddle, PuddleIcon, true, SoundEffects["Horn"], false },
        ["Frozen Solid"] = {30, "Frozen", true, SoundEffects["Beeps"], false },
        
        -- PVMP
        ["Blight"] = {3,"Blight", PuddleIcon, false, SoundEffects["Beepq"], false }, -- defiler
    
        -- Great Barrows
        ["Dissolving"] = {3, "Move out of puddle!!", true, false, SoundEffects["AlarmQ"], false },
        ["Virulent Disease"] = {3, "Move out of puddle!!", true, false, SoundEffects["AlarmQ"], false },
        ["Terrifying Wind"] = {3, "Move out of puddle!!", true, false, SoundEffects["AlarmQ"], false },
    
        -- Woe of the Willow
        ["Poisoned Earth"] = {3, "Move out of puddle!", true, false, SoundEffects["AlarmQ"], false },
        ["Wrathful Roots"] = {3, "Move out of puddle!", true, false, SoundEffects["AlarmQ"], false },
    
        -- Narrowdelve
        ["Flaming Ground"] = {3, "Move out of puddle!", true, false, SoundEffects["AlarmQ"], false },
        ["Strike Point"] = {3, "Move out of puddle!", true, false, SoundEffects["AlarmQ"], false },
        ["Bright Flare"] = {3, "Cleanse in water!", true, false, SoundEffects["AlarmQ"], false },
    
        -- Fornost
        ["Burning Oil"] = {3, "Move out of Fire!", true, false, SoundEffects["AlarmQ"], false },
    
        -- Carn Dum
        ["Faltering Resolve"] = {3, "Kill add to lose debuff!", true, false, SoundEffects["AlarmQ"], false },    
    
        --["Water-lore"] = {15, "Parasites on you!", true, true, nil, false },
    
        -- Sammath Gul
        ["Burning"] = {3, "Cure Wound!", true, false, SoundEffects["Hornq"], false },
        ["Harrassing Spirit"] = {15, "Major Debuff!", false, false, SoundEffects["AlarmQ"], false },
        ["Slow Burn"] = {10, "Move Away From Group!", false, false, SoundEffects["AlarmQ"], false },
        ["Immolation"] = {3, "Move out of flame!", false, false, SoundEffects["AlarmQ"], false },
        ["Running Scared"] = {10, "Feared", false, 10, SoundEffects["AlarmQ"], false },
        ["Acidic Flame"] = {3, "Move out of Green Flame! Follow it!", false, false, SoundEffects["AlarmQ"], false },
        -- All effect added alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 5_countdown, 7 instanceID, 8 role}
        -- Remmorchant
        ["Arcing Lightning"] = {15, "Lightning!", AlertIcon, true, SoundEffects["runout"], false, 1},
        ["Sticky Venom"] = {30, "DO NOT POT", AlertIcon, true, SoundEffects["Beepq"], false, 1},
        ["Fiery Combustion"] = {15, "Move out of the group!", AlertIcon, true, SoundEffects["runout"], false, 1},
        ["Acidic Webs"] = {3, "Move out of web", AlertIcon, false,nil, false, 1},
        ["Deliquesce"] = {3, "Puddle!", AlertIcon, false, nil, false, 1},
        ["Acidic Ichor"] = {3, "Puddle!", false, nil, false, 1},
        ["Corrosive Spittle"] = {3, "Puddle!", false, nil, false, 1},
        ["Necrotic Ooze"] = {30, "Drain shield", nil, true, nil, false, 1},
        ["Hail of Webs"] = {6, "Remove outside of group!", AlertIcon,  false, nil, false, 1, nil, 40, "Hail", true, true},
        ["Smouldering Wrath"] = {15, "Wrath on you", CureIcon, true, SoundEffects["runout"], false, 1},
        ["Dripping Ichor"] = {15, "Puddle on you!", CureIcon, true, SoundEffects["runout"], false, 1},
        ["Parasites"] = {15, "Parasites on you!", AlertIcon, true, SoundEffects["Beep"], false, 1},
        ["Stung!"] = {4, "Remove disease!", true, false, SoundEffects["Beep"], false, 1},
        
        ["Life Drain"] = {nil, nil, nil, nil, nil, nil, 1, nil, 40, "~Life drain", true, false},

        -- DN
        ["Poisoned Ground"] = {3, "Move out!", false, false, SoundEffects["Hornq"], false, 2},
        ["Killing Field"] = {3, "Move out!", false, false, SoundEffects["Hornq"], false, 2},
        ["Fighter's Bane"] = {3, "Kill Limrafn", false, false, nil, false, 2},
        ["Defender's Bane"] = {3, "Kill Limrafn", false, false, nil, false, 2},
        ["Caster's Bane"] = {3, "Kill Limrafn", false, false, nil, false, 2},
        ["Impending Mark: Flâgît's Ire"] = {6, "Move out of room!", true, true, SoundEffects["Hornq"], false, 2},
        ["Mark: Flâgît's Ire"] = {10, "You done fucked up A A ron", true, true, SoundEffects["Hornq"], false, 2},
        ["Disintegration"] = {3, "Move out!", false, false, SoundEffects["Hornq"], false, 2},
        ["Impending Poison"] = {3, "Cure!", false, false, SoundEffects["Hornq"], false, 2},
        ["Suppressing Shadows"] = {5, "Move out!", false, false, SoundEffects["Hornq"], false, 2},
        ["Shattered Armour"] = {3, "Cure Wound!", true, false, SoundEffects["Hornq"], false, 2},

        -- Rift
        ["Cave In"] = {3, "Move out of Cave In", true, false, SoundEffects["AlarmQ"], false, 3},
        ["Melted Armour"] = {3, "Remove wound!", false, false, SoundEffects["Alarmq"], false, 3},
        ["Mortal Wound"] = {3, "Remove wound!", false, false, SoundEffects["Alarmq"], false, 3},
        --["Melted Armour"] = {3, "Remove wound!", false, false, SoundEffects["Alarmq"], false, 3}, miss fear
        --["Melted Armour"] = {3, "Remove wound!", false, false, SoundEffects["Alarmq"], false, 3}, thaurlauch fear
        ["Fatal Affliction"] = {3, "Cure poison!", false, false, SoundEffects["Alarmq"], false, 3},
        ["Blinding Dark"] = {3, "Remove fear!", false, false, SoundEffects["Alarmq"], false, 3},
        ["Marked Prey"] = {3, "Remove fear!", false, false, SoundEffects["Alarmq"], false, 3},
        
        -- Creeps
        --["Smouldering Wrath", nil)

        -- Throne of the Dread Terror
        ["Pull of the Void"] = {5, "Void!", false, 5, SoundEffects["AlarmQ"], false, 8},
        ["Engulfed in Flame"] = {3, "Move out of Flames!!", false, 3, SoundEffects["AlarmQ"], false, 8},
    
        -- AD
        ["Rowdy Crowd"] = {12, "Volley on You!", true, false, SoundEffects["runout"], true, nil, false, 7},
        ["Massive Javelin"] = {7, "Javelin on You!", true, true, SoundEffects["runout"], false, nil, false, 7},
        ["Gorgar's Target"] = {4, "Charge on You! ", true, true, SoundEffects["Stack"], false, nil, false, 7},

        ["Bloody Wound"] = {4, "Bloody Wound on you", true, false, false, false, 7, nil, 60, "<name>", true, false},
        ["Infected Bite"] = {6, "Disease on you ", true, false, false, false, 7},
    
        ["Burning Javelin"] = {3, "Move Out!", true, false, SoundEffects["runout"], true, nil, false, 7},

        -- Harrowing
        ["Flaming Pitch"] = {5, Puddle, PuddleIcon, true, SoundEffects["Beepq"], false, 11},
        ["Volley"] = {5, "Dropping volley aoe", PuddleIcon, true, SoundEffects["Beepq"], false, 11},

        -- Turingwraith 
        ["Shadow of Doubt"] = {9, "Dropping puddle", PuddleIcon, true, SoundEffects["Doubleq"], false, 12}, 
        ["Blood-burst"] = {9, "Aura!", true, SoundEffects["Doubleq"], false, 12},

        -- Roost
        ["Carrying a fell beast egg"] = {10, "You better YEET that egg quickly!", false, false, nil, false, 14},
        ["Acid Puddles"] = {15, Puddles, PuddleIcon, true, nil, false, 14, false, 15, "Puddles", nil, true},

        -- Black Lore
        ["Brewing Storm"] = {3, "Use horn!", false, SoundEffects["Alarms"], false, 15}, -- Nurzum
        ["Scorching Rage"] = {15, "Dropping puddle", PuddleIcon, true, SoundEffects["Horns"], false, 15}, --Black lore final boss
        ["Blind Hatred"] = {4, "Green eye!", PurpleEye, false, SoundEffects["Alarm"], false, 15},

        -- Fallen kings
        ["Encroaching Darkness"] = {10, "Dropping puddle!", PuddleIcon, true, SoundEffects["Hornq"], false, 17}, -- Black blade
        ["Touch of Sorcery"] = {11, "Aoe eye!", PurpleEye, true, SoundEffects["Doubleq"], false, 17}, --Throne aoe eye
        ["Deepening Gloom"] = {2, "Puddle!", PuddleIcon, false, SoundEffects["Doubleq"], false, 17}, --Gloom puddle
        ["Gloom"] = {10, "Puddle!", PurpleEye, true, SoundEffects["Horns"], false, 17}, -- Gloom puddle dot
        ["Disregarded"] = {10, "Threat Wipe!", AlertIcon, true, SoundEffects["AHorn"], false, 17}, -- Witch king + filth
        ["Spreading Fear"] = {7, "Dropping aoe", PurpleEye, true, SoundEffects["Hornq"], false, 17},

        -- Barad Guldur
        ["Frail"] = {10, "Armour Wipe!", false, false, SoundEffects["AlarmQ"], false, 18},
        ["Traumatic Wound"] = {15, "Reduced Max HP", false, false, SoundEffects["AlarmQ"], false, 18},
        ["Insignificance"] = {50, "Reduced Threat", false, false, SoundEffects["AlarmQ"], false, 18},
        ["Losing Threat"] = {10, "Reduced Threat Tier Up!", false, false, SoundEffects["AlarmQ"], false, 18},
        ["Gaining Threat"] = {10, "Gaining Threat from Axe!", false, false, SoundEffects["AlarmQ"], false, 18},
        ["Bottled Shadow"] = {10, "Dropping Puddle!", false, false, SoundEffects["AlarmQ"], false, 18},
        ["Canned Heat"] = {10, "Dropping Puddle!", false, false, SoundEffects["AlarmQ"], false, 18},
        ["Combustion"] = {10, "Move out of group!", false, false, SoundEffects["AlarmQ"], false, 18},
        ["Residual Heat"] = {5, "Debuff Tiered Up!", false, false, SoundEffects["AlarmQ"], false, 18},
        ["Terror"] = {4, "FEAR!", false, 4, SoundEffects["AlarmQ"], false, 18},
        ["Fixate"] = {26, "Kite Fell-beast!", false, false, SoundEffects["AlarmQ"], false, 18},
        ["Seed of Dissent"] = {15, "Drop out of group!", false, false, SoundEffects["AlarmQ"], false, 18},
        ["Frost-Fingered Fear"] = {8, "FEAR!", false, 8, SoundEffects["AlarmQ"], false, 18},

        -- Warg pens
        ["Lethargy"] = {3, "Cure Debuff in Campfire!", true, false, SoundEffects["Hornq"], false, 20},

        --Forges    
        ["Deadly Inferno"] = {3, "Move out of fire!!", true, false, nil, false, nil, false, 24},
        ["Ultimate Inferno"] = {3, "Move out of fire!!", true, false, nil, false, nil, false, 24},
        ["Black Powder Burns"] = {8, "Use Water Pouch!", false, 8, SoundEffects["AlarmQ"], false, 24},


        --Grand Stairs
        ["Terrified"] = {3, "Remove fear!", false, false, SoundEffects["Alarmq"], false, 25},

        -- Filkul 
        ["Profuse Bleeding"] = {5, "Melee dot tier up!", false, false, SoundEffects["Hornq"], false, 30},
        ["Gushing Wound"] = {5, "Tactical dot tier up!", false, false, SoundEffects["Hornq"], false, 30},

        -- Woe of the willow
        ["Corrupted Pool"] = {5, "Dropping puddle!", true, false, SoundEffects["Ping"], false, 33},

        -- FOKD
        ["Mark of Shadow"] = {30, "Mark on You!", nil, true, SoundEffects["Run"], false, 38},

        -- Shadowed king
        ["Shadowed Wound"] = {6, "Shadowed Wound on: ", true, false, false, false, 36},
    -- All effect added alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 5_countdown, 7 instanceID, 8 role 9 timer duration 10 timer text, 11 timer icon, 12 unique, 13 loop, 14 tiering effect}
        --["Writ of Health - Tier "] = {6, "Shadowed Wound on: ", true, false, false, false, nil, nil, 15, "<name>", true, true, false, true},
    
    }
    --]]
    -- [[
    return 
        {
            ["Vethúg's Revenge"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 15.000000,
                        [2.000000] = "Dropping puddle!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Puddle.jpg",
                        [4.000000] = true,
                        [5.000000] = "alarmbuzzer"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Lethargy"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Cure Debuff in Campfire!",
                        [3.000000] = true,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 20.000000
                }
            },
            ["Encroaching Darkness"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "Dropping puddle!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Puddle.jpg",
                        [4.000000] = true,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 17.000000
                }
            },
            ["Acidic Webs"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of web",
                        [3.000000] = "PrimePlugins/RaidTools/Images/AlertRound.tga",
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Touch of Sorcery"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 11.000000,
                        [2.000000] = "Aoe eye!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Eye.jpg",
                        [4.000000] = true,
                        [5.000000] = "alarmdouble_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 17.000000
                }
            },
            ["Disregarded"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "Threat Wipe!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/AlertRound.tga",
                        [4.000000] = true,
                        [5.000000] = "AirHorn"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 17.000000
                }
            },
            ["Flaming Ground"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of puddle!",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Massive Javelin"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 7.000000,
                        [2.000000] = "Javelin on You!",
                        [3.000000] = true,
                        [4.000000] = true,
                        [5.000000] = "runout"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 7.000000
                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Marked Prey"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Remove fear!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzz_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 3.000000
                }
            },
            ["Volley"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 5.000000,
                        [2.000000] = "Dropping volley aoe",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Puddle.jpg",
                        [4.000000] = true,
                        [5.000000] = "alarmbeep_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 11.000000
                }
            },
            ["Immolation"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of flame!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Deadly Inferno"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of fire!!",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 24.000000
                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Dissolving"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of puddle!!",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Killing Field"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 2.000000
                }
            },
            ["Mark of Shadow"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 30.000000,
                        [2.000000] = "Mark on You!",
                        [4.000000] = true,
                        [5.000000] = "runaway"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 38.000000
                }
            },
            ["Wrathful Roots"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of puddle!",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Sticky Venom"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 30.000000,
                        [2.000000] = "DO NOT POT",
                        [3.000000] = "PrimePlugins/RaidTools/Images/AlertRound.tga",
                        [4.000000] = true,
                        [5.000000] = "alarmbeep_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Gaining Threat"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "Gaining Threat from Axe!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Virulent Disease"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of puddle!!",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Seed of Dissent"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 15.000000,
                        [2.000000] = "Drop out of group!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Bloody Wound"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 4.000000,
                        [2.000000] = "Bloody Wound on you",
                        [3.000000] = true,
                        [4.000000] = false,
                        [5.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 60.000000,
                        [2.000000] = "<name>",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 7.000000
                }
            },
            ["Residual Heat"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 5.000000,
                        [2.000000] = "Debuff Tiered Up!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Frail"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "Armour Wipe!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Arcing Lightning"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 15.000000,
                        [2.000000] = "Lightning!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/AlertRound.tga",
                        [4.000000] = true,
                        [5.000000] = "runout"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Mark: Flâgît's Ire"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "You done fucked up A A ron",
                        [3.000000] = true,
                        [4.000000] = true,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 2.000000
                }
            },
            ["Deliquesce"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Puddle!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/AlertRound.tga",
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Defender's Bane"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Kill Limrafn",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 2.000000
                }
            },
            ["Burning Oil"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of Fire!",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Hail of Webs"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 6.000000,
                        [2.000000] = "Remove outside of group!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/AlertRound.tga",
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 40.000000,
                        [2.000000] = "Hail",
                        [3.000000] = true,
                        [4.000000] = true
                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Clinging Frost"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 60.000000,
                        [2.000000] = "Ice drake debuff care",
                        [3.000000] = true,
                        [4.000000] = "alarmbuzz_quiet",
                        [5.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Infected Bite"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 6.000000,
                        [2.000000] = "Disease on you ",
                        [3.000000] = true,
                        [4.000000] = false,
                        [5.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 7.000000
                }
            },
            ["Strike Point"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of puddle!",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Slow Burn"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "Move Away From Group!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Canned Heat"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "Dropping Puddle!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Stung!"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 4.000000,
                        [2.000000] = "Remove disease!",
                        [3.000000] = true,
                        [4.000000] = false,
                        [5.000000] = "alarmbeep"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Melted Armour"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Remove wound!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzz_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 3.000000
                }
            },
            ["Blinding Dark"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Remove fear!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzz_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 3.000000
                }
            },
            ["Faltering Resolve"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Kill add to lose debuff!",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Caster's Bane"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Kill Limrafn",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 2.000000
                }
            },
            ["Parasites"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 15.000000,
                        [2.000000] = "Parasites on you!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/AlertRound.tga",
                        [4.000000] = true,
                        [5.000000] = "alarmbeep"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Profuse Bleeding"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 5.000000,
                        [2.000000] = "Melee dot tier up!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 30.000000
                }
            },
            ["Cave In"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of Cave In",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 3.000000
                }
            },
            ["Life Drain"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 40.000000,
                        [2.000000] = "~Life drain",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Fighter's Bane"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Kill Limrafn",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 2.000000
                }
            },
            ["Disintegration"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 2.000000
                }
            },
            ["Suppressing Shadows"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 5.000000,
                        [2.000000] = "Move out!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 2.000000
                }
            },
            ["Poisoned Earth"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of puddle!",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Corrosive Spittle"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Puddle!",
                        [3.000000] = false,
                        [5.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Mortal Wound"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Remove wound!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzz_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 3.000000
                }
            },
            ["Bottled Shadow"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "Dropping Puddle!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Pull of the Void"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 5.000000,
                        [2.000000] = "Void!",
                        [3.000000] = false,
                        [4.000000] = 5.000000
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 8.000000
                }
            },
            ["Immolated"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "REMOVE WOUNDS!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/ClearWounds.jpg",
                        [4.000000] = false,
                        [5.000000] = "alarmbeep_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Traumatic Wound"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 15.000000,
                        [2.000000] = "Reduced Max HP",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Blight"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Blight",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Puddle.jpg",
                        [4.000000] = false,
                        [5.000000] = "alarmbeep_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Bright Flare"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Cleanse in water!",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Spreading Fear"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 7.000000,
                        [2.000000] = "Dropping aoe",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Eye.jpg",
                        [4.000000] = true,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 17.000000
                }
            },
            ["Shattered Armour"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Cure Wound!",
                        [3.000000] = true,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 2.000000
                }
            },
            ["Combustion"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "Move out of group!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Engulfed in Flame"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of Flames!!",
                        [3.000000] = false,
                        [4.000000] = 3.000000
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 8.000000
                }
            },
            ["Rowdy Crowd"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 12.000000,
                        [2.000000] = "Volley on You!",
                        [3.000000] = true,
                        [4.000000] = false,
                        [5.000000] = "runout"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 7.000000
                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = true
                }
            },
            ["Frozen Solid"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 30.000000,
                        [2.000000] = "Frozen",
                        [3.000000] = true,
                        [4.000000] = "alarmbeep_soft",
                        [5.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Impending Poison"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Cure!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 2.000000
                }
            },
            ["Gorgar's Target"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 4.000000,
                        [2.000000] = "Charge on You! ",
                        [3.000000] = true,
                        [4.000000] = true,
                        [5.000000] = "Stack"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 7.000000
                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Acidic Ichor"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Puddle!",
                        [3.000000] = false,
                        [5.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Poisoned Ground"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 2.000000
                }
            },
            ["Burning Javelin"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move Out!",
                        [3.000000] = true,
                        [4.000000] = false,
                        [5.000000] = "runout"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 7.000000
                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = true
                }
            },
            ["Terror"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 4.000000,
                        [2.000000] = "FEAR!",
                        [3.000000] = false,
                        [4.000000] = 4.000000
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Burning"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Cure Wound!",
                        [3.000000] = true,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Brewing Storm"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Use horn!",
                        [3.000000] = false,
                        [4.000000] = "alarmbuzz_soft",
                        [5.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 15.000000
                }
            },
            ["Dripping Ichor"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 15.000000,
                        [2.000000] = "Puddle on you!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/ClearWounds.jpg",
                        [4.000000] = true,
                        [5.000000] = "runout"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Smouldering Wrath"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 15.000000,
                        [2.000000] = "Wrath on you",
                        [3.000000] = "PrimePlugins/RaidTools/Images/ClearWounds.jpg",
                        [4.000000] = true,
                        [5.000000] = "runout"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Scorching Rage"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 15.000000,
                        [2.000000] = "Dropping puddle",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Puddle.jpg",
                        [4.000000] = true,
                        [5.000000] = "alarmbuzzer_soft"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 15.000000
                }
            },
            ["Fiery Combustion"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 15.000000,
                        [2.000000] = "Move out of the group!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/AlertRound.tga",
                        [4.000000] = true,
                        [5.000000] = "runout"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Flaming Pitch"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 5.000000,
                        [2.000000] = "Dropping puddle!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Puddle.jpg",
                        [4.000000] = true,
                        [5.000000] = "alarmbeep_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 11.000000
                }
            },
            ["Lingering Frost"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 2.000000,
                        [2.000000] = "Puddle!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Puddle.jpg",
                        [4.000000] = false,
                        [5.000000] = "alarmbuzzer"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Shadow of Doubt"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 9.000000,
                        [2.000000] = "Dropping puddle",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Puddle.jpg",
                        [4.000000] = true,
                        [5.000000] = "alarmdouble_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 12.000000
                }
            },
            ["Carrying a fell beast egg"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "You better YEET that egg quickly!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 14.000000
                }
            },
            ["Frost-Fingered Fear"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 8.000000,
                        [2.000000] = "FEAR!",
                        [3.000000] = false,
                        [4.000000] = 8.000000
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Acid Puddles"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 15.000000,
                        [2.000000] = "Dropping puddles!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Puddle.jpg",
                        [4.000000] = true
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 15.000000,
                        [2.000000] = "Puddles",
                        [4.000000] = true
                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 14.000000
                }
            },
            ["Terrifying Wind"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of puddle!!",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Acidic Flame"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of Green Flame! Follow it!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Harrassing Spirit"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 15.000000,
                        [2.000000] = "Major Debuff!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Gloom"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "Puddle!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Eye.jpg",
                        [4.000000] = true,
                        [5.000000] = "alarmbuzzer_soft"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 17.000000
                }
            },
            ["Blood-burst"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 9.000000,
                        [2.000000] = "Aura!",
                        [3.000000] = true,
                        [4.000000] = "alarmdouble_quiet",
                        [5.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 12.000000
                }
            },
            ["Necrotic Ooze"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 30.000000,
                        [2.000000] = "Drain shield",
                        [4.000000] = true
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 1.000000
                }
            },
            ["Impending Mark: Flâgît's Ire"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 6.000000,
                        [2.000000] = "Move out of room!",
                        [3.000000] = true,
                        [4.000000] = true,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 2.000000
                }
            },
            ["Corrupted Pool"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 5.000000,
                        [2.000000] = "Dropping puddle!",
                        [3.000000] = true,
                        [4.000000] = false,
                        [5.000000] = "Info"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 33.000000
                }
            },
            ["Shadowed Wound"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 6.000000,
                        [2.000000] = "Shadowed Wound on: ",
                        [3.000000] = true,
                        [4.000000] = false,
                        [5.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 36.000000
                }
            },
            ["Gushing Wound"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 5.000000,
                        [2.000000] = "Tactical dot tier up!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzzer_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 30.000000
                }
            },
            ["Insignificance"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 50.000000,
                        [2.000000] = "Reduced Threat",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Ultimate Inferno"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Move out of fire!!",
                        [3.000000] = true,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 24.000000
                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Flight"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 45.000000,
                        [2.000000] = "PURPLE EYE!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Eye.jpg",
                        [4.000000] = true,
                        [5.000000] = "runaway"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Deepening Gloom"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 2.000000,
                        [2.000000] = "Puddle!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Puddle.jpg",
                        [4.000000] = false,
                        [5.000000] = "alarmdouble_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 17.000000
                }
            },
            ["Losing Threat"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "Reduced Threat Tier Up!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Blind Hatred"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 4.000000,
                        [2.000000] = "Green eye!",
                        [3.000000] = "PrimePlugins/RaidTools/Images/Eye.jpg",
                        [4.000000] = false,
                        [5.000000] = "alarmbuzz"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 15.000000
                }
            },
            ["Fixate"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 26.000000,
                        [2.000000] = "Kite Fell-beast!",
                        [3.000000] = false,
                        [4.000000] = false
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 18.000000
                }
            },
            ["Terrified"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Remove fear!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzz_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 25.000000
                }
            },
            ["Running Scared"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 10.000000,
                        [2.000000] = "Feared",
                        [3.000000] = false,
                        [4.000000] = 10.000000
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {

                }
            },
            ["Fatal Affliction"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 3.000000,
                        [2.000000] = "Cure poison!",
                        [3.000000] = false,
                        [4.000000] = false,
                        [5.000000] = "alarmbuzz_quiet"
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 3.000000
                }
            },
            ["Black Powder Burns"] = 
            {
                ["Alerts"] = 
                {
                    [1.000000] = 
                    {
                        [1.000000] = 8.000000,
                        [2.000000] = "Use Water Pouch!",
                        [3.000000] = false,
                        [4.000000] = 8.000000
                    }
                },
                ["Timers"] = 
                {
                    [1.000000] = 
                    {

                    }
                },
                ["Restrictions"] = 
                {
                    [1.000000] = 24.000000
                }
            }
        }
        --]]
end

function EventWindowsHandler:GetBaseGroupEffectData()
    -- [1] Duration [2] Message [3] Icon (Special/Game/None) [4] Countdown [5] SoundEffect [6] TargetBox popup [7] TargetBox Icon
    -- [1] Duration [2] Message [3] Icon (Special/Game/None) [4] Countdown [5] SoundEffect [6] instanceID [7] role [8] timerduration, [9] timer text, [10] timer icon, [11] unique, [12] loop, [13] tiering effect}
    --[[
    local GroupEffects = {
        ["Crushing Roots"] = {4, "Root dot on: ", true, false, SoundEffects["Beeps"], nil, nil}, -- Thuringwaith tree dot
        ["Flight"] = {45, "Purple eye on: ", PurpleEyeSmall, true, nil, false, nil}, -- Anvil eye
        ["Vethúg's Revenge"] = {15, "Dot on: ", true, true, nil, nil, nil}, -- Anvil dot
        ["Clinging Frost"] = {4, "Frost debuff on: ", nil, true, SoundEffects["Doubleq"], nil, nil}, -- Anvil debuff
        ["Frozen Solid"] = {4, "Frozen: ", nil, true, nil, nil, nil}, -- Anvil frozen
        ["Scorching Rage"] = {15, "Dot on: ", PuddleIconSmall, true, SoundEffects["Doubleq"], nil, nil}, -- Black lore dot
        ["Blood-burst"] = {9, "Aura on: ", true, true, SoundEffects["Doubleq"], nil, nil}, -- Thuringwaith aura
        ["Touch of Sorcery"] = {11, "Aoe on: ", PurpleEyeSmall, true, SoundEffects["Doubleq"], nil, nil}, --Throne aoe eye
        ["Shadow of Doubt"] = {9, "Puddle on: ", PuddleIconSmall, true, nil, nil, nil}, --Thuringwraith puddle fear

        ["Non-Threatening"] = {5, "Threat wipe on: ", AlertIconSmall, true, SoundEffects["Doubleq"], nil, nil},
        ["Hideous Venom"] = {3, "DoT on: ", AlertIconSmall, false, nil, nil, nil},
        ["Creeping Cold"] = {3, "DoT on: ", AlertIconSmall, false, nil, nil, nil},
        ["Spiteful Words"] = {3, "DoT on: ", AlertIconSmall, false, SoundEffects["Alarmq"], nil, nil},
        ["Blind Hatred"] = {4, "Green eye on: ", PurpleEyeSmall, false, SoundEffects["Alarmq"], nil, nil},
        ["Arcing Lightning"] = {15, "Lightning on: ", true, true, nil, nil, nil},
        ["Sticky Venom"] = {30, "Venom on: ", AlertIconSmall, true, nil, nil, nil},
        ["Fiery Combustion"] = {15, "Fiery Combustion on: ", true, true, nil, nil, nil},
        ["Fixated"] = {45, "Fixated on: ", AlertIconSmall, true, nil, nil, nil},
        ["Life Drain"] = {8, "Life Drain on: ", BloodBoilIcon, true, nil, nil, nil, 40, "~Life drain", true, false},
        ["Shadow-spun Webs"] = {8, "Binding webs on: ", WebIcon, false, nil, 1, nil, 18, "Binding", true, false},
        ["Hail of Webs"] = {nil, nil, nil, nil, nil, 1, nil, 40, "Hail", true, true},
        ["Acidic Fangs - 1"] = {nil, nil, nil, nil, nil, 1, nil, 20, "Acid Frontal", AcidFangIcon, true},
        ["Acidic Fangs - 2"] = {nil, nil, nil, nil, nil, 1, nil, 20, "Acid Frontal", AcidFangIcon, true},
        ["Acidic Fangs - 3"] = {nil, nil, nil, nil, nil, 1, nil, 20, "Acid Frontal", AcidFangIcon, true},
        ["Acidic Fangs - 4"] = {nil, nil, nil, nil, nil, 1, nil, 20, "Acid Frontal", AcidFangIcon, true},
        ["Acidic Fangs - 5"] = {nil, nil, nil, nil, nil, 1, nil, 20, "Acid Frontal", AcidFangIcon, true},
        ["Acidic Fangs - 6"] = {nil, nil, nil, nil, nil, 1, nil, 20, "Acid Frontal", AcidFangIcon, true},
        ["Acidic Fangs - 7"] = {nil, nil, nil, nil, nil, 1, nil, 20, "Acid Frontal", AcidFangIcon, true},
        ["Acidic Fangs - 8"] = {nil, nil, nil, nil, nil, 1, nil, 20, "Acid Frontal", AcidFangIcon, true},


        ["Smouldering Wrath"] = {15, "Wrath on: ", true, true, nil, nil, nil},
        ["Dripping Ichor"] = {15, "Puddle on: ", true, true, nil, nil, nil},
        ["Parasites"] = {15, "Parasites on: ", true, true, nil, nil, nil},
        --["Mending Verse"] = {15, "Test", AlertIconSmall, true, nil},
        --["Daughter's Phermones"] = {65, "Daughter's Phermones on: ", AlertIconSmall, true, nil},
        --["Thossulun's Phermones"] = {65, "Daughter's Phermones on: ", AlertIconSmall, true, nil}
        --["Water-lore"] = {4, "Water-lore on: ", true, true, nil},

        -- Great Barrows
        ["Lurking Sickness"] = {15, "Cure disease on: ", true, 15, nil, nil, nil},

        -- Garth Agarwen (spring festival)
        ["Grievous Wound"] = {3, "Cure Wound on: ", true, false, nil, nil, nil},

        -- Narrow Delve
        ["Battered and Bruised"] = {3, "Cure wound on: ", true, 3, SoundEffects["Alarmq"], 32, nil},
        ["Vicious Bite"] = {3, "Cure wound on: ", true, 3, nil, 32, nil},
        ["Fear of the Witch-King"] = {3, "Cure fear on: ", true, 3, nil, 32, nil},
        ["Vengeance of the Fallen"] = {3, "Cure fear on: ", true, 3, nil, 32, nil},
        ["Spirit Wrack"] = {3, "Cure fear on: ", true, 3, nil, 32, nil},
        ["Howling Winds"] = {3, "Cure fear on: ", true, 3, nil, 32, nil},
        ["Freezing!"] = {5, "Frozen! Drop fire on: ", AlertIconSmall, false, SoundEffects["Alarmq"], 32, nil},

        -- Fornost
        ["Serious Wound"] = {3, "Cure wound on: ", true, false, nil, nil, nil},

        -- Barad Guluran
        ["Distraught"] = {3, "Cure Fear on: ", true, false, nil, nil, nil},
        ["Affliction through Shadow"] = {10, "Cure Fear ASAP on: ", true, 10, nil, nil, nil},
        ["Affliction through Flame"] = {10, "Cure Wound ASAP on: ", true, 10, nil, nil, nil},

        -- Urugarth
        ["Chilled"] = {3, "Cure Fear on: ", true, false, nil, nil, nil},

        -- Carn Dum
        ["Impaled"] = {3, "Impale(root) on: ", true, false, nil, nil, nil},
        
        

        -- Helegrod
        ["Fatal Poison"] = {15, "Cure poison ASAP on: ", true, 15, nil, nil, nil},
        ["Baleful Eye"] = {15, "Cure fear ASAP on: ", true, 15, nil, nil, nil},

        -- Nala Dum
        ["Deadly Acid"] = {3, "Cure Poison on!: ", true, false, nil, nil, nil}, 

        -- Roots of Fangorn


        --PvMP
        ["Field Promotion"] = {4, "Bubble on: ", true, false, nil, 6, nil},
        ["Grievous Wounds"] = {5, "Grave wound on: ", true, true, nil, 6, nil},
        ["Oathbreaker's Shame"] = {20, "OB on: ", true, true, nil, 6, nil},
        ["Heart Seeker"] = {8, "HS on: ", true, true, nil, 6, nil},
        ["Essence of Winter"] = {5, "Essence of Winter on: ", true, false, nil, 6, nil},
        --["Telling Mark"] = {3, "Telling Mark on: ", nil, false, nil},
        --["Blessing of Darkness"] = {4, "Blessing on: ", nil, false, nil},
        ["Ancient Craft"] = {5, "AC on: ", true, false, nil, 6, nil},
        ["Shadow Vortex"] = {10, "Vortex on: ", true, true, nil, 6, nil},
        ["Impending End"] = {20, "Free: ", true, true, nil, 6, nil},

        -- AD
        ["Rowdy Crowd"] = {12, "Volleys on: ", true, true, nil, 7, nil},
        ["Massive Javelin"] = {7, "Javelin on: ", true, true, nil, 7, nil},
        ["Gorgar's Target"] = {4, "Charge on: ", true, true, nil, 7, nil},

        ["Frost Trap"] = {6, "Frost trap on: ", true, false, nil, nil, nil},

        -- FOKD
        ["Combust"] = {6, "Combust on: ", true, false, nil, 38, nil},
        --["Bloody Wound"] = {60, "Bloody Wound on: ", true, true, SoundEffects["tauntBoss"], false, nil}
        -- [1] Duration [2] Message [3] Icon (Special/Game/None) [4] Countdown [5] SoundEffect [6] instanceID [7] role [8] timerduration, [9] timer text, [10] timer icon, [11] unique, [12] loop, [13] windowID}
        -- AD
        ["Infected Bite"] = {6, "Disease on: ", true, false, false, 7, nil},
        ["Shadowed Wound"] = {6, "Shadowed Wound on: ", true, false, false, 7, nil},
        ["Bloody Wound"] = {4, "Bloody Wound on", true, false, false, 7, nil, 60, "<name>", true, false},

        -- DN
        ["Virulent Contagion"] = {4, "Move away from: ", true, false, nil, 2, nil},
        ["Paralyzing Contagion"] = {4, "Move away from: ", true, false, nil, 2, nil},
        ["Impending Mark: Flâgît's Ire"] = {6, "Eye on: ", true, true, nil, 2, nil},
        ["Mark: Flâgît's Ire"] = {10, "Mark on: ", true, true, nil, 2, nil},
        ["Deadly Poison"] = {3, "Poison on: ", true, false, nil, 2, nil},
        ["Major Disease"] = {3, "Disease on: ", true, false, nil, 2, nil},
        ["Major Wound"] = {3, "Wound on: ", true, false, nil, 2, nil},
        ["Mortal Terror"] = {3, "Fear on: ", true, false, nil, 2, nil},

        -- Rift
        ["Sapping Disease"] = {3, "Cure disease on: ", true, false, nil, 3, nil},

        -- Fallen kings
        ["Encroaching Darkness"] = {4, "Puddle on: ", PuddleIconSmall, false, nil, 17, nil}, -- Black blade
        ["Spreading Fear"] = {7, "Aoe on: ", PurpleEyeSmall, true, nil, 17, nil},
        ["Gloom"] = {10, "Puddle on: ", PurpleEyeSmall, true, SoundEffects["Doulbeq"], 17, nil},

        -- Barad Guldur
        ["Frail"] = {10, "Armour Wipe on: ", true, 10, nil, 18, nil},
        ["Traumatic Wound"] = {15, "Reduced Max HP on: ", true, 15, nil, 18, nil},
        ["Insignificance"] = {5, "Threat Reduction on: ", true, 5, nil, 18, nil},
        ["Bottled Shadow"] = {10, "Shadow Puddle on: ", true, 10, nil, 18, nil},
        ["Canned Heat"] = {10, "Fire Puddle on: ", true, 10, nil, 18, nil},
        ["Call of the Shadows"] = {9, "AOE DOT on: ", true, 9, nil, 18, nil},
        ["Combustion"] = {10, "BIG AOE on: ", true, 10, nil, 18, nil},
        ["Total Threat Loss"] = {3, "Threat Wipe on: ", true, false, nil, 18, nil},
        ["Fixate"] = {26, "Fell-beast On: ", true, 26, nil, 18, nil},
        ["Seed of Dissent"] = {15, "Purple Eye on: ", true, 15, nil, 18, nil},

        -- Sammath Gul
        ["Harrassing Spirit"] = {15, "Major Debuff on: ", true, 15, nil, 19, nil},
        ["Slow Burn"] = {10, "Move Away From: ", true, 10, nil, 19, nil},

        -- Fil Gashan
        ["Sickening Poison"] = {3, "Cure Poison on!: ", true, false, nil, 22, nil},

        -- Skumfil
        ["Marked for Imprisonment"] = {5, "Kill Daze Add On!: ", AlertIconSmall, true, nil, 23, nil},
        ["Deadly Poison"] = {3, "Cure Poison on!: ", true, false, nil, 23, nil},    

        -- Grand Stairs
        ["Poison Dagger"] = {3, "Cure Poison on!: ", true, false, nil, 25, nil}, 

        -- Garth Agarwen 27
        ["Deep Wound"] = {3, "Cure wound on: ", true, false, nil, 27, nil},

        -- Garth Agarwen 28
        ["Deep Wound"] = {3, "Cure wound on: ", true, false, nil, 28, nil},

        -- Garth Agarwen 29
        ["Deep Wound"] = {3, "Cure wound on: ", true, false, nil, 29, nil},

        -- Filkul
        ["Profuse Bleeding"] = {5, "DoT tier up on: ", true, false, nil, 30, nil},

        -- 36
        ["Lightning Surge"] = {3, "Cure Wound on: ", true, false, nil, 36, nil},
        ["Deadly Immolation"] = {3, "Cure Wound on: ", true, false, nil, 36, nil},
        ["Deadly Frost"] = {3, "Cure Wound on: ", true, false, nil, 36, nil},
        ["Lethal Acid"] = {3, "Cure Poison on: ", true, false, nil, 36, nil},
        ["Deadly Poison"] = {3, "Cure Poison on: ", true, false, nil, 36, nil},
        ["Mortal Terror"] = {3, "Cure Fear on: ", true, false, nil, 36, nil},
        ["Major Disease"] = {3, "Cure Disease on: ", true, false, nil, 36, nil},
        ["Major Power Drain"] = {3, "Cure Fear on: ", true, false, nil, 36, nil},

        --38
        ["Mark of Shadow"] = {30, "Mark on: ", true, true, nil, 38, nil, 45, "Shadow Eye", true, true},
        ["Liquid Gold - "] = {nil, nil, nil, nil, nil, 38, nil, 12, "<name>", true, nil, false, true},
        ["Building Flames - "] = {nil, nil, nil, nil, nil, 38, nil, 60, "<name>", true, nil, false, true},
        ["Lockdown"] = {7, "Lockdown on: ", true, false, nil, 38, nil, 35, "Lockdown", true, false},


        
        --["Mending Verse"] = {30, "Mark on: ", true, true, nil, nil, nil, 45, "Shadow Eye", true, true},
        --["Writ of Health - Tier "] = {nil, nil, nil, nil, nil, nil, nil, 12, "<name>", true, nil, false, true},
        --["Writ of Health - Tier "] = {30, "Mark on: ", true, true, nil, nil, 2, 45, "<name>", true, true, false, true},
        --["Writ of Health - Tier "] = {6, "Shadowed Wound on: ", true, true, false, nil, nil, 15, "<name>", true, true, false, true},
    }
    -- [1] Duration [2] Message [3] Icon (Special/Game/None) [4] Countdown [5] SoundEffect [6] instanceID [7] role [8] timerduration, [9] timer text, [10] timer icon, [11] unique, [12] loop, [13] tiering effect}
    --]]
    return 
    {
        ["Vethúg's Revenge"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Dot on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Blood-burst"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 9.000000,
                    [2.000000] = "Aura on: ",
                    [3.000000] = true,
                    [4.000000] = true,
                    [5.000000] = "alarmdouble_quiet"
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Encroaching Darkness"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 4.000000,
                    [2.000000] = "Puddle on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/PuddleSmall.jpg",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 17.000000
            }
        },
        ["Combust"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 6.000000,
                    [2.000000] = "Combust on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 38.000000
            }
        },
        ["Major Disease"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Disease on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 36.000000
            }
        },
        ["Impaled"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Impale(root) on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Mortal Terror"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Fear on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 36.000000
            }
        },
        ["Field Promotion"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 4.000000,
                    [2.000000] = "Bubble on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 6.000000
            }
        },
        ["Lurking Sickness"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Cure disease on: ",
                    [3.000000] = true,
                    [4.000000] = 15.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Acidic Fangs - 2"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 20.000000,
                    [2.000000] = "Acid Frontal",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AcidFangIcon.tga",
                    [4.000000] = true
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 1.000000
            }
        },
        ["Impending End"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 20.000000,
                    [2.000000] = "Free: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 6.000000
            }
        },
        ["Frost Trap"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 6.000000,
                    [2.000000] = "Frost trap on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Mark of Shadow"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 30.000000,
                    [2.000000] = "Mark on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 45.000000,
                    [2.000000] = "Shadow Eye",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 38.000000
            }
        },
        ["Sticky Venom"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 30.000000,
                    [2.000000] = "Venom on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Poison Dagger"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Poison on!: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 25.000000
            }
        },
        ["Seed of Dissent"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Purple Eye on: ",
                    [3.000000] = true,
                    [4.000000] = 15.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 18.000000
            }
        },
        ["Howling Winds"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure fear on: ",
                    [3.000000] = true,
                    [4.000000] = 3.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 32.000000
            }
        },
        ["Frail"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 10.000000,
                    [2.000000] = "Armour Wipe on: ",
                    [3.000000] = true,
                    [4.000000] = 10.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 18.000000
            }
        },
        ["Parasites"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Parasites on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Sickening Poison"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Poison on!: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 22.000000
            }
        },
        ["Lethal Acid"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Poison on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 36.000000
            }
        },
        ["Spiteful Words"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "DoT on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                    [4.000000] = false,
                    [5.000000] = "alarmbuzz_quiet"
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Insignificance"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 5.000000,
                    [2.000000] = "Threat Reduction on: ",
                    [3.000000] = true,
                    [4.000000] = 5.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 18.000000
            }
        },
        ["Building Flames - "] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 60.000000,
                    [2.000000] = "<name>",
                    [3.000000] = true,
                    [5.000000] = false
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 38.000000,
                [3.000000] = true
            }
        },
        ["Arcing Lightning"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Lightning on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Acidic Fangs - 6"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 20.000000,
                    [2.000000] = "Acid Frontal",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AcidFangIcon.tga",
                    [4.000000] = true
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 1.000000
            }
        },
        ["Heart Seeker"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 8.000000,
                    [2.000000] = "HS on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 6.000000
            }
        },
        ["Fear of the Witch-King"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure fear on: ",
                    [3.000000] = true,
                    [4.000000] = 3.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 32.000000
            }
        },
        ["Battered and Bruised"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure wound on: ",
                    [3.000000] = true,
                    [4.000000] = 3.000000,
                    [5.000000] = "alarmbuzz_quiet"
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 32.000000
            }
        },
        ["Spirit Wrack"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure fear on: ",
                    [3.000000] = true,
                    [4.000000] = 3.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 32.000000
            }
        },
        ["Creeping Cold"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "DoT on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Major Power Drain"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Fear on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 36.000000
            }
        },
        ["Chilled"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Fear on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Major Wound"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Wound on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 2.000000
            }
        },
        ["Profuse Bleeding"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 5.000000,
                    [2.000000] = "DoT tier up on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 30.000000
            }
        },
        ["Liquid Gold - "] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 12.000000,
                    [2.000000] = "<name>",
                    [3.000000] = true,
                    [5.000000] = false
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 38.000000,
                [3.000000] = true
            }
        },
        ["Slow Burn"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 10.000000,
                    [2.000000] = "Move Away From: ",
                    [3.000000] = true,
                    [4.000000] = 10.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 19.000000
            }
        },
        ["Acidic Fangs - 5"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 20.000000,
                    [2.000000] = "Acid Frontal",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AcidFangIcon.tga",
                    [4.000000] = true
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 1.000000
            }
        },
        ["Crushing Roots"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 4.000000,
                    [2.000000] = "Root dot on: ",
                    [3.000000] = true,
                    [4.000000] = false,
                    [5.000000] = "alarmbeep_soft"
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Mark: Flâgît's Ire"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 10.000000,
                    [2.000000] = "Mark on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 2.000000
            }
        },
        ["Ancient Craft"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 5.000000,
                    [2.000000] = "AC on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 6.000000
            }
        },
        ["Serious Wound"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure wound on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Acidic Fangs - 8"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 20.000000,
                    [2.000000] = "Acid Frontal",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AcidFangIcon.tga",
                    [4.000000] = true
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 1.000000
            }
        },
        ["Freezing!"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 5.000000,
                    [2.000000] = "Frozen! Drop fire on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                    [4.000000] = false,
                    [5.000000] = "alarmbuzz_quiet"
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 32.000000
            }
        },
        ["Infected Bite"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 6.000000,
                    [2.000000] = "Disease on: ",
                    [3.000000] = true,
                    [4.000000] = false,
                    [5.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 7.000000
            }
        },
        ["Flight"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 45.000000,
                    [2.000000] = "Purple eye on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/EyeSmall.jpg",
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = false
            }
        },
        ["Life Drain"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 8.000000,
                    [2.000000] = "Life Drain on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/Bloodboil.tga",
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 40.000000,
                    [2.000000] = "~Life drain",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Lockdown"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 7.000000,
                    [2.000000] = "Lockdown on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 35.000000,
                    [2.000000] = "Lockdown",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 38.000000
            }
        },
        ["Oathbreaker's Shame"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 20.000000,
                    [2.000000] = "OB on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 6.000000
            }
        },
        ["Deadly Frost"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Wound on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 36.000000
            }
        },
        ["Grievous Wound"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Wound on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Grievous Wounds"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 5.000000,
                    [2.000000] = "Grave wound on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 6.000000
            }
        },
        ["Shadowed Wound"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 6.000000,
                    [2.000000] = "Shadowed Wound on: ",
                    [3.000000] = true,
                    [4.000000] = false,
                    [5.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 7.000000
            }
        },
        ["Acidic Fangs - 1"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 20.000000,
                    [2.000000] = "Acid Frontal",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AcidFangIcon.tga",
                    [4.000000] = true
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 1.000000
            }
        },
        ["Deadly Immolation"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Wound on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 36.000000
            }
        },
        ["Spreading Fear"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 7.000000,
                    [2.000000] = "Aoe on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/EyeSmall.jpg",
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 17.000000
            }
        },
        ["Virulent Contagion"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 4.000000,
                    [2.000000] = "Move away from: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 2.000000
            }
        },
        ["Gloom"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 10.000000,
                    [2.000000] = "Puddle on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/EyeSmall.jpg",
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 17.000000
            }
        },
        ["Total Threat Loss"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Threat Wipe on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 18.000000
            }
        },
        ["Deadly Poison"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Poison on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 36.000000
            }
        },
        ["Shadow of Doubt"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 9.000000,
                    [2.000000] = "Puddle on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/PuddleSmall.jpg",
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Combustion"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 10.000000,
                    [2.000000] = "BIG AOE on: ",
                    [3.000000] = true,
                    [4.000000] = 10.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 18.000000
            }
        },
        ["Vicious Bite"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure wound on: ",
                    [3.000000] = true,
                    [4.000000] = 3.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 32.000000
            }
        },
        ["Marked for Imprisonment"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 5.000000,
                    [2.000000] = "Kill Daze Add On!: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 23.000000
            }
        },
        ["Frozen Solid"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 4.000000,
                    [2.000000] = "Frozen: ",
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Acidic Fangs - 3"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 20.000000,
                    [2.000000] = "Acid Frontal",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AcidFangIcon.tga",
                    [4.000000] = true
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 1.000000
            }
        },
        ["Essence of Winter"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 5.000000,
                    [2.000000] = "Essence of Winter on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 6.000000
            }
        },
        ["Deadly Acid"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Poison on!: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Call of the Shadows"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 9.000000,
                    [2.000000] = "AOE DOT on: ",
                    [3.000000] = true,
                    [4.000000] = 9.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 18.000000
            }
        },
        ["Paralyzing Contagion"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 4.000000,
                    [2.000000] = "Move away from: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 2.000000
            }
        },
        ["Harrassing Spirit"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Major Debuff on: ",
                    [3.000000] = true,
                    [4.000000] = 15.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 19.000000
            }
        },
        ["Lightning Surge"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Wound on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 36.000000
            }
        },
        ["Vengeance of the Fallen"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure fear on: ",
                    [3.000000] = true,
                    [4.000000] = 3.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 32.000000
            }
        },
        ["Dripping Ichor"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Puddle on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Smouldering Wrath"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Wrath on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Affliction through Shadow"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 10.000000,
                    [2.000000] = "Cure Fear ASAP on: ",
                    [3.000000] = true,
                    [4.000000] = 10.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Fiery Combustion"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Fiery Combustion on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Acidic Fangs - 4"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 20.000000,
                    [2.000000] = "Acid Frontal",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AcidFangIcon.tga",
                    [4.000000] = true
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 1.000000
            }
        },
        ["Sapping Disease"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure disease on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 3.000000
            }
        },
        ["Bloody Wound"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 4.000000,
                    [2.000000] = "Bloody Wound on",
                    [3.000000] = true,
                    [4.000000] = false,
                    [5.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 60.000000,
                    [2.000000] = "<name>",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 7.000000
            }
        },
        ["Gorgar's Target"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 4.000000,
                    [2.000000] = "Charge on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 7.000000
            }
        },
        ["Clinging Frost"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 4.000000,
                    [2.000000] = "Frost debuff on: ",
                    [4.000000] = true,
                    [5.000000] = "alarmdouble_quiet"
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Massive Javelin"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 7.000000,
                    [2.000000] = "Javelin on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 7.000000
            }
        },
        ["Rowdy Crowd"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 12.000000,
                    [2.000000] = "Volleys on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 7.000000
            }
        },
        ["Non-Threatening"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 5.000000,
                    [2.000000] = "Threat wipe on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                    [4.000000] = true,
                    [5.000000] = "alarmdouble_quiet"
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Bottled Shadow"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 10.000000,
                    [2.000000] = "Shadow Puddle on: ",
                    [3.000000] = true,
                    [4.000000] = 10.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 18.000000
            }
        },
        ["Scorching Rage"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Dot on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/PuddleSmall.jpg",
                    [4.000000] = true,
                    [5.000000] = "alarmdouble_quiet"
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Shadow-spun Webs"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 8.000000,
                    [2.000000] = "Binding webs on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/Webs.tga",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 18.000000,
                    [2.000000] = "Binding",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 1.000000
            }
        },
        ["Shadow Vortex"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 10.000000,
                    [2.000000] = "Vortex on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 6.000000
            }
        },
        ["Impending Mark: Flâgît's Ire"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 6.000000,
                    [2.000000] = "Eye on: ",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 2.000000
            }
        },
        ["Traumatic Wound"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Reduced Max HP on: ",
                    [3.000000] = true,
                    [4.000000] = 15.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 18.000000
            }
        },
        ["Fixated"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 45.000000,
                    [2.000000] = "Fixated on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Touch of Sorcery"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 11.000000,
                    [2.000000] = "Aoe on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/EyeSmall.jpg",
                    [4.000000] = true,
                    [5.000000] = "alarmdouble_quiet"
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Acidic Fangs - 7"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 20.000000,
                    [2.000000] = "Acid Frontal",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AcidFangIcon.tga",
                    [4.000000] = true
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 1.000000
            }
        },
        ["Deep Wound"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure wound on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 29.000000
            }
        },
        ["Hideous Venom"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "DoT on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AlertRoundSmall.tga",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Distraught"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Cure Fear on: ",
                    [3.000000] = true,
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Fatal Poison"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Cure poison ASAP on: ",
                    [3.000000] = true,
                    [4.000000] = 15.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Blind Hatred"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 4.000000,
                    [2.000000] = "Green eye on: ",
                    [3.000000] = "PrimePlugins/RaidTools/Images/EyeSmall.jpg",
                    [4.000000] = false,
                    [5.000000] = "alarmbuzz_quiet"
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Fixate"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 26.000000,
                    [2.000000] = "Fell-beast On: ",
                    [3.000000] = true,
                    [4.000000] = 26.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 18.000000
            }
        },
        ["Canned Heat"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 10.000000,
                    [2.000000] = "Fire Puddle on: ",
                    [3.000000] = true,
                    [4.000000] = 10.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 18.000000
            }
        },
        ["Affliction through Flame"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 10.000000,
                    [2.000000] = "Cure Wound ASAP on: ",
                    [3.000000] = true,
                    [4.000000] = 10.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Baleful Eye"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Cure fear ASAP on: ",
                    [3.000000] = true,
                    [4.000000] = 15.000000
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Restrictions"] = 
            {
    
            }
        },
        ["Hail of Webs"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
    
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 40.000000,
                    [2.000000] = "Hail",
                    [3.000000] = true,
                    [4.000000] = true
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 1.000000
            }
        }
    }
end

function EventWindowsHandler:GetBaseChatPlayerCombatData()
     -- Chat alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 windowID}
     return { 
    }
end

function EventWindowsHandler:GetBaseChatEnemyCombatData()
    --[[
    return {
        ["Shadow Force"] = {nil, nil, nil, nil, nil, 36, nil, 45, "Shadow Force", ShadowFace},
        ["Rage and Fury"] = {nil, nil, nil, nil, nil, nil, nil, 25, "Rage and Fury", RageIcon},
        ["Vomit Darkness"] = {nil, nil, nil, nil, nil, 1, nil, 15, "Darkness", DarknessIcon},
        ["Acidic Fangs"] = {nil, nil, nil, nil, nil, 1, nil, 20, "Acid Frontal", AcidFangIcon},
        ["Apocalypse Cleave (Distributed)"] = {nil, nil, nil, nil, nil, 38, nil, 20, "Apocalypse Cleave", 1090648289},
        ["Thrash"] = {nil, nil, nil, nil, nil, 7, nil, 50, "Thrash", ThrashIcon},
        ["Flame Thrash"] = {nil, nil, nil, nil, nil, 7, nil, 50, "Flame Thrash", FireIcon},

            --["Gaze of Sauron"] = {4, "Purple eye!", PurpleEyeSmall, true}
        ["Shadow Force"] = {45, "AoE CD", nil, true},
        ["Bursting Flame"] = {3, "Move in melee range!", nil, false},
        ["Baleful Roar"] = {3, "Check for hope", nil, false},
        ["Bone Grind"] = {3, "Move out of Bone Grind", nil, false},
        ["Cave In"] = {3, "Move out of Cave In", nil, false},
        ["Poisoned Ground"] = {2, "Move out!", nil, false},
        ["Killing Field"] = {2, "Move out!", nil, false},
        ["Disintegration"] = {2, "Move out!", nil, false},
        ["Whirlwind"] = {2, "Move out!", nil, false},
        ["Withering Strike"] = {3, "Don't forget to interrupt!", nil, false},
        ["Stinger Thrust"] = {3, "Stinger Thrust!", nil, false},
        ["Ultimate Scorched Earth"] = {3, "Interrupt knockback!", nil, false}, 
        ["Shock Wave"] = {3, "Avoid floor effects!", nil, false},
        ["Desecrated Ground"] = {2, "Move out!", nil, false},
        --["Bleeding Ears"] = {30, "REFLECT!! REFLECT!! REFLECT!!", DarknessIcon, true},
        ["Flame Jet"] = {3, "Avoid floor flames!!", nil, false}, 
    }
    --]]

    return 
    {
        ["Apocalypse Cleave (Distributed)"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 20.000000,
                    [2.000000] = "Apocalypse Cleave",
                    [3.000000] = 1090648289.000000
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 38.000000
            }
        },
        ["Ultimate Scorched Earth"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Interrupt knockback!",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Cave In"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Move out of Cave In",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Vomit Darkness"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 15.000000,
                    [2.000000] = "Darkness",
                    [3.000000] = "PrimePlugins/RaidTools/Images/DarknessIcon.tga"
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 1.000000
            }
        },
        ["Shadow Force"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 45.000000,
                    [2.000000] = "AoE CD",
                    [4.000000] = true
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Baleful Roar"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Check for hope",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Flame Thrash"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 50.000000,
                    [2.000000] = "Flame Thrash",
                    [3.000000] = "PrimePlugins/RaidTools/Images/FireIcon.tga"
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 7.000000
            }
        },
        ["Bursting Flame"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Move in melee range!",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Shock Wave"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Avoid floor effects!",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Bone Grind"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Move out of Bone Grind",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Rage and Fury"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 25.000000,
                    [2.000000] = "Rage and Fury",
                    [3.000000] = "PrimePlugins/RaidTools/Images/RageIcon.tga"
                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Disintegration"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 2.000000,
                    [2.000000] = "Move out!",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Whirlwind"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 2.000000,
                    [2.000000] = "Move out!",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Flame Jet"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Avoid floor flames!!",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Killing Field"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 2.000000,
                    [2.000000] = "Move out!",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Acidic Fangs"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 20.000000,
                    [2.000000] = "Acid Frontal",
                    [3.000000] = "PrimePlugins/RaidTools/Images/AcidFangIcon.tga"
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 1.000000
            }
        },
        ["Withering Strike"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Don't forget to interrupt!",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Thrash"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 50.000000,
                    [2.000000] = "Thrash",
                    [3.000000] = "PrimePlugins/RaidTools/Images/ThrashIcon.tga"
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 7.000000
            }
        },
        ["Desecrated Ground"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 2.000000,
                    [2.000000] = "Move out!",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Stinger Thrust"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 3.000000,
                    [2.000000] = "Stinger Thrust!",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Poisoned Ground"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 2.000000,
                    [2.000000] = "Move out!",
                    [4.000000] = false
                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Restrictions"] = 
            {

            }
        },
        ["Ground Smash"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [1.000000] = 45.000000,
                    [2.000000] = "Ground smash",
                    [3.000000] = 1091840283.000000
                }
            },
            ["Restrictions"] = 
            {
                [1.000000] = 38.000000
            }
        },
        ["Erupting Earth"] = 
        {
            ["Alerts"] = 
            {
                [1.000000] = 
                {

                }
            },
            ["Timers"] = 
            {
                [1.000000] = 
                {
                    [3.000000] = 1090541218,
                    [2.000000] = "Erupting ground",
                    [1.000000] = 25,
                    [5.000000] = false,
                    [4.000000] = false
                }
            },
            ["Restrictions"] = 
            {

            }
        },
    }
end
 